package DTDDoc;

import java.io.*;

import java.util.HashSet;
import java.util.Properties;

import org.apache.regexp.RE;
import org.apache.regexp.RESyntaxException;


/** This is the command-line version of DTDDoc. Please try to use the
 *  Ant task instead (it's easier, more powerful and not doomed to be
 *  deprecated :))
 * 
 * @deprecated The command-line version of DTDDoc is deprecated in favour
 * of an Ant Task version. Please try to migrate. */

public class DTDDoc {

	/** Interpret a property of a configuration into an array of RegExp.
	 *  The property must be a coma separated list of RegExp.
	 *  @param configuration Where to look for the property.
	 *  @param propertyName The name of the property.*
	 *  @return An array of RegExp. */
	 
    private RE[] propertyToREArray( Properties configuration, String propertyName)
        throws RESyntaxException {

        String s[] = Tools.listToArray( configuration.getProperty( propertyName), ",");

        RE l[] = new RE[s.length];
        for( int i=0; i<s.length; i++)
            l[i] = new RE( s[i]);

        return l;
    }

    /** Checks if a given string matches one out of several regular expressions.
     *
     *  @param s the string to match.
     *  @param regexps the regular expressions to look for.
     *  @return true if s matches one of the reg. exp. given in regexps. False
     *          in any other case. */

    private boolean satisfyOneRE( String s, RE[] regexps) {
    	if ( regexps == null )
    		return false;
    	for( int i=0; i<regexps.length; i++)
            if( regexps[i].match(s))
                return true;
        return false;
    }

    /** Find all the files located under a directory.
     *  Will take the acceptList and rejectList into account !
     *
     *  @param source the directory to look for. If it happens to be a file,
     *                 then the file will be the only one scanned.
     *  @return a set of all the found files. Each file being represented
     *          as a File. */

    private HashSet fileScanner( File source, RE[] acceptList, RE[] rejectList) throws IOException {

        HashSet files = new HashSet();

		// Basically, we recurse on the directory (tree) structure.
		
        if( source.isDirectory()) {

            File[] allFiles = source.listFiles();

            if( allFiles != null)

                for( int i=0; i<allFiles.length; i++)
                    files.addAll( fileScanner( allFiles[i], acceptList, rejectList));

        } else
		
		    // source is a simple file

            if( source.isFile() &&
                satisfyOneRE( source.getCanonicalPath(), acceptList) &&
                !satisfyOneRE( source.getCanonicalPath(), rejectList) ) {

                files.add( source);

            }

        return files;

    }

	/** Configuration for the session, loaded from a .properties file. */
    private Properties configuration = null;

    /** Get a configuration value.
     *
     *  @param key Configuration value to look for.
     *  @return the value. Returns null if the configuration value is not
     *          found.  */

    public String getConfig( String key) {
        if( configuration == null)
            return null;
        else
            return configuration.getProperty( key);
    }
	
	/** Test if a value of a property of the configuration is true.
	 *  @param key The name of the property to look for.
	 *  @return true if the property is set to "true", false in any other
	 *      case. */

    public boolean isConfigTrueFor( String key) {
        return "true".equals( getConfig( key));
    }

    /** Set up the configuration from a file. If we can't open the file, then
     *  we revert to "default.properties". If this still doesn't work, then
     *  we're into troubles.
     *
     *  @param source where to get the configuration from. */

    private void setConfiguration( File source)
        throws FileNotFoundException, IOException, RESyntaxException {

        if( !source.canRead()) {
            source = new File("default.properties");
            if( !source.canRead()) {
                return;
            }
        }

        configuration = new Properties();
        configuration.load( new FileInputStream( source));
	}
    
    public void setConfiguration( Properties prop ) {
    	configuration = prop;
    }

	
    public static void main( String[] args ) throws Exception {

    	Properties p = new Properties();
    	p.setProperty( "docTitle", "TEST" );
    	p.setProperty( "DTDsourceDir", "C:/travail/soft/jxmlpad/xml-samples/personal.dtd" );
    	p.setProperty( "destinationDir", "c:/personal.html" );


		DTDDoc doc = new DTDDoc();
		
/*		if( args != null && args[0] != null)
		    doc.setConfiguration( new File(args[0]));
		else
		    doc.setConfiguration( new File(""));*/
		
		doc.setConfiguration( p );

		String docTitle = doc.getConfig("docTitle");
		
		// Figure out the source directory (where to get the DTD from)
				
        File sourceDir = new File( doc.getConfig("DTDsourceDir"));
        
		// and the destination dir (where to put the DTD)
        
		File destDir = new File( doc.getConfig("destinationDir"));
		boolean showHiddenTags = doc.isConfigTrueFor("showHiddenTags");
		boolean showFixmeTags = doc.isConfigTrueFor("showFixmeTags");


		HashSet scan = doc.fileScanner( sourceDir,
					null, null );
/*		                   doc.propertyToREArray( doc.configuration, "acceptList"),
						   doc.propertyToREArray( doc.configuration, "rejectList")); */ 

        DTDCommenter commenter = new DTDCommenter();
		commenter.commentDTDs( scan, sourceDir, destDir, showHiddenTags, showFixmeTags, docTitle, null);
    }
}
