package com.japisoft.datasource.ftp;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemException;
import org.apache.commons.vfs.FileSystemManager;
import org.apache.commons.vfs.FileSystemOptions;
import org.apache.commons.vfs.FileType;
import org.apache.commons.vfs.VFS;
import org.apache.commons.vfs.auth.StaticUserAuthenticator;
import org.apache.commons.vfs.impl.DefaultFileSystemConfigBuilder;

import com.japisoft.datasource.DataSource;
import com.japisoft.datasource.DataSourceContainer;
import com.japisoft.datasource.DataSourceItem;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FTPDataSourceContainer extends FTPDataSource implements DataSourceContainer {

	public FTPDataSourceContainer( String host, String user, String password, String path ) {
		super( host, user, password, path );
	}

	public FTPDataSourceContainer( String uri ) {
		super( uri );
	}
	
	public FTPDataSourceContainer( FileObject fo ) {
		super( fo );
	}
	
	FileObject connect() throws Exception {
		if ( fo == null ) {		
			FileSystemOptions opts = null;
			if ( user != null ) {
				StaticUserAuthenticator auth = new StaticUserAuthenticator( null, user, password ); 
				opts = new FileSystemOptions(); 
				try {
					DefaultFileSystemConfigBuilder.getInstance().setUserAuthenticator( opts, auth );
				} catch (FileSystemException e) {
				}
			}
			FileSystemManager fsManager = VFS.getManager();
			fo = fsManager.resolveFile( "ftp://" + host + "/" + path, opts );
		}
		return fo;
	}

	public List<DataSource> list() throws Exception {
		connect();
		FileObject[] children = fo.getChildren();
		ArrayList<DataSource> res = new ArrayList<DataSource>();
		if ( children != null ) {
			for ( FileObject fo : children ) {
				if ( fo.getType() == FileType.FILE ) {
					res.add( new FTPDataSourceItem( fo ) );
				} else
				if ( fo.getType() == FileType.FOLDER ) {
					res.add( new FTPDataSourceContainer( fo ) );
				}
			}
		}
		return res;
	}

	public DataSourceItem createItem(String name) throws Exception {
		connect();
		FileObject fo2 = fo.resolveFile( name );
		fo2.createFile();
		return new FTPDataSourceItem( fo2 );
	}
	
	public DataSourceContainer createContainer(String name) throws Exception {
		connect();
		FileObject fo2 = fo.resolveFile( name );
		fo2.createFolder();
		return new FTPDataSourceContainer( fo2 );
	}

}
