package com.japisoft.dtdparser.node;

import java.io.*;

import com.japisoft.dtdparser.CannotFindElementException;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ElementRefDTDNode
	extends ElementDTDNode
	implements DocumentWritable {
	public ElementRefDTDNode() {
		super();
		setNodeType(ELEMENT_REF);
	}

	/** This name is the reference. You can find the real ELEMENT by the
	 * <code>RootDTDNode</code> */
	public ElementRefDTDNode(String name) {
		super(name);
		setNodeType(ELEMENT_REF);
	}

	/** This is the max of the occurence between this element ref
	 * and the element set */
	public int getOperator() {
//		ElementSetDTDNode set = (ElementSetDTDNode) getDTDParentNode();
//		return Math.max(set.getOperator(), super.getOperator());
		return super.getOperator();
	}

	public String toString() {
		return "[ELEMENTREF] " + getName() + getOperatorString();
	}

	/** @return the reference node: null value means an error in the DTD */
	public ElementDTDNode getReferenceNode() {
		if (getName() == null)
			return null;
		try {
			return getRoot().getElementDefinitionByName(getName());
		} catch( CannotFindElementException e ) {
			return null;
		}
	}

	public void writeDocument(PrintWriter output) throws IOException {
		ElementDTDNode ref = getReferenceNode();
		if (ref != null)
			ref.writeDocument(output);
	}

}

// ElementRefDTDNode ends here
