package com.japisoft.dtdparser.node;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EntityDTDNode extends DTDNode {

	public static final int INTERNAL_ENTITY = 0;

	public static final int SYSTEM_ENTITY = 1;

	public static final int PUBLIC_ENTITY = 2;

	public static final String IGNORE = "IGNORE";

	public static final String INCLUDE = "INCLUDE";

	public EntityDTDNode() {
		super();
		setNodeType(ENTITY);
	}

	public EntityDTDNode(String name, String value) {
		this();
		setName(name);
		setValue(value);
	}

	private boolean parameter;

	/** Set this entity as parameter : example &lt;!ENTITY A % "B"&gt; */
	public void setParameter(boolean parameter) {
		this.parameter = parameter;
	}

	/** @return true if this entity is a parameter for other DTD part */
	public boolean isParameter() {
		return parameter;
	}

	/** Set the entity nature : INTERNAL, PUBLIC or SYSTEM */
	public void setType(int type) {
		this.type = type;
	}

	/** @return the entity nature */
	public int getType() {
		return type;
	}

	private int type = INTERNAL_ENTITY;

	private String name;

	private String value;

	/** Set the name of the entity */
	public void setName(String name) {
		this.name = name;
	}

	/** @return the name of the entity */
	public String getName() {
		return name;
	}

	/** Set the value of the entity */
	public void setValue(String value) {
		this.value = value;
	}

	/** @return the value of the entity */
	public String getValue() {
		return value;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer( getDTDComment() ).append( "<!ENTITY ").append(getName());
		if (isParameter())
			sb.append(" %");
		if (type == SYSTEM_ENTITY)
			sb.append(" SYSTEM");
		else if (type == PUBLIC_ENTITY)
			sb.append(" PUBLIC ").append("\"-//\"");
		sb.append(" \"").append(getValue()).append("\">");
		return sb.toString();
	}

}

// EntityDTDNode ends here
