package com.japisoft.editix.action.css;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.StringReader;

import javax.swing.AbstractAction;

import org.w3c.css.sac.CSSException;
import org.w3c.css.sac.CSSParseException;
import org.w3c.css.sac.DocumentHandler;
import org.w3c.css.sac.ErrorHandler;
import org.w3c.css.sac.InputSource;
import org.w3c.css.sac.LexicalUnit;
import org.w3c.css.sac.Locator;
import org.w3c.css.sac.SACMediaList;
import org.w3c.css.sac.SelectorList;

import com.japisoft.editix.editor.css.CSSEditor;
import com.japisoft.editix.editor.css.helper.Keywords;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.xml.CheckableAction;
import com.steadystate.css.parser.SACParserCSS21;
import com.steadystate.css.sac.DocumentHandlerExt;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CheckAction extends AbstractAction 
		implements ErrorHandler, DocumentHandlerExt, CheckableAction {

	private XMLContainer editor = null;
	private SACParserCSS21 parser = null;
	
	public void charset(String characterEncoding) throws CSSException {
	}

	public void comment(String arg0) throws CSSException {
	}

	public void endDocument(InputSource arg0) throws CSSException {
	}

	public void endFontFace() throws CSSException {
	}

	public void endMedia(SACMediaList arg0) throws CSSException {
	}

	public void endPage(String arg0, String arg1) throws CSSException {
	}

	public void endSelector(SelectorList arg0) throws CSSException {
	}

	public void ignorableAtRule(String arg0) throws CSSException {
	}

	public void importStyle(String arg0, SACMediaList arg1, String arg2)
			throws CSSException {
	}

	public void namespaceDeclaration(String arg0, String arg1)
			throws CSSException {
	}

	public void property(String arg0, LexicalUnit arg1, boolean arg2)
			throws CSSException {
		if ( !Keywords.isAProperty( arg0 ) ) {
			error( new CSSParseException("Unknown property " + arg0, parser.getLocator() ) );
		}
	}

	public void startDocument(InputSource arg0) throws CSSException {
	}

	public void startFontFace() throws CSSException {
	}

	public void startMedia(SACMediaList arg0) throws CSSException {
	}

	public void startPage(String arg0, String arg1) throws CSSException {
	}

	public void startSelector(SelectorList arg0) throws CSSException {
	}

	// ---------------------------------------------------------------
	
	public boolean checkDocument(XMLContainer editor, boolean silentMode ) {
		
		this.editor = editor;
		parser = new SACParserCSS21();
		parser.setDocumentHandler( this );
		parser.setErrorHandler( this );

		try {
			editor.getErrorManager().initErrorProcessing();
			// parser.setDocumentHandler( this );
			parser.parseStyleSheet( 
					new InputSource( 
							new StringReader( 
									editor.getText() ) ) );
			
			if ( !editor.getErrorManager().hasLastError() ) {
				if ( !silentMode )
					EditixFactory.buildAndShowInformationDialog( "Your CSS is correct" );
				editor.getErrorManager().notifyNoError(false);
			} else {
				EditixFactory.buildAndShowErrorDialog( "Error(s) found" );
			}

			editor.getErrorManager().stopErrorProcessing();
			
		} catch (IOException e1) {
		}

		boolean hasError = 
			editor.getErrorManager().hasLastError();
		
		parser = null;
		editor = null;
		
		return !hasError;
	}

	public void actionPerformed(ActionEvent e) {

		if ( Manager.isFree() ) {		

			EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
			BrowserCaller.displayURL( "http://www.editix.com" );
			
		} else {

			editor = ( CSSEditor )EditixFrame.THIS.getSelectedContainer();
			if ( editor == null ) {
				EditixFactory.buildAndShowErrorDialog( "Can't Check your CSS" );
				return;
			}
			
			checkDocument( editor, false );
			
		}
	}

	public void error(CSSParseException e) throws CSSException {
		editor.getErrorManager().notifyError(
				this,
				true,
				null,
				e.getLineNumber(),
				e.getColumnNumber(),
				-1,
				e.getMessage(),
				false );
	}

	public void fatalError(CSSParseException e) throws CSSException {
		editor.getErrorManager().notifyError(
				this,
				true,
				null,
				e.getLineNumber(),
				e.getColumnNumber(),
				-1,
				e.getMessage(),
				false );
	}

	public void warning(CSSParseException e) throws CSSException {
	}

}
