package com.japisoft.editix.action.dtdschema;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JOptionPane;

import com.japisoft.editix.action.dtdschema.generator.MetaNode;
import com.japisoft.editix.action.dtdschema.generator.SchemaGenerator;
import com.japisoft.editix.action.dtdschema.generator.Transformer;
import com.japisoft.editix.document.DocumentModel;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.framework.xml.parser.ParseException;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.XMLDocumentInfo;
import com.japisoft.xmlpad.editor.XMLTemplate;
import com.japisoft.xmlpad.tree.parser.InnerXMLParser;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class GenerateAction extends AbstractAction {

	private Transformer transformer;

	protected boolean formatResult = false;
	
	public GenerateAction( Transformer transformer ) {
		this.transformer = transformer;
	}

	public void actionPerformed( ActionEvent e ) {

		try {
		
			// Get the current root
	
			XMLContainer container = EditixFrame.THIS.getSelectedContainer();
			
			// Must switch to the inner parser for having text nodes
			
			InnerXMLParser parser = new InnerXMLParser();
			Document docTmp = parser.parseContent( container.getText() );
			
	//		FPNode root = ( FPNode )container.getTree().getModel().getRoot();
			
			FPNode root = (FPNode)docTmp.getRoot();
			
			
			if ( root == null ) {
				JOptionPane.showMessageDialog( container.getView(), "Can't generate a schema for this document " );
				return;
			}
	
			MetaNode metaRoot = SchemaGenerator.getMetaModel( root );
			MetaModelUpdatePanel mmup = null;
	
			if ( DialogManager.showDialog(
					EditixFrame.THIS,
					"Meta model",
					"Your document content",
					"Check if EditiX has found the best attribute type by selecting each node and change it if needed, then press OK",
					null,
					mmup = new MetaModelUpdatePanel( metaRoot ) ) == 
						DialogManager.OK_ID ) {
			
				transformer.setSequenceMode( mmup.hasDefaultSequence() );
				
				String content = SchemaGenerator.generate( metaRoot, transformer );
		
				XMLDocumentInfo doc = DocumentModel.getDocumentForType( transformer.getType() );
				IXMLPanel panel = EditixFactory.getPanelForDocument( doc );
				XMLContainer newContainer = panel.getMainContainer();
		
				newContainer.setAutoNewDocument( false );
				newContainer.setDocumentInfo( doc );
		
				XMLTemplate template = new XMLTemplate();
				template.setRawContent(
						( transformer.hasVersion() ? "<?xml version=\"1.0\" " + "encoding=\"${default-encoding}\"?>\n" : "" ) + 
							content );
		
				newContainer.setText( template.toString( doc ) );
				
				EditixFrame.THIS.addContainer( panel );
				
				if ( formatResult )
					ActionModel.activeActionById( "format", null );
			}
			
		} catch( ParseException p ) {
			
			EditixFactory.buildAndShowErrorDialog( "Wrong XML document, fix it before generating" );
			
		}

	}

}
