package com.japisoft.editix.action.dtdschema;

import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.AbstractAction;

import org.apache.xerces.impl.dtd.XMLDTDLoader;
import org.apache.xerces.xni.XNIException;
import org.apache.xerces.xni.parser.XMLErrorHandler;
import org.apache.xerces.xni.parser.XMLInputSource;
import org.apache.xerces.xni.parser.XMLParseException;

import com.japisoft.editix.action.file.SaveAction;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.xml.CheckableAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ParseDTDAction extends AbstractAction implements XMLErrorHandler, CheckableAction {

	private XMLContainer container;
	
	public boolean checkDocument(XMLContainer container, boolean silentMode) {
		
		this.container = container;

		XMLDTDLoader
		loader = new XMLDTDLoader();
		loader.setErrorHandler( this );
		container.getErrorManager().initErrorProcessing();
		try {
			loader.loadGrammar(
					new XMLInputSource(
							null,
							container.getCurrentDocumentLocation(),
							null
					)
			);
			if ( !silentMode )
				EditixFactory.buildAndShowInformationDialog( "DTD Checked : No syntax error" );
			container.getErrorManager().notifyNoError( false );
		} catch (XNIException e1) {
		} catch (IOException e1) {
		}
		container.getErrorManager().stopErrorProcessing();
		boolean error = container.getErrorManager().hasLastError();
		this.container = null;
		
		return !error;
	}

	public void actionPerformed( ActionEvent e ) {
		container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;
		if ( container.getCurrentDocumentLocation() == null ) {
			EditixFactory.buildAndShowInformationDialog( "Please save your document before checking" );
			return;
		}
		SaveAction sa = ( SaveAction )ActionModel.restoreAction( "save" );
		sa.save( container );
		checkDocument( container, false );
	}

	public void error(String arg0, String arg1, XMLParseException arg2) throws XNIException {
		container.getErrorManager().notifyError(
				container,
				true,
				container.getCurrentDocumentLocation(),
				arg2.getLineNumber(),
				arg2.getColumnNumber(),
				-1,
				arg2.getMessage(),
				false );		
	}

	public void fatalError(String arg0, String arg1, XMLParseException arg2) throws XNIException {
		container.getErrorManager().notifyError(
				container,
				true,
				container.getCurrentDocumentLocation(),
				arg2.getLineNumber(),
				arg2.getColumnNumber(),
				-1,
				arg2.getMessage(),
				false );		
	}

	public void warning(String arg0, String arg1, XMLParseException arg2) throws XNIException {
	}


}
