package com.japisoft.editix.action.dtdschema.generator.transformer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import com.japisoft.editix.action.dtdschema.generator.MetaAttribute;
import com.japisoft.editix.action.dtdschema.generator.MetaNode;
import com.japisoft.editix.action.dtdschema.generator.MetaObject;
import com.japisoft.editix.action.dtdschema.generator.Transformer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DTDTransformer extends AbstractTransformer {

	public String getType() {
		return "DTD";
	}
	
	public boolean hasVersion() { return false; }

	protected void generateMetaNode( MetaNode node, StringBuffer sb ) {
		sb.append( System.getProperty( "line.separator" ) );
		sb.append( "<!ELEMENT ").append( node.getName() ).append( "  " );

		if ( node.getChildren().size() == 0 ) {
			if ( node.acceptText() )
				sb.append( "( #PCDATA )" );
			else
				sb.append( "EMPTY" );
		}

		for ( int i = 0; i < node.getChildren().size(); i++ ) {
			if ( i == 0 ) {
				sb.append( "( " );
				if ( node.acceptText() )
					sb.append( "#PCDATA | " );
			}

			boolean choiceMode = false;
			
			if ( i > 0 ) {
				if ( sequenceMode ) {
					sb.append( " , " );
				} else {
					sb.append( " | " );
					choiceMode = true;
				}
			}

			MetaNode child = ( MetaNode )( node.getChildren().get( i ) );
			sb.append( child.getName() );

			if ( !choiceMode ) {
			
				if ( node.hasMultipleOccurence( child ) ) {
					
					if ( node.canBeMissing( child ) ) 
					
						sb.append( "*" );
					
					else
						
						sb.append( "+" );
					
				} else
				if ( node.canBeMissing( child ) ) {
					
					sb.append( "?" );
					
				}
				
			}

			if ( i == node.getChildren().size() - 1 ) {
				sb.append( " )" );
			}
			
			if ( choiceMode ) {
				sb.append( "*" );
			}
			
		}

		sb.append( ">" ).append( System.getProperty( "line.separator" ) );
		if ( node.hasAttributes() ) {
			sb.append( "<!ATTLIST " ).append( node.getName() ).append( System.getProperty( "line.separator" ) );
			for ( int i = 0; i < node.getAttributes().size(); i++ ) {
				MetaAttribute ma = ( MetaAttribute )node.getAttributes().get( i );

				String finalType = translateType( ma.getType() );

				sb.append( ma.getName() ).append( " " ).append( finalType ).append( " ").append(
						ma.isAlways() ? "#REQUIRED" : "#IMPLIED" ).append( System.getProperty( "line.separator" ) );
			}
			sb.append( ">" ).append( System.getProperty( "line.separator" ) );
		}
	}
	
	private String translateType( String type ) {
		String finalType = "CDATA";
		
		if ( MetaObject.ID_TYPE.equals( type ) )
			finalType = "ID";
		else
		if ( MetaObject.IDREF_TYPE.equals( type ) )
			finalType = "IDREF";

		return finalType;
	}
	
}
