package com.japisoft.editix.action.file;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JDialog;
import javax.swing.JLabel;

import com.japisoft.editix.action.xml.format.FormatAction;
import com.japisoft.editix.document.DocumentModel;
import com.japisoft.editix.document.TemplateInfo;
import com.japisoft.editix.document.TemplateModel;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.InformationDialog;
import com.japisoft.framework.app.toolkit.Toolkit;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.xml.format.FormatterConfig;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.XMLDocumentInfo;
import com.japisoft.xmlpad.action.ActionModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NewAction extends AbstractAction {

	public static void prepare( TemplateInfo info ) {
		if ( info.encoding == null )
			info.encoding =   
				Preferences.getPreference( 
					"file", 
					"rw-encoding", 
					Toolkit.FILE_ENCODING )[ 0 ]; 
	}

	public void actionPerformed( ActionEvent e ) {
		TemplateInfo info = getDocumentInfo();
		if ( info == null ) {
			return;
		}
		
		prepare( info );
		signalToUser( info.type );

		String wizardContent = info.startWizard();
		boolean wizardMode = info.hasWizard();

		if ( wizardMode && wizardContent == null )
			return;
		
		XMLDocumentInfo doc = 
			DocumentModel.getDocumentForType( info.type );
		IXMLPanel panel = 
			EditixFactory.getPanelForDocument( doc );
		XMLContainer container = 
			panel.getMainContainer();
		XMLDocumentInfo newDoc = 
			doc.cloneDocument();

		// Load the template
 		if ( info.location != null ) { 			
 			try {
 				TemplateModel.resolveTemplate(info.encoding, info.location, newDoc); 
 			} catch( Throwable th ) {
 				EditixFactory.buildAndShowErrorDialog( "Can't load " + info.location );
 				th.printStackTrace();
 				return;
 			}
 		}
 		
		container.setDocumentInfo( newDoc );
		
		com.japisoft.xmlpad.action.ActionModel.activeActionByName(
			ActionModel.NEW_ACTION,
			container,
			container.getEditor()
		);
		
		container.setProperty( "encoding", com.japisoft.editix.toolkit.Toolkit.getCurrentFileEncoding() );
		if ( wizardContent != null ) {
			container.setText( wizardContent );
			
			if ( "XML".contentEquals( info.type ) ) {
				FormatAction.format(
					container,
					null,
					new FormatterConfig(),
					null
				);

			}
			
			if ( "XSLT".equals( info.type ) ) {
				File source = info.getWizardSource();
				panel.setProperty( "xslt.data.file", source );
			}
		}
		
		
		EditixFrame.THIS.addContainer( panel );
		
		container.getEditor().notifyCurrentLocation( true );
	}

	static void signalToUser( String type ) {
		if ( "EXML".equals( type ) ) {
			if ( Preferences.getPreference( "system", "usertype" + type, true ) ) {
				JDialog dialog = new InformationDialog( 
						new JLabel( 
								"Note that you are using a special editing mode for Large XML documents. Choose another document type for common documents", 
								JLabel.LEFT ),		
						"usertype" + type );
				dialog.setVisible( true );
				dialog.dispose();
			}
		}		
	}

	public void setTemplate( TemplateInfo ti ) {
		putValue( "template", ti );
	}
	
	private TemplateInfo getDocumentInfo() {
		if ( getValue( "template" ) != null )
			return ( TemplateInfo )getValue( "template" );
		
		SelectTemplatePanel stp = new SelectTemplatePanel();

		if (
			DialogManager.showDialog(
				EditixFrame.THIS, 
				"Create a document",
				"New document", 
				"Select a template for building a new document", 
				null, 
				stp ) == DialogManager.OK_ID ) {

			return stp.getTemplateInfo();			
		}

		return null;
	}

}
