package com.japisoft.editix.action.file;

import java.awt.event.ActionEvent;
import java.io.File;
import java.net.URLDecoder;

import javax.swing.AbstractAction;

import com.japisoft.framework.application.descriptor.ActionModel;
import com.japisoft.editix.document.DocumentModel;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class OpenFileAtAction extends AbstractAction {

	public void actionPerformed( ActionEvent e ) {
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;
		XMLPadDocument doc = container.getXMLDocument();
		String word = doc.getWordAt(
			container.getEditor().getCaretPosition() );
		
		if ( word == null ) {
			EditixFactory.buildAndShowErrorDialog( "No file found" );
			return;
		}

		String fileName = null;

		if ( word.startsWith( "file://" ) ) {
			fileName = URLDecoder.decode( word.substring( 7 ) );
		} else
			if ( word.startsWith( "file:" ) ) {
				fileName = URLDecoder.decode( word.substring( 5 ) );
			} else {
				String loc = container.getCurrentDocumentLocation();
				if ( loc == null ) {
					fileName = word;
				} else {
					File f = new File( new File( loc ).getParentFile(), word );
					if ( f.exists() ) {
						fileName = f.toString();
					} else
						fileName = word;
				}
			}
		
		String type = DocumentModel.getDocumentByFileName( fileName ).getType();

		if ( fileName.indexOf( "://" ) == -1 ) {
			if (!OpenAction.openFile( type, true, new File( fileName ), null )) {
				EditixFactory.buildAndShowErrorDialog("Can't open " + fileName );
			}
		}
		else
			ActionModel.activeActionById( ActionModel.OPEN, e, fileName, type );
	}

}
