package com.japisoft.editix.action.file.export;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class Tools {

	// Cause : Bug xerces for local name
	public static String getLocalNameForQName( String qname ) {
		int i = qname.indexOf( ":" );
		if ( i > -1 )
			return qname.substring( i + 1 );
		return qname;
	}

	public static String toClassName( String name ) {
		if ( name.length() > 1 )
			return Character.toUpperCase( name.charAt( 0 ) ) + name.substring( 1 );
		else
			return name.toUpperCase();
	}	

	public static String toAddName( String name ) {
		return toAnyName( "add", name );
	}
		
	public static String toSetName( String name ) {
		return toAnyName( "set", name );
	}

	public static String toGetName( String name ) {
		return toAnyName( "get", name );
	}

	public static String toAnyName( String prefix, String name ) {
		//name = convertIfKeyWord( name );
		if ( name.length() > 1 )
			return prefix + Character.toUpperCase( name.charAt( 0 ) ) + name.substring( 1 );
		else
			return prefix + name.toUpperCase();
	}

	public static final String[] KEYWORDS = new String[] {
		"abstract",
		"assert",
		"boolean",
		"break",
		"byte",
		"case",
		"catch",
		"char",
		"class",
		"const",
		"continue",
		"default",
		"do",
		"double",
		"else",
		"enum",
		"extends",
		"false",
		"final",
		"finally",
		"float",
		"for",
		"goto",
		"if",
		"implements",
		"import",
		"instanceof",
		"int",
		"interface",
		"long",
		"native",
		"new",
		"null",
		"package",
		"private",
		"protected",
		"public",
		"return",
		"short",
		"static",
		"strictfp",
		"super",
		"switch",
		"synchronized",
		"this",
		"throw",
		"throws",
		"transient",
		"true",
		"try",
		"void",
		"volatile",
		"while"
	};

	public static String convertIfKeyWord( String value ) {
		for ( int i = 0; i < KEYWORDS.length; i++ ) {
			if ( value.equals( KEYWORDS[ i ] ) ) {
				return "_" + value;
			}
		}
		return value;
	}

	public static void warning( String message ) {
		System.out.println( "* Warning : " + message );
	}
	
	public static void error( String message ) {
		System.err.println( "** Error : " + message );
	}
	
	public static boolean DEBUG;
	static {
		try {
			DEBUG = "true".equals( System.getProperty( "debug" ) );
		} catch( Throwable th ) {
			// For java web start
		}
	}

	public static void debug( String message ) {
		if ( DEBUG )
			System.out.println( message );
	}
}

