package com.japisoft.editix.action.file.imp.spreadsheet;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;

import com.japisoft.framework.app.toolkit.Toolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SpreadSheetConfPanel extends JPanel implements ActionListener {

    /**
     * Creates new form SpreadSheetImportPanel
     */
    public SpreadSheetConfPanel() {
        initComponents();
        cbEncoding.removeItemAt( 0 );
        for ( String e : Toolkit.FILE_ENCODING ) {
        	cbEncoding.addItem( e );
        }
    }
    
    @Override
    public void addNotify() {
    	super.addNotify();
    	cpColNames.addActionListener( this );
    }
    
    @Override
    public void removeNotify() {
    	super.removeNotify();
    	cpColNames.removeActionListener( this );
    }
    
	@Override
	public void actionPerformed(ActionEvent e) {
		if ( cpColNames.getSelectedIndex() == 0 ) {
			txtFromTheRow.setText( "2" );
		} else
			txtFromTheRow.setText( "1" );
	}
    
    public void setExcelMode( boolean excelMode ) {
    	cbColSep.setEnabled( !excelMode );
    	cbEncoding.setEnabled( !excelMode );
    }
    
    public boolean isExcelMode() { return !cbColSep.isEnabled(); }
    
    public String getColSep() { 
    	String tmp = (String)cbColSep.getSelectedItem();
    	if ( "Semicolon".equalsIgnoreCase( tmp ) )
    		return ";";
    	if ( "Comma".equalsIgnoreCase( tmp ) )
    		return ",";
    	if ( "Tabulation".equalsIgnoreCase( tmp ) )
    		return "\t";
    	return tmp;
    }
    
    public boolean useFirstRowForColNames() {
    	return "First Row".equalsIgnoreCase( (String)cpColNames.getSelectedItem() );
    }
    
    public String getColTag() { return (String)cpColNames.getSelectedItem(); }
    
    public String getRowTag() { return txtRowName.getText(); }
    
    public boolean allSheets() { return "All".equalsIgnoreCase( (String)cbSheet.getSelectedItem() ); }
    
    public String sheetName() { return (String)cbSheet.getSelectedItem(); }
    
    public int fromTheCol() {
    	try {
    		return Integer.parseInt( txtFromTheCol.getText() );
    	} catch( NumberFormatException pne ) {
    		return 1;
    	}
    }
    
    public boolean allTheCol() { return "Last".equalsIgnoreCase( (String)cbToTheCol.getSelectedItem() ); }
    
    public int toTheCol() {    	
    	try {
    		int i = Integer.parseInt( (String)cbToTheCol.getSelectedItem() );
    		return i;
    	} catch( NumberFormatException exc ) {
    	}
    	return Integer.MAX_VALUE;
    }
    
    public int fromTheRow() { 
    	return Integer.parseInt( txtFromTheRow.getText() );
    	
    }
    
    public int toTheRow() { return Integer.parseInt( txtFromTheRow.getText() ); }
    
    public boolean allTheRow() { return "Last".equalsIgnoreCase( (String)cbToTheRow.getSelectedItem() ); }
    
    public boolean tagCellValue() { return rbTagMode.isSelected(); }
    
    public boolean attributeCellValue() { return rbAttMode.isSelected(); }

    public String getEncoding() { return (String)cbEncoding.getSelectedItem(); }
    
    private Map<Integer,String> colNames = null;
    
    public String getColName( int index ) {
    	if ( !useFirstRowForColNames() )
    		return (String)cpColNames.getSelectedItem();
    	if ( colNames == null )
    		return "col";
    	return colNames.get( index );
    }
    
    public void setColName( int index, String colName ) {
    	if ( colNames == null )
    		colNames = new HashMap<Integer,String>();
    	colNames.put( index, colName );
    }
    
    public void addSheet( String name ) {
    	cbSheet.addItem( name );
    }
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">                          
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        jLabel1 = new javax.swing.JLabel();
        cbColSep = new javax.swing.JComboBox<>();
        jLabel2 = new javax.swing.JLabel();
        cpColNames = new javax.swing.JComboBox<>();
        jLabel3 = new javax.swing.JLabel();
        txtRowName = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        cbSheet = new javax.swing.JComboBox<>();
        jLabel5 = new javax.swing.JLabel();
        txtFromTheRow = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        cbToTheRow = new javax.swing.JComboBox<>();
        jLabel7 = new javax.swing.JLabel();
        rbAttMode = new javax.swing.JRadioButton();
        rbTagMode = new javax.swing.JRadioButton();
        jLabel8 = new javax.swing.JLabel();
        txtFromTheCol = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        cbToTheCol = new javax.swing.JComboBox<>();
        jLabel10 = new javax.swing.JLabel();
        cbEncoding = new javax.swing.JComboBox<>();

        jLabel1.setText("Column Separator");

        cbColSep.setEditable(true);
        cbColSep.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Semicolon", "Tabulation", "Comma", " " }));

        jLabel2.setText("Column Names");

        cpColNames.setEditable(true);
        cpColNames.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "First Row" }));

        jLabel3.setText("Row Name");

        txtRowName.setText("row");

        jLabel4.setText("Sheet");

        cbSheet.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "All" }));

        jLabel5.setText("From the Row ");

        txtFromTheRow.setText("2");

        jLabel6.setText("To the Row");

        cbToTheRow.setEditable(true);
        cbToTheRow.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Last" }));

        jLabel7.setText("Cell values");

        buttonGroup1.add(rbAttMode);
        rbAttMode.setText("Attributes");

        buttonGroup1.add(rbTagMode);
        rbTagMode.setSelected(true);
        rbTagMode.setText("Tags");

        jLabel8.setText("From the column");

        txtFromTheCol.setText("1");

        jLabel9.setText("To the Column");

        cbToTheCol.setEditable(true);
        cbToTheCol.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Last", " " }));

        jLabel10.setText("Encoding");

        cbEncoding.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Default" }));

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel1)
                    .addComponent(jLabel2)
                    .addComponent(jLabel3)
                    .addComponent(jLabel4)
                    .addComponent(jLabel7)
                    .addComponent(jLabel8)
                    .addComponent(jLabel9)
                    .addComponent(jLabel5)
                    .addComponent(jLabel6)
                    .addComponent(jLabel10))
                .addGap(28, 28, 28)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(cbToTheCol, 0, 188, Short.MAX_VALUE)
                    .addComponent(cbToTheRow, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(cpColNames, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(txtRowName)
                    .addComponent(cbSheet, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(cbColSep, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(txtFromTheCol)
                    .addComponent(txtFromTheRow)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(rbTagMode)
                            .addComponent(rbAttMode))
                        .addGap(0, 0, Short.MAX_VALUE))
                    .addComponent(cbEncoding, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(cbColSep, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(cpColNames, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(txtRowName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addComponent(cbSheet, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(txtFromTheRow, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(cbToTheRow, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel8)
                    .addComponent(txtFromTheCol, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel9)
                    .addComponent(cbToTheCol, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(rbAttMode))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(rbTagMode)
                .addGap(18, 18, 18)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel10)
                    .addComponent(cbEncoding, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(32, Short.MAX_VALUE))
        );
    }// </editor-fold>                        


    // Variables declaration - do not modify                     
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JComboBox<String> cbColSep;
    private javax.swing.JComboBox<String> cbEncoding;
    private javax.swing.JComboBox<String> cbSheet;
    private javax.swing.JComboBox<String> cbToTheCol;
    private javax.swing.JComboBox<String> cbToTheRow;
    private javax.swing.JComboBox<String> cpColNames;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JRadioButton rbAttMode;
    private javax.swing.JRadioButton rbTagMode;
    private javax.swing.JTextField txtFromTheCol;
    private javax.swing.JTextField txtFromTheRow;
    private javax.swing.JTextField txtRowName;
    // End of variables declaration                   

}
