package com.japisoft.editix.action.file.imp.spreadsheet.csv;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

import com.japisoft.editix.action.file.imp.spreadsheet.Row;
import com.japisoft.editix.action.file.imp.spreadsheet.Sheet;
import com.japisoft.editix.action.file.imp.spreadsheet.SpreadSheetConfPanel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CSVSheet implements Sheet {

	CSVSheet( SpreadSheetConfPanel conf, File f ) throws Exception {

		Reader r = null;
		if ( "DEFAULT".equalsIgnoreCase( conf.getEncoding() ) )
			r = new FileReader( f );
		else
			r = new InputStreamReader( new FileInputStream( f ), conf.getEncoding() );
		
		BufferedReader br = new BufferedReader( r );
				;
		try {
			String l = null;
			while ( ( l = br.readLine() ) != null ) {
				scanRow( conf, l );
			}
		} finally {
			br.close();
		}
	}
	
	private void scanRow( SpreadSheetConfPanel conf, String l ) {
		String sep = conf.getColSep();
		l = l.trim();

		String esep = sep;
		
		if ( l.startsWith( "\"" ) &&
			l.endsWith( "\"" ) ) {
			esep = "\"" + sep + "\"";

			// Remove the first and the last string quote
			l = l.substring( 1 );
			l = l.substring( 0, l.length() - 1 );
		}
		
		String[] cols = l.split( esep );
		addRow( cols );
	}
	
	private List<CSVRow> rows = null; 
			
	private void addRow( String[] cols ) {
		if ( rows == null )
			rows = new ArrayList<CSVRow>();
		rows.add( new CSVRow( cols ) );
	}
	
	@Override
	public int getRowCount() {
		if ( rows == null )
			return 0;
		return rows.size();
	}
	
	@Override
	public Row getRow(int index) {
		return rows.get( index - 1 );
	}
	
	@Override
	public String getName() {
		return null;
	}
	
}
