package com.japisoft.editix.action.file.project;

import java.awt.event.ActionEvent;
import java.io.File;

import com.japisoft.editix.action.panels.PanelAction;
import com.japisoft.editix.ui.panels.Panel;
import com.japisoft.editix.ui.panels.project2.NewProjectPanel;
import com.japisoft.editix.ui.panels.project2.ProjectPanel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationModel.ApplicationModelListener;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.dialog.actions.OKAction;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NewProjectAction extends PanelAction implements ApplicationModelListener {

	private static final String PROJECT_HISTORY = "lastProjects.txt";

	public NewProjectAction() {
		super();
		// For avoiding duplicate call
		if ( getClass() == NewProjectAction.class ) {
			ApplicationModel.addApplicationModelListener( this );
		}
	}

	public void actionPerformed(ActionEvent e) {
		NewProjectPanel npp = new NewProjectPanel();
		if ( DialogManager.showDialog(
				null, 
				"Create a a new project",
				"New Project", 
				"Choose your project name and location",
				null,
				npp ) == OKAction.ID ) {

			File projectLocation = npp.getProjectLocation();
			if ( !projectLocation.exists() )
				projectLocation.mkdirs();
			loadProject( projectLocation );
		}
	}

	public void fireApplicationData(String key, Object... values) {
		if ( "lastProject".equals( key ) ) {
			String fileName = ( String )values[ 0 ];
			if ( fileName == null ) {
				try {
					hide();
				} catch( Exception exc ) {}
			} else {
				showHide();				
				File f = new File( fileName );
				ApplicationModel.setSharedProperty( "lastProject", f );
				( ( ProjectPanel )getPanel() ).loadProject( f );	
			}
		} else
		if ( "quit".equals( key ) ) {
			stop();
		}
	}

	public void loadProject( File projectLocation ) {

		String projectPath = null;

		if ( projectLocation != null )
			projectPath = projectLocation.toString();
		
		ApplicationModel.fireApplicationValue( 
			"lastProject",
			projectPath );


	}
	
	private ProjectPanel pp = null;
	
	@Override
	public Panel preparePanel() {
		if ( pp == null ) {
			pp = new ProjectPanel(); 
		}
		return pp;
	}
	
}
