package com.japisoft.editix.action.file.project;

import java.awt.Component;
import java.awt.HeadlessException;
import java.io.File;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileView;

import com.japisoft.editix.ui.SelectableEncoding;
import com.japisoft.editix.ui.panels.project2.DefaultProject;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ProjectFileChooser extends JFileChooser implements SelectableEncoding {

	private JComboBox combo;

	public ProjectFileChooser() {
		init();
	
		addChoosableFileFilter( new FileFilter() {			
			@Override
			public String getDescription() {
				return "Editix's Project";
			}
			@Override
			public boolean accept( File f ) {
				return DefaultProject.isProjectPath( f );
			}
		} );
		
	}

	private void init() {
		setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
		setMultiSelectionEnabled(false);
		setFileView( new ProjectFileView() );		
	}

	public int showDialog( Component parent, String approveButtonText ) throws HeadlessException {
		return super.showDialog( parent, approveButtonText );
	}
	
	public String getSelectedEncoding() {
		return ( String )combo.getSelectedItem();
	}

	class ProjectFileView extends FileView {
		private Icon projectIcon = null;

		public ProjectFileView() {
			super();
			projectIcon = new ImageIcon( 
				getClass().getResource( "environment2.png" ) 
			);
		}

		@Override
		public Icon getIcon( File f ) {
			if ( f.isDirectory() ) {
				if ( DefaultProject.isProjectPath( f ) ) {
					return projectIcon;
				}
			}
			return super.getIcon( f );
		}
	}

}
