package com.japisoft.editix.action.panels;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Icon;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.panels.Panel;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.p3.Manager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PanelAction extends AbstractAction {

	private Panel p;
	
	public boolean isPrepared() {
		return p != null;
	}
	
	protected Panel getPanel() {
		return p;
	}
	
	public Panel preparePanel() {
		if ( p == null ) {
			try {
				String cl = ( String )getValue( "param" );
				if ( cl == null )
					throw new RuntimeException( "No param found ???" );
				Class _c = Class.forName( cl );
				p = ( Panel )_c.newInstance();
				p.setIcon( ( Icon )getValue( SMALL_ICON ) );
				p.setId( (String)getValue( "id" ) );
				p.init();
			} catch ( ClassNotFoundException e1 ) {
				e1.printStackTrace();
			} catch ( InstantiationException e1 ) {
				e1.printStackTrace();
			} catch ( IllegalAccessException e1 ) {
				e1.printStackTrace();
			}
		}	
		return p;
	}

	protected void stop() {
		if ( p != null )
			p.stop();		
	}

	protected boolean alwaysShown = false;

	public void setForceAlwaysShown( boolean alwaysShown ) {
		this.alwaysShown = alwaysShown;
	}
	
	protected Object initParameter = null; 

	public void setInitParameter( Object initParameter ) {
		this.initParameter = initParameter;
	}

	public void actionPerformed(ActionEvent e) {

		if ( Manager.isFree() ) {

			String param2 = ( String )getValue( "param" );
			if ( "com.japisoft.editix.db.ui.DbPanel".equals( param2 ) || 
					"com.japisoft.editix.ui.panels.project.ProjectPanel".equals( param2 ) ||
						"com.japisoft.editix.ui.panels.project.ProjectPanel".equals( param2 ) ||
							"com.japisoft.editix.action.search.file.FileSearchPanel".equals( param2 ) ) {

				EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
				BrowserCaller.displayURL( "http://www.editix.com" );
				return;
				
			}
			
		}		

		showHide();
		
	}

	
	protected void showHide() {

		p = preparePanel();
		
		if ( p != null ) {
			
			p.setParams( ( String )getValue( "param2" ) );

			if ( alwaysShown )
				p.showPanel();
			else
				p.showHidePanel();
		}
				
	}
	
	protected void hide() {

		p = preparePanel();
		
		if ( p != null ) {
			p.stop();
			//if ( p.isShown() )
				p.hidePanel();
		}
		
		
	}
	
}
