package com.japisoft.editix.action.xml;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.text.BadLocationException;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CommentAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null ) {
			EditixFactory.buildAndShowErrorDialog( "No document" );
			return;
		}
		
		// Check if we are inside a comment
		boolean in = container.getXMLDocument().isInsideComment( container.getEditor().getCaretPosition() );
		if ( in ) {
			// Remove the comment
			Integer[] l = container.getXMLDocument().getCommentDelimiters( container.getEditor().getCaretPosition() );
			if ( l != null ) {
				int start = l[ 0 ].intValue();
				int stop = l[ 1 ].intValue();
				container.getEditor().select( 
						start, 
						stop + 1 );
				container.getEditor().replaceSelection( 
						container.getEditor().getSelectedText().substring(
								4,
								container.getEditor().getSelectedText().length() - 3
						)
				);
				return;
			} else {
				EditixFactory.buildAndShowErrorDialog( "Can't remove the current comment" );
				return;
			}
		}

		String selection = container.getEditor().getSelectedText();
		if ( selection == null ) {
			// Try with the current node
			FPNode n = container.getCurrentElementNode();
			if ( n != null ) {
				int start = n.getStartingOffset();
				int stop = n.getStoppingOffset();
				container.getEditor().select( start, stop + 1 );
				selection = container.getEditor().getSelectedText();
			}
		}

		if ( selection != null ) {
				// Test for comment inside
				if ( selection.indexOf( "<!--" ) > -1 ) {
					EditixFactory.buildAndShowWarningDialog( "Please remove the comment(s) inside before operating" );
					return;
				}			
				String newContent = "<!--" + selection + "-->";
				container.getEditor().replaceSelection( newContent );
		} else {
			try {
				// Insert empty comment at the cursor location
				container.getEditor().getDocument().insertString(
						container.getEditor().getCaretPosition(),
						"<!-- -->", null );
				container.getEditor().setCaretPosition(
						container.getEditor().getCaretPosition() + 4 );
			} catch (BadLocationException e1) {
				EditixFactory.buildAndShowErrorDialog( "Can't comment" );
			}
		}
			
	}

}
