package com.japisoft.editix.action.xml;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JPanel;
import javax.swing.text.Document;
import javax.swing.text.Element;

import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.bookmark.BookmarkPosition;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SurroundAction extends AbstractAction {

	private SurroundPanel panel = null;

	public void surround( XMLContainer container, String tag, String attributes, boolean splitMode, String splitSeq, boolean trimValues, boolean bookmarkSelection ) {
		if ( !bookmarkSelection )
			surround( 
				container, 
				tag, 
				attributes, 
				splitMode, 
				splitSeq, 
				trimValues, 
				container.getEditor().getSelectionStart(), 
				container.getEditor().getSelectionEnd(), false );
		else {

			for ( int i = 0;i < container.getBookmarkContext().getModel().getBookmarkCount(); i++ ) {
				BookmarkPosition position = container.getBookmarkContext().getModel().getBookmarkPositionAt( i );
				int offset = position.getOffset();
				int index = container.getDocument().getDefaultRootElement().getElementIndex( offset );
				Element e = container.getDocument().getDefaultRootElement().getElement( index );
				container.getEditor().setSelectionStart( e.getStartOffset() );
				container.getEditor().setSelectionEnd( e.getEndOffset() );

				surround( 
						container, 
						tag, 
						attributes, 
						splitMode, 
						splitSeq, 
						trimValues, 
						e.getStartOffset(), 
						e.getEndOffset(),
						true );	
				
				
			}
			
			container.getBookmarkContext().getModel().removeAllBookmarks();
			container.getEditor().getHighlighter().removeAllHighlights();
			
		}
	}
	
	public void surround( XMLContainer container, String tag, String attributes, boolean splitMode, String splitSeq, boolean trimValues, int start, int end, boolean cr ) {
	
		if ( end == start ) {
			// For a selection to the end of the line
			Document doc = container.getEditor().getDocument();
			int index = doc.getDefaultRootElement().getElementIndex( start );
			end = doc.getDefaultRootElement().getElement( index ).getEndOffset();
			start = doc.getDefaultRootElement().getElement( index ).getStartOffset();
			container.getEditor().setSelectionStart( start );
			container.getEditor().setSelectionEnd( end );
		}
		
		String content = container.getEditor().getSelectedText();
		String[] parts = null;

		if ( splitMode ) {
			parts = content.split( splitSeq );
		} else {
			parts = new String[] { content };
		}
		
		
		if ( tag.startsWith( "<" ) ) {
			tag = tag.substring( 1 );
		}
		if ( tag.endsWith( ">" ) )
			tag = tag.substring( 0, tag.length() - 1 );
		
		String startTag = "<" + tag;
		
		if ( attributes != null && !"".equals( attributes ) ) {
			startTag += " ";
			startTag += attributes;
		}
		
		startTag += ">";

		int i = tag.indexOf( " " );
		if ( i == -1 )
			i = tag.length();
		String endTag = "</" + tag.substring( 0, i ) + ">";
		
		StringBuffer sbRes = new StringBuffer();
		for ( i = 0; i < parts.length; i++ ) {
			sbRes.append( startTag );
			
			String value = parts[ i ];
			if ( trimValues ) {
				value = value.trim();
			}
			
			sbRes.append( value );
			sbRes.append( endTag );
			if ( splitMode || cr )
				sbRes.append( "\n" );
		}
		
		container.getEditor().replaceSelection( sbRes.toString() );
		
	}
	
	public void surround( XMLContainer container ) {
		surround( container, panel.getTag(), panel.getAttributes(), panel.splitMode(), panel.getSplitSeq(), panel.trimMode(), panel.bookmarkSelections() );		
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {

		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null ) {
			return;
		}

		if ( panel == null )
			panel = new SurroundPanel();

		if ( DialogManager.showDialog( 
			EditixFrame.THIS, 
			"Surround by", 
			"Surround", 
			"Surround your lines by a tag\nYou may insert new attributes too", 
			null, panel ) == DialogManager.OK_ID ) {

			if ( panel.getTag() == null || panel.getTag().length() == 0 ) {
				EditixFactory.buildAndShowErrorDialog( "Invalid tag" );
				return;
			}
			
			surround( container );

			panel.addTag( panel.getTag() );
			panel.addSplit( panel.getSplitSeq() );

			EditixApplicationModel.INTERFACE_BUILDER.setEnabledActionForId( "repeatSurroundTag", true );
			
		}

	}

	static class SurroundPanel extends JPanel {
		
		public SurroundPanel() {
			initComponents();
		}

//////  /**
//////  * This method is called from within the constructor to initialize the form.
//////  * WARNING: Do NOT modify this code. The content of this method is always
//////  * regenerated by the Form Editor.
//////  */
////// @SuppressWarnings("unchecked")
 // <editor-fold defaultstate="collapsed" desc="Generated Code">                          
 private void initComponents() {

     bgSelection = new javax.swing.ButtonGroup();
     jLabel3 = new javax.swing.JLabel();
     jLabel1 = new javax.swing.JLabel();
     cbTag = new javax.swing.JComboBox();
     chkSplitMode = new javax.swing.JCheckBox();
     cbSplit = new javax.swing.JComboBox();
     jLabel2 = new javax.swing.JLabel();
     cbAttributes = new javax.swing.JComboBox();
     rbSelection = new javax.swing.JRadioButton();
     jLabel4 = new javax.swing.JLabel();
     rbBookmarks = new javax.swing.JRadioButton();
     cbTrim = new javax.swing.JCheckBox();

     jLabel3.setText("jLabel3");

     setBorder(javax.swing.BorderFactory.createTitledBorder("Surround by"));

     jLabel1.setText("Tag");

     cbTag.setEditable(true);

     chkSplitMode.setText("Split lines with :");
     chkSplitMode.setToolTipText("Cut into several lines");

     cbSplit.setEditable(true);
     cbSplit.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "carriage return" }));

     jLabel2.setText("Attributes");

     cbAttributes.setEditable(true);

     bgSelection.add(rbSelection);
     bgSelection.add(rbBookmarks);
     
     rbSelection.setSelected(true);
     rbSelection.setText("Selection");

     jLabel4.setText("Apply to");

     rbBookmarks.setText("Bookmarks");

     cbTrim.setText("Trim values");
     cbTrim.setSelected( true );
     cbTrim.setToolTipText("Remove whitespaces before and after");

     javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
     this.setLayout(layout);
     layout.setHorizontalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
             .addContainerGap()
             .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                 .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                     .addComponent(cbTrim)
                     .addGap(32, 32, 32)
                     .addComponent(chkSplitMode)
                     .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 6, Short.MAX_VALUE)
                     .addComponent(cbSplit, javax.swing.GroupLayout.PREFERRED_SIZE, 102, javax.swing.GroupLayout.PREFERRED_SIZE))
                 .addGroup(layout.createSequentialGroup()
                     .addComponent(jLabel1)
                     .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                     .addComponent(cbTag, javax.swing.GroupLayout.PREFERRED_SIZE, 264, javax.swing.GroupLayout.PREFERRED_SIZE))
                 .addGroup(layout.createSequentialGroup()
                     .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                         .addComponent(jLabel2)
                         .addComponent(jLabel4))
                     .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                     .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                         .addGroup(layout.createSequentialGroup()
                             .addComponent(rbSelection)
                             .addGap(18, 18, 18)
                             .addComponent(rbBookmarks)
                             .addGap(0, 0, Short.MAX_VALUE))
                         .addComponent(cbAttributes, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
             .addContainerGap())
     );
     layout.setVerticalGroup(
         layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
         .addGroup(layout.createSequentialGroup()
             .addGap(18, 18, 18)
             .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(jLabel1)
                 .addComponent(cbTag, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addGap(18, 18, 18)
             .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(jLabel2)
                 .addComponent(cbAttributes, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
             .addGap(18, 18, 18)
             .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(rbSelection)
                 .addComponent(jLabel4)
                 .addComponent(rbBookmarks))
             .addGap(18, 18, 18)
             .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                 .addComponent(chkSplitMode)
                 .addComponent(cbSplit, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                 .addComponent(cbTrim))
             .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
     );
 }// </editor-fold>                        

	    public String getTag() { 
	    	return ( String )cbTag.getSelectedItem(); 
	    }
	    
	    public String getAttributes() {
	    	return (String)cbAttributes.getSelectedItem();
	    }
	    
	    public boolean bookmarkSelections() {
	    	return rbBookmarks.isSelected();
	    }
	    
	    public boolean trimMode() {
	    	return cbTrim.isSelected();
	    }

	    public boolean splitMode() { 
	    	return chkSplitMode.isSelected();
	    }

	    public String getSplitSeq() {
	    	String tmp = ( String )cbSplit.getSelectedItem();
	    	if ( "carriage return".equalsIgnoreCase( tmp ) )
	    		tmp = "\n";
	    	if ( tmp == null )
	    		tmp = "\n";
	    	return tmp; 
	    }

	    public void addTag( String tag ) {
	    	DefaultComboBoxModel model = ( DefaultComboBoxModel )cbTag.getModel();
	    	for ( int i = 0; i < model.getSize(); i++ ) {
	    		if ( tag.equalsIgnoreCase( ( String )model.getElementAt( i ) ) )
	    			return;
	    	}
	    	model.addElement( tag );
	    }
	    
	    public void addSplit( String tag ) {
	    	DefaultComboBoxModel model = ( DefaultComboBoxModel )cbSplit.getModel();
	    	for ( int i = 0; i < model.getSize(); i++ ) {
	    		if ( tag.equalsIgnoreCase( ( String )model.getElementAt( i ) ) )
	    			return;
	    	}
	    	model.addElement( tag );
	    }
	    
	    // Variables declaration - do not modify                     
	    private javax.swing.ButtonGroup bgSelection;
	    private javax.swing.JComboBox cbTag;
	    private javax.swing.JCheckBox chkSplitMode;
	    private javax.swing.JCheckBox cbTrim;
	    private javax.swing.JComboBox cbSplit;
	    private javax.swing.JComboBox<String> cbAttributes;
	    private javax.swing.JLabel jLabel1;
	    private javax.swing.JLabel jLabel2;
	    private javax.swing.JLabel jLabel3;
	    private javax.swing.JLabel jLabel4;
	    private javax.swing.JRadioButton rbBookmarks;
	    private javax.swing.JRadioButton rbSelection;
	    // End of variables declaration       		
	}
	
	
}
