package com.japisoft.editix.action.xml;

import java.awt.Dimension;
import java.io.File;

import com.japisoft.editix.toolkit.Toolkit;
import com.japisoft.editix.ui.EditixDialog;
import com.japisoft.framework.ui.text.PathBuilder;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class UseDefaultDialog extends EditixDialog {
	private UseDefaultPanel panel;

	public UseDefaultDialog( 
		String title, 
		String comment, 
		String element, 
		String ext,
		PathBuilder builder ) {
		super( "Default", title, comment );
		panel = new UseDefaultPanel( element, ext, builder );
		getContentPane().add( panel );
	}

	public void setDelegateForRoots( UseDefaultRootBuilder builder ) {
		panel.setUseDefaultRootBuilder( builder );
	}
	
	protected Dimension getDefaultSize() { 
		return new Dimension( 350, 130 ); 
	}

	public void setRoot( String tag) {
		panel.tfRoot.setSelectedItem( tag );
	}
	
	public void disableRoot() {
		panel.tfRoot.setEnabled( false );
	}

	public void disableRelativePath() {
		panel.relPathCb.setSelected( false );
		panel.relPathCb.setEnabled( false );
	}
	
	public void setFileLocation( String loc ) {
		panel.ftfFile.setFilePath( loc );
	}
	
	public void setDefaultDirectoryForLocation( String defLoc ) {
		panel.ftfFile.setCurrentDirectory( defLoc );
	}
	
	public String getDefaultDirectoryForLocation() {
		return panel.ftfFile.getCurrentDirectory();
	}
	
	public String getRoot() {
		String _ = ( String )panel.tfRoot.getSelectedItem();
		if ( "".equals( _ ) )
			return null;
		return _;
	}

	private String currentXMLLocation = null;
	
	public void setXMLFileLocation( String location ) {
		currentXMLLocation = location;
		if ( currentXMLLocation == null )
			disableRelativePath();
	}

	public String getFileLocation() {
		if ( panel.relPathCb.isSelected() ) {
			if ( currentXMLLocation != null ) {
				if ( panel.ftfFile.getText() != null )
				return com.japisoft.framework.app.toolkit.Toolkit.getRelativePath(
					new File( panel.ftfFile.getText() ), new File( currentXMLLocation ) );
			}
		}
		return panel.ftfFile.getText();
	}

	public String toURILocation() {
		String s = getFileLocation();

		File f = new File( s );
		if ( f.exists() ) {
			return f.toString();
		}
		return s;
	}

}
