package com.japisoft.editix.action.xml;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.editix.project.ProjectManager;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.editix.ui.pathbuilder.RNGPathBuilder;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class UseRelaxNGAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;
		UseDefaultDialog udd = new UseDefaultDialog( 
				"RelaxNG", 
				"Use an external RelaxNG document. Please use a project for maintaining the binding", "RelaxNG", "rng", new RNGPathBuilder() );
		udd.disableRoot();
		udd.disableRelativePath();
		String path = null;
		
		if ( container.getSchemaAccessibility() != null && 
				container.getSchemaAccessibility().getRelaxNGValidationLocation() != null ) {
			path = container.getSchemaAccessibility().getRelaxNGValidationLocation().location;
		}

		if ( path == null ) {
			path = Preferences.getPreference( "xmlconfig", "defaultRelaxNG", ( String )null );			
		}

		udd.setFileLocation( path );		
		udd.setVisible( true );

		if ( udd.isOk() ) {
			if ( udd.getFileLocation() != null ) {
				container.setProperty( ProjectManager.RELAXNG_PROPERTY, udd.getFileLocation() );
				container.getSchemaAccessibility().setRelaxNGValidationLocation(
					udd.getFileLocation() );
				Preferences.setPreference( "xmlconfig", "defaultRelaxNG", udd.getFileLocation() );
				EditixFactory.buildAndShowInformationDialog( "<html><body>Your schema has been assigned<br><em>Store your XML document in a project for saving the binding</em></body><html>" );
			}
		}
	}

}
