package com.japisoft.editix.action.xml.format;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.job.BasicJob;
import com.japisoft.framework.job.Job;
import com.japisoft.framework.job.JobManager;
import com.japisoft.framework.ui.toolkit.BrowserCaller;
import com.japisoft.framework.xml.format.Formatter;
import com.japisoft.framework.xml.format.FormatterConfig;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.refactor2.AbstractRefactor;
import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FormatAction extends AbstractAction {
	
	protected FormatterConfig getFormatterConfig() {
		return new FormatterConfig();
	}

	protected AbstractRefactor getRefactor() {
		return null;
	}

	public static AbstractRefactor LAST_REFACTOR = null;

	public void actionPerformed(ActionEvent e) {

		XMLContainer container = 
			EditixFrame.THIS.getSelectedContainer();
		if ( container == null ) {
			EditixFactory.buildAndShowErrorDialog( "No document" );
			return;
		}

		format( container, getRefactor(), getFormatterConfig(), (String)getValue( "param" ) );
		
	}

	
	public static void format( 
			XMLContainer container, 
			AbstractRefactor ar, 
			FormatterConfig config,
			String silenceParamValue ) {
	
		String content = container.getAccessibility().getText();
		try {

			if ( ar != null ) {
				
				if ( Manager.isFree() ) {
					EditixFactory.buildAndShowInformationDialog( "This action is not available inside the Free Edition.\nPlease look at http://www.editix.com" );
					BrowserCaller.displayURL( "http://www.editix.com" );
					return;					
				}
				
				LAST_REFACTOR = ar;				
			}

			if ( config == null )
				config = new FormatterConfig();
			
			String location = null;
			
			FPNode node = 
				container.getCurrentNode();
			
			if ( node != null )
				location = node.getXPathLocation();
			
			content = Formatter.format(
					content, 
					null, 
					config, 
					ar );

			container.getAccessibility().setText( content );
			container.setModifiedState( true );
	
			// Restore the current location
			
			if ( location != null ) {
				
				JobManager.addJob(
						new RestoreLocationJob( container, location ) );

			}
			
		} catch (Throwable e1) {
			ApplicationModel.debug( e1 );
			if ( !"".equals( e1.getMessage() ) && !"silence".equals( silenceParamValue ) )	// Silent exception
				EditixFactory.buildAndShowErrorDialog( "Can't format : " + e1.getMessage() );
		}
	
	}
	
	static class RestoreLocationJob extends BasicJob {
		
		private XMLContainer container;
		private String location;
		
		RestoreLocationJob( XMLContainer container, String location ) {
			this.container = container;
			this.location = location;
		}
		
		public void dispose() {
			container = null;
		}

		public Object getSource() {
			return null;
		}

		public boolean isAlone() {
			return false;
		}

		public void stopIt() {
		}

		public void run() {
			try {
				
				FPNode rootNode = ( FPNode )container.getTree().getModel().getRoot();
				if ( rootNode != null ) {
					FPNode node = rootNode.getNodeForXPathLocation( location, true );
					if ( node != null ) {
						try {
							container.getEditor().setCaretPosition( node.getStartingOffset() + 1 );
						} catch( IllegalArgumentException exc ) {
						}
					}
				}

			} finally {
				dispose();
			}
		}
		
	}
	
}
