package com.japisoft.editix.action.xsl;

import java.awt.event.ActionEvent;
import java.io.File;
import java.net.MalformedURLException;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.BadLocationException;

import com.japisoft.editix.toolkit.Toolkit;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AssignXSLTAction extends AbstractAction {

	public void actionPerformed(ActionEvent e) {
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if ( container == null )
			return;

		JFileChooser chooser = new JFileChooser();

		chooser.setFileFilter(new FileFilter() {
			public boolean accept(File f) {
				return f.isDirectory() || f.toString().toLowerCase().endsWith( ".xsl" );
			}
			public String getDescription() {
				return "XSLT (*.xsl)";
			}
		});

		if ( chooser.showOpenDialog( EditixFrame.THIS )
			== JFileChooser.APPROVE_OPTION ) {
			XMLPadDocument doc = container.getXMLDocument();
			int next = Math.max( 0, doc.nextTag( 0 ) - 1 );
			try {
				
				String href = null;
				if ( container.getCurrentDocumentLocation() != null ) {
					href = com.japisoft.framework.app.toolkit.Toolkit.getRelativePath(
							chooser.getSelectedFile(),
							new File( container.getCurrentDocumentLocation() ) );
				} else 
					href = Toolkit.toURL( chooser.getSelectedFile() ); 
				
				doc.insertString( next, "<?xml-stylesheet type=\"text/xsl\" href=\"" + href + "\"?>", null );
			} catch( BadLocationException exc ) {
			} catch( MalformedURLException exc ) {
			}
		}
	}

}
