package com.japisoft.editix.action.xsl;

import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;

import com.japisoft.editix.wizard.document.JSON2XMLDocument;
import com.japisoft.framework.toolkit.FileToolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class StreamSourceFactory {

	private StreamSourceFactory() {
	}
	
	private static StreamSourceFactory INSTANCE = null;

	public static StreamSourceFactory Instance() {
		if ( INSTANCE == null )
			INSTANCE = new StreamSourceFactory();
		return INSTANCE;
	}
	
	public StreamSource getStreamSource( String filePath ) throws Throwable {
		String type = FileToolkit.fileExt( filePath );
		if ( "json".equals( type ) || "jso".equals( type ) ) {
			// Convert it to XML first
			Document doc = JSON2XMLDocument.JSONTODOM( new File( filePath ) );
			StringWriter sw = new StringWriter();
			TransformerFactory.newInstance().newTransformer().transform( new DOMSource( doc ), new StreamResult( sw ) );
			return new StreamSource( new StringReader( sw.toString() ) );
		}
		return new StreamSource( filePath );
	}
	
}
