package com.japisoft.editix.db;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class AbstractNodeDb implements TreeNode, NodeDb {

	protected String url = null;
	protected String user = null;
	protected String password = null;

	protected ArrayList children = null;
	protected MutableTreeNode parent = null;
	
	public void setParent(NodeDb parent) {
		this.parent = parent;
	}	

	public void setParent(MutableTreeNode newParent) {
		this.parent = newParent;
	}	
	
	public void addChild( NodeDb node ) {
		if ( children == null )
			children = new ArrayList();
		node.setParent( this );
		
		// Order it
		
		String name = node.toString();
		boolean added = false;
		
		for ( int i = 0; i < children.size(); i++ ) {
			
			NodeDb n = ( NodeDb )children.get( i );
			if ( n.toString().compareTo( name ) > 0 ) {
			
				children.add( i, node );
				added = true;
				break;
				
			}
			
			
		}
	
		if ( !added )
			children.add( node );
	}

	public Enumeration children() {
		checkOpen();
		if ( children == null )
			return null;
		return Collections.enumeration( children );
	}

	public boolean getAllowsChildren() {
		return true;
	}

	public TreeNode getChildAt(int childIndex) {
		checkOpen();
		return ( TreeNode )children.get( childIndex );
	}

	private boolean open = false;
	private boolean error = false;
	private String errorMessage = null;
	
	private void checkOpen() {
		if ( !open ) {
			if ( children == null ) {
				try {
					open();
				} catch (Exception e) {
					error = true;
					errorMessage = e.getMessage();
				}
			}
			open = true;
		}
	}
	
	public int getChildCount() {
		checkOpen();
		if ( children == null )
			return 0;
		return children.size();
	}

	public int getIndex(TreeNode node) {
		if ( children == null )
			return -1;
		return children.indexOf( node );
	}

	public TreeNode getParent() {
		return parent;
	}

	public boolean isLeaf() {
		checkOpen();
		return 
			( children == null ) || 
				( children.size() == 0 );
	}

	public void close() {
		open = false;
		children = null;
		errorMessage = null;
		error = false;
	}

	public void open() throws Exception {
	}
	
	public String getErrorMessage() {
		return errorMessage;
	}

	public boolean hasError() {	
		return error;
	}
	
	public void setUserObject(Object object) {
	}
	
	public void insert(MutableTreeNode child, int index) {
	}
	public void remove(int index) {
	}
	public void remove(MutableTreeNode node) {
		if ( children != null )
			children.remove( node );
	}
	public void removeFromParent() {
	}

	protected String driverName;
	
	public void setDriverName( String driverName ) {		
		this.driverName = driverName;
	}

	public String getDriverName() {
		return driverName;
	}
	
	public boolean canBeDeleted() {		
		return true;
	}

	public boolean delete() throws Exception {
		if ( parent != null ) {
			close();
			parent.remove( this );
			return true;
		} else
			return false;
	}
	
	protected String name;

	protected void prepareName() {}
	
	public String toString() {
		if ( name == null )
			prepareName();
		return name;
	}

	public void refresh() throws Exception {
		// Go into
		for ( int i = 0; i < getChildCount(); i++ ) {
			NodeDb node = ( NodeDb )getChildAt( i );
			node.close();
		}
		// Remove children link
		children = null;
		open();
	}
	
}
