package com.japisoft.editix.db;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class Driver {
	protected String name;
	public String url;
	public String jars;
	public String dbxmlClass;

	public boolean canAddContainer = false;
	public boolean canRemoveContainer = false;
	public boolean query = false;
	public boolean embedded = false;
	public boolean system = false;
	
	public Driver( String name ) {
		this.name = name;
	}
	
	public Driver( 
			FPNode driverNode ) {
		this.name = driverNode.getAttribute( "name" );
		this.url = driverNode.getAttribute( "url" );
		this.jars = driverNode.getAttribute( "jars" );
		this.system = "true".equals( driverNode.getAttribute( "system" ) );
		this.dbxmlClass = driverNode.getAttribute( "xmldbDriver" );
		
		String home = System.getProperty( "user.home" );
		home = home.replace( '\\', '/' );

		this.url = this.url.replaceFirst( "\\$H", home );
		
		this.embedded = "true".equals( 
				driverNode.getAttribute( "embedded", "false" ) );

		this.canAddContainer = "true".equals( 
				driverNode.getAttribute( 
						"addc", 
							"false" ) );

		this.canRemoveContainer = "true".equals( 
				driverNode.getAttribute( 
						"removec", 
							"true" ) );
		
		this.query = "true".equals( 
				driverNode.getAttribute( 
						"query", 
							"false" ) );
		
	}

	private boolean install = false;

	public void install() {
		if ( !install ) {
			install = true;

			if ( jars != null ) {
			
				String[] drivers = jars.split( ";" );
				//URL root = ClassLoader.getSystemResource( "/dbdrivers" );
				URL[] res = new URL[ drivers.length ];

				for ( int i = 0;i < drivers.length; i++ ) {

					// Convert absolute path to URL
					File f = new File( drivers[ i ] );
					try {

						if ( f.exists() ) {
							res[ i ] = f.toURI().toURL();
						} else
							res[ i ] = new URL( drivers[ i ] );

					} catch ( MalformedURLException e ) {
						
						EditixApplicationModel.debug( e );

					}

				}

				URLClassLoader loader = new URLClassLoader( res );
				
				install( loader );
			
			} else
				
				install( ClassLoader.getSystemClassLoader() );
		}
	}

	protected void install( ClassLoader loader ) {
	}

	public boolean test( String url, String user, String password ) throws Exception {
		install();
		return false;
	}

	public String toString() {
		return name;
	}

	public abstract RootNodeDb getRoot( String url, String user, String password ) throws Exception;

	public String toXml() {
		String res = "<driver system='" + 
			system + 
			"' name='" + 
			name + 
			"' class='" + 
			getClass().getName() +
			"' url='" +
			url +
			"' xmldbDriver='" + 
			dbxmlClass + 
			"'";
		if ( jars != null ) {
			res += " jars='" + jars + "'"; 
		} 
		res +=
			" addc='" +
			canAddContainer +
			"' removec='" + 
			canRemoveContainer +
			"' query='" + 
			query + "'/>";
		return res;
	}

}
