package com.japisoft.editix.db;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import com.japisoft.editix.db.xmldb.XmlDbDriver;
import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.xml.parser.FPParser;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DriverDbManager {

	private static ArrayList drivers = null;

	public static int getDriverCount() {
		install();
		if ( drivers == null )
			return 0;
		return drivers.size();

	}
	
	public static Driver getDriver( int index ) {
		if ( drivers == null )
			return null;
		return ( Driver )drivers.get( index );
	}
	
	public static void removeDriver( Driver d ) {
		if  ( drivers != null )
			drivers.remove( d );
	}
	
	public static void addDriver( Driver d ) {
		if ( drivers != null )
			drivers.add( d );
	}

	public static Driver getDriverByName( String name ) {
		install();
		for ( int i = 0; i < getDriverCount(); i++ ) {
			if ( getDriver(  i ).toString().equalsIgnoreCase( name ) )
				return getDriver( i );
		}
		return null;
	}
	
	private static boolean isInstalled = false;
	
	public static void save() {
		File currentOne = new File( 
				EditixApplicationModel.getAppUserPath(),  
				"dbdrivers.xml" );

		StringBuffer sb = new StringBuffer();
		sb.append( "<drivers>" );

		for ( int i = 0; i < DriverDbManager.getDriverCount(); i++ ) {
			sb.append( DriverDbManager.getDriver( i ).toXml() );
		}

		sb.append( "</drivers>" );
		try {
			FileWriter fw = new FileWriter( currentOne );
			try {
				fw.write( sb.toString() );
			} finally {
				fw.close();
			}
		} catch (IOException e) {
		}
	}

	public static void install() {
	
		if ( isInstalled )
			return;

		isInstalled = true;

		try {
			InputStream input = null;
			
			File currentOne = new File( EditixApplicationModel.getAppUserPath(),  "dbdrivers.xml" ); 
			if ( !currentOne.exists() ) {
				input = 
					ClassLoader.getSystemClassLoader().getResourceAsStream( 
					"dbdrivers/drivers.xml" );
				if ( input == null )
					input = 
						ClassLoader.getSystemClassLoader().getResourceAsStream( "drivers.xml" );
			}
			else
				input = new FileInputStream( currentOne );
			
			if ( input == null ) {
				System.out.println( "Can't find drivers.xml" );
				return;
			}

			FPParser p = new FPParser();
			Document doc = p.parse(input);
			FPNode root = ( FPNode )doc.getRoot();
			for ( int i = 0; i < root.childCount(); i++ ) {
				FPNode c = root.childAt( i );
				if ( c.matchContent( "driver" ) ) {
					Driver d = null;
					
					String driverClass = c.getAttribute( "class" );
					if ( driverClass != null ) {

						try {
							ApplicationModel.debug( "Loading " + driverClass );
							
							if ( "com.japisoft.editix.db.xmldb.XmlDbDriver".equals( driverClass ) ) {
								d = new XmlDbDriver( c );
							} else
								d = ( Driver )Class.forName( driverClass ).getConstructors()[ 0 ].newInstance( new Object[] { c } );

							if ( drivers == null )
								drivers = new ArrayList();
							drivers.add( d );
						} catch ( Exception e ) {
							ApplicationModel.debug( e );
						}
 					} else
 						System.out.println( "Can't find driver class ???" );
				}
			}

		} catch (Exception e) {
			ApplicationModel.debug( e );
		}

	}

}
