package com.japisoft.editix.db.berkley;

import com.japisoft.editix.db.AbstractNodeDb;
import com.japisoft.editix.db.ContainerNodeDb;
import com.japisoft.editix.db.NodeDb;
import com.japisoft.editix.ui.EditixFactory;
import com.sleepycat.dbxml.XmlContainer;
import com.sleepycat.dbxml.XmlDocument;
import com.sleepycat.dbxml.XmlException;
import com.sleepycat.dbxml.XmlResults;
import com.sleepycat.dbxml.XmlValue;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BerkleyContainerDbImpl extends AbstractNodeDb implements ContainerNodeDb {

	protected XmlContainer container;
	
	public BerkleyContainerDbImpl( XmlContainer container ) {
		this.container = container;
	}

	public boolean canCreateSubContainer() {
		return false;
	}

	public ContainerNodeDb createSubContainer(String id) throws Exception {
		return null;
	}

	public String getContent(String id) throws Exception {		
		return container.getDocument( id ).getContentAsString();
	}

	public boolean delete( String docId ) throws Exception {
		if ( EditixFactory.buildAndShowConfirmDialog( "This operation may crash EditiX due a problem with the berkley API, are you sure ?" ) ) {
			container.deleteDocument( docId );
			return true;
		} else
			return false;
	}

	public NodeDb[] request(String query ) throws Exception {
		return ( ( BerkleyRootDbImpl )getParent() ).request( query );
	}

	public void setContent(String id, String content) throws Exception {
		container.putDocument( id, content );
	}
	
	public void setContent( XmlDocument doc ) throws Exception {
		container.updateDocument( doc );
		container.sync();
	}

	public void open() throws Exception {
		XmlResults res = container.getAllDocuments( null );
		if ( res != null ) {
			while ( res.hasNext() ) {
				XmlValue val = res.next();			
				XmlDocument doc = val.asDocument();
				addChild( 
						new BerkleyFileNodeImpl( 
								doc ) );
			}
			// res.delete();
		}
	}	

	public void close() {
		try {
			container.close();
		} catch (XmlException e) {
		}
		super.close();
	}

	public void refresh() throws Exception {
		
	}

}
