package com.japisoft.editix.document.xslfo;

import java.io.StringReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;

import com.japisoft.editix.action.fop.EditixFOPFactory;
import com.japisoft.framework.xml.XSLTTransformer;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.xml.validator.Validator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FOValidator implements Validator {

	public int validate(XMLContainer container, boolean silentMode) {
		
		container.getErrorManager().initErrorProcessing();
		try {
		
			try {
				FopFactory factory = EditixFOPFactory.newInstance( container.getCurrentDocumentLocationURL() );
				Fop fop = factory.newFop( MimeConstants.MIME_PLAIN_TEXT, new ByteArrayOutputStream() );
				TransformerFactory factory2 = XSLTTransformer.getTransformerFactory();
				Transformer transformer = factory2.newTransformer(); // identity transformer
				Result res = new SAXResult(fop.getDefaultHandler());
				transformer.transform( new StreamSource( new StringReader( container.getText() )), res);
			} catch( Exception exc ) {
				exc.printStackTrace();
				String message = exc.getMessage();
				
				if ( message != null && !"".equals( message ) ) {
					Pattern p = Pattern.compile( "position\\s(\\d+):(\\d+)" );
					Matcher m = p.matcher( message );			
					int row = -1;
					int col = -1;

					if ( m.find() ) {
						row = Integer.parseInt( m.group( 1 ) );
						col = Integer.parseInt( m.group( 2 ) );
					}
	
					container.getErrorManager().notifyError(
						this,
						false,
						container.getCurrentDocumentLocation(),
						row,
						col,
						-1,
						message,
						false );	
				}

				return ERROR;
			}
			return OK;
			
		} finally {
			container.getErrorManager().stopErrorProcessing();
		}
	}

}
