package com.japisoft.editix.document.xslfo;

import java.util.Enumeration;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.walker.NodeNameCriteria;
import com.japisoft.framework.xml.parser.walker.OrCriteria;
import com.japisoft.framework.xml.parser.walker.TreeWalker;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FoMasterReferenceHandler extends AbstractHelperHandler {

	protected void installDescriptors( 
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String addedString ) {	
		XMLContainer container = document.getContainer();
		if ( container == null )
			return;
		FPNode root = container.getRootNode();
		if ( root == null )
			return;
		if ( addedString == null )
			addedString = "";		
		// Search for page master-name
		TreeWalker tw = new TreeWalker( root );
		FPNode node = tw.getFirstTagNodeByName( "layout-master-set", false );
		if ( node != null ) {
			tw = new TreeWalker( node );
			Enumeration e = tw.getNodeByCriteria( new OrCriteria(
					new NodeNameCriteria( "simple-page-master" ),
					new NodeNameCriteria( "page-sequence-master" ) ), false );
			while ( e.hasMoreElements() ) {
				FPNode n = ( FPNode )e.nextElement();
				String mr = n.getAttribute( "master-name" );
				if ( mr != null ) {
					addDescriptor(
							new BasicDescriptor( 
									addedString + 
									mr + 
									addedString ) );
				}
			}
		}
	}

	protected String getActivatorSequence() {
		return null;
	}

	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset,
			String activatorString ) {
		if ( activatorString == null ) {
			return match(
				document, 
				offset, 
				"", 
				"master-reference=\"" );
		} else {
			if  ( "\"".equals( activatorString ) ) {
				return match(
						document, 
						offset, 
						"", 
						"master-reference=" );
			} else
				return false;
		}
	}

	public String getTitle() {
		return "Master reference page";
	}

	public int getPriority() {
		return 1;
	}

}
