package com.japisoft.editix.document.xslt;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class VariableParamNameHandler extends AbstractHelperHandler {

	protected void installDescriptors( 
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String addedString ) {	
		XMLContainer container = document.getContainer();
		if ( container == null )
			return;

		if ( addedString == null )
			addedString = "";
		
		currentNode = container.getCurrentElementNode();
		if ( currentNode != null )
			orderedSearch( 
					addedString, 
					currentNode.getFPParent(), 
					currentNode );
	}

	private void orderedSearch( String addedString, FPNode parentNode, FPNode endChild ) {
		if ( parentNode != null ) {
			for ( int i = 0; i < parentNode.childCount(); i++ ) {
				FPNode child = parentNode.childAt( i );
				
				if ( child == endChild ) {
					orderedSearch( addedString, parentNode.getFPParent(), parentNode );
					break;
				}
				
				if ( child.matchContent( "variable" ) ||
						child.matchContent( "param" ) ) {
					
					if ( child.hasAttribute( "name" ) )
						addDescriptor( 
								new BasicDescriptor(
										addedString +
										child.getAttribute( "name" ) ) );
					
				}
			}
		}
	}

	protected String getActivatorSequence() {
		return "$";
	}

	public boolean haveDescriptors(FPNode currentNode,
			XMLPadDocument document, boolean insertBefore, int offset,
			String activatorString) {
		if ( "$".equals( activatorString ) ) {
			return document.isInsideAttributeValue( offset );
		} 
		return false;
	}

	public String getTitle() {
		return "variables or params";
	}

}
