package com.japisoft.editix.document.xslt;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.japisoft.editix.ui.xslt.XMLDataSourcePanel;
import com.japisoft.editix.ui.xslt.XSLTEditor;
import com.japisoft.framework.collection.FastVector;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;

import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XPathNodeHandler extends AbstractHelperHandler {

	@Override
	protected String getActivatorSequence() {
		return null;
	}
	
	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset,
			String activatorString ) {
		return activatorString == null && document.isInsideAttributeValue( offset );
	}
	
	protected void installDescriptors(
			FPNode currentNode,
			XMLPadDocument document,
			int offset, 
			String activatorString ) {
		
		XMLContainer container = document.getContainer();
		XSLTEditor panel = ( XSLTEditor )container.getParentPanel();
		XMLDataSourcePanel sourcePanel = panel.getSourcePanel();
		FPNode rootNode = sourcePanel.getRootNode();
		if ( rootNode != null ) {
			String context = getXPathContext( currentNode );
			List<String> res = choice( rootNode, context );
			for ( String r : res ) {
				addDescriptor( new BasicDescriptor( r ) );
			}
		}		
	}	

	private List<String> choice( FPNode rootNode, String context ) {
		ArrayList<String> res = new ArrayList<String>();
		Document doc = rootNode.getDocument();
		List<FPNode> fv = doc.getFlatNodes();
		if ( fv != null ) {
			for ( int i = 0; i < fv.size(); i++ ) {
				FPNode node = ( FPNode )fv.get( i );
				if ( context == null ) {
					addResult( res, node );
				} else {
					if ( node.matchContent( context ) ) {
						// Attribute
						List<FPNode> attributes = node.getViewAttributeNodes();
						if ( attributes != null ) {
							for ( FPNode attNode : attributes ) {
								addResult( res, attNode );
							}
						}						
						// Children only
						for ( int j = 0; j < node.getChildCount(); j++ ) {
							addResult( res, ( FPNode )node.getChildAt( j ) );
						}
					}					
				}
			}
		}
		Collections.sort( res );
		return res;
	}

	private void addResult( List<String> res, FPNode node ) {
		if ( node.isTag() ) {
			if ( !res.contains( node.getContent() ) )
				res.add( node.getContent() );
		} else {
			if ( node.isAttribute() ) {
				if ( !res.contains( "@" + node.getContent() ) )
					res.add( "@" + node.getContent() );
			}
		}
	}

	private String getXPathContext( FPNode currentNode ) {
		String context = null;
		currentNode = currentNode.getFPParent();
		while ( currentNode != null ) {
			if ( currentNode.matchContent( "for-each" ) ) {
				context = currentNode.getAttribute( "select" );
				break;
			} else
			if ( currentNode.matchContent( "template" ) ) {
				context = currentNode.getAttribute( "match" );
				break;
			} else
				currentNode = currentNode.getFPParent();
		}
		if ( context != null ) {
			String[] tmp = context.split( "/" );
			for ( int i = 0; i < tmp.length; i++ ) {
				if ( !tmp[ i ].startsWith( "@" ) )
					context = tmp[ i ];
			}
			int i = context.lastIndexOf( "[" );
			if ( i > 0 )
				context = context.substring( 0, i );
		}
		if ( "/".equals( context ) )
			return null;
		return context;
	}

}
