package com.japisoft.editix.editor.css.helper;

import java.awt.Color;
import java.io.FileInputStream;
import java.io.IOException;

import com.japisoft.dtdparser.DTDParser;
import com.japisoft.dtdparser.node.ElementDTDNode;
import com.japisoft.dtdparser.node.RootDTDNode;
import com.japisoft.editix.document.DocumentModel;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.XMLDocumentInfo;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.handler.AbstractHelperHandler;
import com.japisoft.xmlpad.helper.model.BasicDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SelectorHandler extends AbstractHelperHandler {

	private RootDTDNode node = null;
	private boolean rejectIt = false;

	protected void installDescriptors( 
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String addedString ) {

		// Parse the HTML DTD
		XMLDocumentInfo info = 
			DocumentModel.getDocumentForType( "XHTML" );
		String HTMLDTD = info.getDefaultDTDLocation();
		if ( HTMLDTD.startsWith( "file:" ) )
			HTMLDTD = HTMLDTD.substring( 5 );
		if ( HTMLDTD.indexOf( ":" ) > -1 ) {
			if ( HTMLDTD.startsWith( "/" ) )
				HTMLDTD = HTMLDTD.substring( 1 );
		}

		DTDParser parser = new DTDParser();
		try {
			parser.parse( new FileInputStream( HTMLDTD ) );
			node = parser.getDTDElement();
		} catch (IOException e) {
			rejectIt = true;
		}
		
		if ( node != null ) {
				for ( int i = 0; i < node.getDTDNodeCount(); i++ ) {
					Color pink2 = Color.PINK.darker();
					if ( node.getDTDNodeAt( i ).isElement() ) {
						addDescriptor( new BasicDescriptor( ( ( ElementDTDNode )node.getDTDNodeAt( i ) ).getName() ) );						
					}
				}
		}
	}

	protected String getActivatorSequence() {
		return null;
	}
	
	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset,
			String activatorString ) {
		return activatorString == null && !PropertiesHandler.inSelector( document, offset );
	}

	public String getTitle() {
		return "Selector";
	}

	public int getPriority() {
		return -1;
	}

}
