package com.japisoft.editix.editor.css.kit;

import java.awt.Color;
import java.util.HashMap;

import com.japisoft.editix.editor.css.helper.Keywords;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SyntaxLexer {

	Color valueOfProperty = new Color( 150, 0, 0 );
	Color defaultColor = Color.BLACK;
	Color classColor = Color.ORANGE.darker();
	Color idColor = classColor;
	Color commentColor = Color.DARK_GRAY;
	
	SyntaxLexer( String preferenceGroup ) {
		Color c = new Color( 0, 150, 0 );
		c = Preferences.getPreference( preferenceGroup, "keywordsColor", c );				
		mapOfColor.put( "{", c );
		mapOfColor.put( "}", c );
		mapOfColor.put( ":", c );
		mapOfColor.put( ";", c );
		c = new Color( 0, 0, 150 );
		c = Preferences.getPreference( preferenceGroup, "propertiesColor", c );
		for ( int i = 0; i < Keywords.properties.length; i++ ) {
			mapOfColor.put( Keywords.properties[ i ], c );
		}
		commentColor = Preferences.getPreference( preferenceGroup, "commentColor", commentColor );		
		mapOfColor.put( "/*", commentColor );
		valueOfProperty = Preferences.getPreference( preferenceGroup, "valueColor", valueOfProperty );
		defaultColor = Preferences.getPreference( preferenceGroup, "defaultColor", defaultColor );
		classColor = Preferences.getPreference( preferenceGroup, "classColor", classColor );
		idColor = Preferences.getPreference( preferenceGroup, "idColor", idColor );
	}

	private int tokenCount;
	private Token[] maxTokens = new Token[ 100 ];

	public int getTokenCount() {
		return tokenCount;
	}

	private HashMap mapOfLastTokens = new HashMap();
	private HashMap mapOfColor = new HashMap();
	private StringBuffer sb = new StringBuffer();

	private void resetBuffer() {
		sb.delete( 0, sb.length() + 1 );
	}
	
	public Token[] getTokenForLine(String str, int line) {
		tokenCount = 0;
		
		Token previousLastToken = ( Token )mapOfLastTokens.get(
				new Integer( line - 1 ) 
		);
		Token currentLastToken = ( Token )mapOfLastTokens.get(
				new Integer( line ) 
		);

		resetBuffer();
		
		Token couldbeLastToken = null;
		Token mustBeLastToken = null;
		boolean checkForWord = false;
		boolean commentMode = false;
		boolean cssValueMode = false;
		
		if ( previousLastToken != null && 
				previousLastToken.color == Color.DARK_GRAY )
			commentMode = true;

		for ( int i = 0; i < str.length(); i++ ) {

			char c = str.charAt( i );

			if ( !commentMode && i > 0 ) {
				// Comment opening flag
				if ( c == '*' && 
						str.charAt( i - 1 ) == '/' ) {

					sb.deleteCharAt( sb.length() - 1 );
					addToken( sb.toString() );
					resetBuffer();
					addToken( "/*" );
					commentMode = true;
					continue;
					
				}
			}

			if ( commentMode ) {

				if ( c == '/' && i > 0 &&
					str.charAt( i - 1 ) == '*' ) {
					// Comment closing flag
					sb.append( c );
					addToken( sb.toString(), Color.DARK_GRAY );
					// For a non commment closing at end
					mustBeLastToken = new Token( "", null ); 
					resetBuffer();
					commentMode = false;
				} else
					sb.append( c );
					
				continue;
			}
			
			
			if ( !cssValueMode ) {
			
				if ( c == ':' ) {
					cssValueMode = true;
				}

			} else {
				
				if ( c == '{' ) {
					
					cssValueMode = false;
					
				} else {
				
					if ( c == ';' ) {
						cssValueMode = false;
						addToken( sb.toString(), valueOfProperty );
						resetBuffer();
						addToken( ";" );
						cssValueMode = false;				
					} else
						sb.append( c );
	
					continue;
					
				}
			}
			
			if ( !checkForWord ) {

				if ( c != ' ' && 
						c != '\t' && 
							c!= '\n' && 
								c != '\r' && 
									c!= ';' && 
										c != '{' && 
											c != ':' && 
												c != '}' && 
													c != '/' ) {

					addToken( sb.toString() );
					resetBuffer();

					sb.append( c );

					checkForWord = true;

				} else {
					
					if ( c == '{' || c == '}' || c == ':' || c == ';' ) {
						addToken( sb.toString() );
						resetBuffer();
						addToken( "" + c );
					}
					else
						sb.append( c );
				}

			} else {

				if ( c == ' ' || 
						c == '\t' ||
							c == '\n' ||
								c == '\t' ||
									c == '{' ||
										c == ':' ||
											c == ';' ||
												c == '}' || 
													c == ',' ) {

					couldbeLastToken = addToken( sb.toString() );
					resetBuffer();
					sb.append( c );
					checkForWord = false;

				} else
					sb.append( c );

			}

			if ( couldbeLastToken != null )
				mustBeLastToken = couldbeLastToken;
		}

		Color c = null;
		if ( commentMode )	// Force a gray
			c = Color.DARK_GRAY;

		couldbeLastToken = addToken( sb.toString(), c );

		if ( couldbeLastToken != null )
			mustBeLastToken = couldbeLastToken;

		if ( currentLastToken != null && 
				currentLastToken.color == Color.DARK_GRAY && 
					!commentMode ) {
			mustBeLastToken = new Token( "", null );
			/*mustBeLastToken.mustRepaint = true;
			System.out.println( "FORCE REPAINT !" + line ); */
		}

		if ( mustBeLastToken != null )
			mapOfLastTokens.put( 
					new Integer( line ), 
					mustBeLastToken );

		return maxTokens;
	}

	Token addToken( String token ) {
		return addToken( token, null );
	}
	
	Token addToken( String token, Color forceColor ) {
		Color c = forceColor;
		if ( c == null )
			c = ( Color )mapOfColor.get( token.toLowerCase() );
		
		if ( tokenCount == maxTokens.length ) {
			Token[] tmp = new Token[ maxTokens.length * 2 ];
			System.arraycopy(
					maxTokens,0,tmp,0,maxTokens.length
			);
			maxTokens = tmp;
		}

		Token t = null;

		// Not a keyword
		if ( c == null ) {
			
			c = defaultColor;

			// Class
			if ( token.length() > 1 ) {
				if ( token.startsWith( "." ) )
					c = classColor;
				else
				if ( token.startsWith( "#" ) )
					c = idColor;
			}
				
			
			t = new Token( token, c );
			maxTokens[ tokenCount++ ] = t;
			return null;
		}
		else {
			t = new Token( token, c );
			maxTokens[ tokenCount++ ] = t;
		}
		return t;
	}

	void dump() {
		System.out.println();
		for ( int i = 0; i < tokenCount; i++ ) {
			System.out.print( "[" + maxTokens[ i ] + "]" );
		}
	}

	public static void main( String[] args ) {
		String s = "a color : flsfksldfkjsdlkj;dddd";
		SyntaxLexer sl = new SyntaxLexer( "preferenceGroup" );
		sl.getTokenForLine( s, 0 );
		sl.dump();
	}
	
}
