package com.japisoft.editix.editor.html;

import java.awt.BorderLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.KeyStroke;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.ui.browser.Browser;
import com.japisoft.framework.ui.browser.BrowserFactory;
import com.japisoft.xmlpad.IXMLPanel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class HTMLPreview extends JPanel implements DocumentListener, ActionListener {

	private boolean autoRefresh = false;
	private IXMLPanel editor = null;
	private JCheckBox cb = null;
	private Browser browser = null;
	
	HTMLPreview( IXMLPanel editor ) {
		this.editor = editor;
		editor.getMainContainer().getEditor().getDocument().addDocumentListener( this );
		autoRefresh = Preferences.getPreference( "HTMLEditor", "auto-refresh", false );
		setLayout( new BorderLayout() );

		editor.getMainContainer().getEditor().getActionMap().put( "refresh", new RefreshAction() );
		editor.getMainContainer().getEditor().getInputMap().put( KeyStroke.getKeyStroke( KeyEvent.VK_F5, 0), "refresh" );

		JToolBar tb = new JToolBar();
		tb.add( editor.getMainContainer().getEditor().getActionMap().get( "refresh" ) );

		cb = new JCheckBox( "Auto refresh" );
		cb.setSelected( autoRefresh );
		tb.add( cb );

		add( tb, BorderLayout.SOUTH );
	}

	@Override
	public void addNotify() {
		super.addNotify();
		cb.addActionListener( this );
	}
	
	@Override
	public void removeNotify() {
		super.removeNotify();
		cb.removeActionListener( this );
	}

	public void actionPerformed(ActionEvent e) {
		autoRefresh = cb.isSelected();
	}

	public void dispose() {
		editor.getMainContainer().getEditor().getDocument().removeDocumentListener( this );
		editor = null;
	}

	public void changedUpdate(DocumentEvent e) {
		if ( autoRefresh ) {
			refresh();
		}
	}

	public void insertUpdate(DocumentEvent e) {
		if ( autoRefresh ) {
			refresh();
		}		
	}

	public void removeUpdate(DocumentEvent e) {
		if ( autoRefresh ) {
			refresh();
		}		
	}

	void refresh() {
		if ( browser == null ) {
			browser = BrowserFactory.getInstance().newBrowser();
			add( browser.getView(), BorderLayout.CENTER );
			invalidate();
			validate();
			repaint();
		}
				
		browser.setHTML( getHTMLPage(), null );
	}
	
	private String getHTMLPage() {
		String html = editor.getMainContainer().getText();
		if( editor.getMainContainer().getCurrentDocumentLocation() != null ) {
			int i = html.indexOf( "<head>" );
			if ( i > -1 ) {
				try {
					String uri = new File( 
						editor.getMainContainer().getCurrentDocumentLocation() ).getParentFile().toURL().toExternalForm();
					String baseInjector = "<base href='" + uri + "'>";
					html = html.substring( 0, i + 6 ) + baseInjector + html.substring( i + 6 );
				} catch( Exception exc ) {
				}
			}
		}
		
		if ( html.startsWith( "<?xml" ) ) {
			int i = html.indexOf( "?>" );
			if ( i > -1 )
				html = html.substring( i + 2 );
		}
		
		return html;
	}

	/////////////////////////////////////////////////////////////////////
	
	class RefreshAction extends AbstractAction {
	
		public RefreshAction() {
			putValue( 
				Action.SHORT_DESCRIPTION, 
				"Refresh the CSS Preview" 
			);
			putValue( 
				Action.SMALL_ICON, 
				new ImageIcon( 
					getClass().getResource( "refresh.png" )
				) 
			);
		}

		public void actionPerformed(ActionEvent e) {
			refresh();
		}
		
	}
	
}
