package com.japisoft.editix.editor.json.kit;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JSONDocument extends XMLPadDocument {
	
	private boolean autoIndent = false;
	
	public JSONDocument( String preferenceGroupe ) {
		super(null);
		autoIndent = Preferences.getPreference( preferenceGroupe, "autoIndent", true );
	}

	public void setAutoIndent( boolean value ) {
		this.autoIndent = value;
	}
	
	public boolean isAutoIndent() {
		return this.autoIndent;
	}

	@Override
	public void insertString(int offs, String str, AttributeSet a)
			throws BadLocationException {
		super.insertString(offs, str, a);
		if ( autoIndent ) {
			if ( "{".equals( str ) || 
					"[".equals( str ) ) {
				insertObjectArray( offs, str );
			}				
		}
	}

	public void insertObjectArray( int offs, String str ) throws BadLocationException {
		String close = "}";
		if ( "[".equals( str ) )
			close = "]";
		String indent = getIndentAtOffset(offs);			
		super.insertString(offs + 1, "\n" + indent + "\t\n" + indent + close, null );
		getCurrentEditor().setCaretPositionWithoutNotification( offs + indent.length() + 3 );		
	}

}
