package com.japisoft.editix.editor.json.parser;

import java.util.ArrayList;

import java.util.List;
import java.util.Stack;

import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JSONParser {

	public FPNode parse( String content ) {
		char[] chrs = content.toCharArray();
		return parse( chrs );
	}
	
	private Stack<FPNode> stacks = null;
	private FPNode current = null;
	private  int offset = 0;
	
	private FPNode parse( char[] content ) {
		stacks = new Stack<FPNode>();
		FPNode root = null;
		StringBuffer string = null;
		String lastString = null;
		int lastStringOffset = 0;
		FPNode node = null;
		
		while ( offset < content.length ) {
			char c = content[ offset ];
			switch( c ) {
				case '{' :
					node = new FPNode( FPNode.TAG_NODE, "object" );
					node.setApplicationObject( "object" );
					if ( root == null )
						root = node;
					stacks.push( node );
					break;
				case '}' :
					List<FPNode> res = new ArrayList<FPNode>();
					boolean closeLast = false;
					
					while ( stacks.size() > 0 ) {

						FPNode value = stacks.pop();
						FPNode property = stacks.pop();
						
						property.appendChild( value );
						res.add( property );
						
						if ( "object".equals( stacks.peek().getApplicationObject() ) )
								break;
						
						/*
						
						// Close the last property
						node = stacks.pop();
						if ( "property".equals( 
							node.getApplicationObject() ) && !closeLast ) {
							if ( res.size() > 0 ) {
								node.addNode( res.get( 0 ) );
								res.remove( 0 );
							}
							closeLast = true;
						}
						
						res.add( 0, node );
						
						*/
					}
					
					java.util.Collections.reverse( res );
					
					if ( stacks.size() > 0 ) {
						FPNode object = stacks.peek();
						object.stoppingOffset = offset;

						for( FPNode co : res ) {
							object.addNode( co );
						}
						if ( stacks.size() > 0 ) {
							// node = stacks.pop();
							// node.addNode( object );
							// stacks.push( node );
						}
					} else {
						// Error
					}
					break;
				case '[' :
					break;
				case ']':
					break;
				case ':':
					node = stacks.pop();
					if ( node.getType() == FPNode.TEXT_NODE ) {
						FPNode stringNode = node;
						node = new FPNode( FPNode.TAG_NODE, stringNode.getContent() );
						node.setApplicationObject( "property" );
						node.startingOffset = stringNode.startingOffset;
						node.stoppingOffset = offset;
						stacks.push( node );
					}
					break;
				case '"':
					if ( string == null ) {
						string = new StringBuffer();
						lastStringOffset = offset;
					}
					else
					{
						lastString = string.toString();
						node = new FPNode( FPNode.TEXT_NODE, lastString );
						node.startingOffset = lastStringOffset;
						node.stoppingOffset = offset + 1;
						stacks.push( node );
						string = null;
					}
					break;
				case ',' :
					/*
					FPNode value = stacks.pop();
					FPNode property = stacks.pop();
					property.addNode( value );
					stacks.push( property );
					*/
					break;
				default :
					if ( string != null ) {
						string.append( c );
					}
			}
			
			offset++;
		}
		
		return root;
	}
	
	public static void main( String[] args ) {
		
		JSONParser p = new JSONParser();
		FPNode root = p.parse( "{ \"aa\" : \"bb\", \"cc\" : { \"dd\" : \"ee\", \"ff\":\"gg\" }, \"hh\":\"ii\" }" );
		// FPNode root = p.parse( "{ \"aa\" : \"bb\", \"cc\" : { \"dd\" : \"ee\" }, }" );
		// FPNode root = p.parse( "{ \"aa\" : \"bb\", \"cc\" : \"dd\" }" );		
		System.out.println( root.getRawXML( 1 ) );
		
	}
	
}

