package com.japisoft.editix.editor.jsx;

import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.text.EditorKit;
import javax.swing.text.PlainDocument;

import com.japisoft.editix.editor.js.helper.CommentHandler;
import com.japisoft.editix.editor.js.helper.ObjectInScopeHandler;
import com.japisoft.editix.editor.js.helper.ParametersHandler;
import com.japisoft.editix.editor.js.kit.JSEditorKit;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xmlpad.ComponentFactory;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.editor.EditorContext;
import com.japisoft.xmlpad.editor.XMLEditor;

//.jsx
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JSXEditor extends XMLContainer {

	public JSXEditor() {
		super( new ComponentFactoryForCssEditor() );
		getUIAccessibility().setToolBarAvailable( false );
		setStatusBarAvailable( false );
		getUIAccessibility().setPopupAvailable( false );
		getUIAccessibility().setTreePopupAvailable( false );
		setAutoNewDocument( false );
		setErrorPanelAvailable( true );
		getDocumentInfo().setRealTimeTree( false );
		setDisposeAction(false);
		setTreeAvailable( false );
		
		getDocument().putProperty(PlainDocument.tabSizeAttribute,
				new Integer(Preferences.getPreference("file", "tab-size", 2)));
		
		ArrayList assistant = new ArrayList();
		assistant.add( new CommentHandler() );
		assistant.add( new ObjectInScopeHandler() );
		assistant.add( new ParametersHandler() );
		assistant.add( new DOMHandler() );
		getHelperManager().resetHandlers( assistant, false );
		
		getHelperManager().resetHandlers( assistant, false );
	}
	
	static class ComponentFactoryForCssEditor extends ComponentFactory {
		public XMLEditor getNewXMLEditor(EditorContext context) {
			return new CustomXMLEditorForCss( context );
		}
	}
	
	static class CustomXMLEditorForCss extends XMLEditor {
		CustomXMLEditorForCss(EditorContext context) {
			super(context);
		}
		public EditorKit getEditorKit() {
			return new JSEditorKit( "JSEditor" );
		}
	}
	
	public static void main( String[] args ) {
		ApplicationModel.SHORT_APPNAME = "test";
		JFrame f = new JFrame();
		f.add( new JSXEditor().getView() );
		f.setSize( 300, 300 );
		f.setVisible( true );
	}

}
