package com.japisoft.editix.editor.svg;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.util.Iterator;

import javax.swing.Action;
import javax.swing.JComponent;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.framework.dockable.InnerWindowProperties;
import com.japisoft.framework.dockable.JDock;

import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.XMLDocumentInfo;
import com.japisoft.xmlpad.bookmark.BookmarkContext;
import com.japisoft.xmlpad.tree.parser.Parser;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SVGContainer extends JDock implements IXMLPanel {

	private IXMLPanel editor = null;
	private SVGPreview preview = null;
	
	public SVGContainer() {
		editor = EditixFactory.buildNewContainer();	
		
		setLayout( new BorderLayout() );
		addInnerWindow( new InnerWindowProperties( "editor", "SVG Editor", editor.getView() ), BorderLayout.CENTER );
		preview = new SVGPreview( editor );
		preview.setPreferredSize( new Dimension( 0, 200 ) );
		addInnerWindow( new InnerWindowProperties( "preview", "SVG Preview", preview ), BorderLayout.SOUTH );
	}

	public void copy() {
		editor.copy();
	}

	public void cut() {
		editor.cut();
	}
	
	public Parser createNewParser( boolean lightweightMode ) {
		return null;
	}

	public IXMLPanel getPanelParent() {
		return null;
	}
	
	@Override
	public String getCurrentDocumentLocation() {
		return editor.getCurrentDocumentLocation();
	}
	
	public Action getAction(String actionId) {
		return editor.getAction( actionId );
	}

	public BookmarkContext getBookmarkContext() {
		return editor.getBookmarkContext();
	}

	public XMLContainer getMainContainer() {
		return editor.getMainContainer();
	}

	public Iterator getProperties() {
		return editor.getProperties();
	}

	public Object getProperty(String name, Object def) {
		return editor.getProperty( name, def );
	}

	public Object getProperty(String name) {
		return editor.getProperty( name );
	}

	public XMLContainer getSelectedContainer() {
		return editor.getMainContainer();
	}

	public XMLContainer getSubContainer(String type) {
		return null;
	}

	public XMLContainer getSubContainerAt(int index) {
		return null;
	}

	public int getSubContainerCount() {
		return 0;
	}

	public void paste() {
		editor.paste();
	}
	
	@Override
	public Object print() {
		return editor;
	}
	
	public void postLoad() {
		editor.postLoad();
	}

	public void prepareToSave() {
		editor.prepareToSave();
	}

	public boolean reload() {
		return editor.reload();
	}

	public void selectSubContainer(IXMLPanel panel) {
	}

	public void setAutoDisposeMode(boolean disposeMode) {
		editor.setAutoDisposeMode( disposeMode );
	}

	public void setDocumentInfo(XMLDocumentInfo info) {
		editor.setDocumentInfo( info );
	}

	public void setProperty(String name, Object content) {
		editor.setProperty( name, content );
	}

	@Override
	public void dispose() {
		super.dispose();
		preview.dispose();
		editor = null;
	}
	
	// --------------------------------------------------------------------------------------------------
	
	class CustomInnerPanel extends InnerPanel implements IXMLPanel {

		public IXMLPanel getPanelParent() {
			return SVGContainer.this;
		}
		
		public Parser createNewParser( boolean lightweightMode ) {
			return null;
		}		
		
		@Override
		public String getCurrentDocumentLocation() {
			return editor.getCurrentDocumentLocation();
		}		
		
		public void dispose() {
			SVGContainer.this.dispose();
		}

		public void setAutoDisposeMode(boolean disposeMode) {
			SVGContainer.this.setAutoDisposeMode(disposeMode);
		}

		public XMLContainer getMainContainer() {
			return SVGContainer.this.getMainContainer();
		}

		public XMLContainer getSubContainerAt(int index) {
			return null;
		}
		
		public int getSubContainerCount() {
			return 0;
		}
		
		public void selectSubContainer(IXMLPanel panel) {
		}
		
		public BookmarkContext getBookmarkContext() {
			return SVGContainer.this.getBookmarkContext();
		}

		public XMLContainer getSelectedContainer() {
			return SVGContainer.this.getSelectedContainer();
		}
		
		public void copy() {
			SVGContainer.this.copy();
		}
		
		public void cut() {
			SVGContainer.this.cut();
		}
		
		public void paste() {
			SVGContainer.this.paste();
		}

		@Override
		public Object print() {
			return SVGContainer.this.print();
		}
		
		public void setDocumentInfo(XMLDocumentInfo info) {
			SVGContainer.this.setDocumentInfo( info );
		}		
		
		public Action getAction(String actionId) {
			return SVGContainer.this.getAction( actionId );
		}

		public Iterator getProperties() {
			return SVGContainer.this.getProperties();
		}

		public Object getProperty(String name, Object def) {
			return SVGContainer.this.getProperty(name, def);
		}

		public Object getProperty(String name) {
			return SVGContainer.this.getProperty(name);
		}

		public XMLContainer getSubContainer(String type) {
			return null;
		}

		public void prepareToSave() {
			SVGContainer.this.prepareToSave();
		}	

		public void postLoad() {
			SVGContainer.this.postLoad();
		}

		public boolean reload() {
			return SVGContainer.this.reload();
		}		

		public JComponent getView() {
			return this;
		}

		public void setProperty(String name, Object content) {
			SVGContainer.this.setProperty( name, content );
		}
	}

	protected InnerPanel createInnerView() {
		return new CustomInnerPanel();
	}
	
	
}
