package com.japisoft.editix.editor.xsd.view.element;

import java.awt.BorderLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import org.w3c.dom.Element;

import com.japisoft.editix.editor.xsd.Changeable;
import com.japisoft.editix.editor.xsd.Factory;
import com.japisoft.editix.editor.xsd.toolkit.SchemaHelper;
import com.japisoft.editix.editor.xsd.view.View;
import com.japisoft.editix.editor.xsd.view.element.simpletype.SimpleTypeViewImpl;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ElementViewImpl extends JPanel implements View, Changeable {	
	private JTabbedPane tp = new JTabbedPane( JTabbedPane.BOTTOM );
	private PropertiesViewImpl av = new PropertiesViewImpl(); 
	private SimpleTypeViewImpl stv = null;

	private Factory factory = null;
	
	public ElementViewImpl( 
			Factory factory, 
			PropertiesViewListener listener ) {
		stv = new SimpleTypeViewImpl( factory );
		setLayout( new BorderLayout() );
		add( tp );
		tp.addTab( "Properties", 
				new JScrollPane( av.getView(), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER) );
		tp.addTab( "Simple type", 
				stv.getView() );
		av.setPropertiesViewListener( listener );
	}

	public void setDesignerMode( boolean designerMode ) {
		av.setDesignerMode( designerMode );
	}	

	public boolean isDesignerMode() {
		return av.isDesignerMode();
	}
			
	public void init( Element schemaNode ) {
		av.init( schemaNode );
		stv.init( schemaNode );
		
		
		if ( schemaNode != null ) {
			tp.setEnabledAt( 0, true );
			tp.setEnabledAt( 1, "element".equals( schemaNode.getLocalName() )
					|| "simpleType".equals( schemaNode.getLocalName() )
						|| "attribute".equals( schemaNode.getLocalName() ) );
			if ( !tp.isEnabledAt( 1 ) ) {
				if ( tp.getSelectedIndex() == 1 )
					tp.setSelectedIndex( 0 );
			}
			
			// Check for complexType case
			if ( "element".equals( schemaNode.getLocalName() ) ) {
				if ( SchemaHelper.hasChild( schemaNode, "complexType" ) ) {
					tp.setEnabledAt( 1, false );
				}
			}

		} else {
			tp.setSelectedIndex( 0 );
			tp.setEnabledAt( 0, false );
			tp.setEnabledAt( 1, false );
		}
	}
	
	public boolean isChanged() { 
		return av.isChanged() || stv.isChanged();
	}
	
	public JComponent getView() {
		return this;
	}

	public void dispose() {
		av.dispose();
		stv.dispose();
	}

	public void stopEditing() {		
		av.stopEditing();
	}
	
	@Override
	public void copy() {
	}
	@Override
	public void cut() {
	}
	@Override
	public void paste() {
	}
	
}
