package com.japisoft.editix.filedragging;

import java.io.File;
import java.util.List;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLEditor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSDFileDragging extends DefaultFileDragging {

	@Override
	public void drag(XMLEditor editor, List<File> files) {
		FPNode root = editor.getXMLContainer().getRootNode();
		boolean importMode = false;
		if ( root != null ) {
			if ( root.hasAttribute( "targetNamespace" ) ) {
				importMode = true;
			}
			
			String prefix = root.getNameSpacePrefix();
			if ( prefix != null && !"".equals( prefix ) )
				prefix += ":";
			else
				prefix = "";
			
			for ( File f : files ) {
				
				String path = null;
				if ( sameParent(f, editor) ) {
					path = f.getName();
				} else
					path = f.toString();

				if ( "xsd".equals( getFileExt( f ) ) ) {
					if ( importMode ) {
						editor.insertText("<" + prefix + "import schemaLocation=\"" + path + "\"/>" );
					} else {
						editor.insertText("<" + prefix + "include schemaLocation=\"" + path + "\"/>" );
					}
				}
			}
		} else
			super.drag( editor, files );
	}
	
}
