package com.japisoft.editix.main.steps;

import com.japisoft.editix.action.options.CheckVersionAction;
import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.ApplicationStep;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CheckNewVersionStep implements ApplicationStep, Runnable {

	public boolean isFinal() {
		return false;
	}

	public void start(String[] args) {
		if ( Preferences.getPreference( "interface", "checkNewVersion", true ) ) {
			new Thread( this ).start();
		}
	}
	
	public void stop() {}
	
	@Override
	public void quit() {
	}
	
	public void run() {
		try {
			String lastVersion = CheckVersionAction.getLastVersion();
			String currentVersion = EditixApplicationModel.getAppVersion();
			
			try {
				double dlastVersion = Double.parseDouble( lastVersion );
				double dcurrentVersion = Double.parseDouble( currentVersion );
				
				if ( dcurrentVersion < dlastVersion ) {
					ApplicationModel.fireApplicationValue( "information", "A new version " + lastVersion + " is available" );
				}
			} catch( NumberFormatException exc ) {
				ApplicationModel.fireApplicationValue( "information", "A new version " + lastVersion + " is available" );
			}
			
		} catch( Exception exc ) {
			ApplicationModel.debug( exc );
		}
	}

}
