package com.japisoft.editix.mapper.xslt;

import com.japisoft.editix.mapper.AbstractMapper;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CallersMapper extends AbstractMapper {

	public boolean canMap( FPNode node ) {
		if ( "call-template".equals( node.getContent() ) )
			return true;
		if ( "apply-templates".equals( node.getContent() ) )
			return true;
		return false;
	}
	
	private static String cleanXPath( String xpath ) {
		if ( xpath == null )
			return "null";
		if ( "/".equals( xpath ) )
			return "/";
		int i = xpath.lastIndexOf( "/" );
		if ( i > -1 ) {
			xpath = xpath.substring( i + 1 );
		}
		i = xpath.lastIndexOf( "[" );
		if ( i > -1 ) {
			xpath = xpath.substring( 0, i );
		}
		return xpath;
	}

	public static boolean matchSelect( String s1, String s2 ) {
		return cleanXPath( s1 ).equals( cleanXPath( s2 ) );
	}

	@Override
	protected boolean isMatchingNode(
		FPNode sourceNode,
		FPNode walkingNode) {
		boolean template = "template".equals( walkingNode.getContent() );
		if ( template ) {
			String name = sourceNode.getAttribute( "name" );
			if ( name != null ) {
				if ( name.equals( walkingNode.getAttribute( "name" ) ) )
					return true;
			} else {
				String mode = sourceNode.getAttribute( "mode" );
				if ( mode != null ) {
					if ( !mode.equals( walkingNode.getAttribute( "mode" ) ) )
						return false;
				}
				String mustMatch = sourceNode.getAttribute( "select" );
				
				if ( mustMatch != null ) {
					if ( mode == null && 
							walkingNode.hasAttribute( "mode" ) ) {
						return false;
					}
					String currentMatch = walkingNode.getAttribute( "match" );
					return matchSelect( mustMatch, currentMatch );
				}
			}
		}
		return false;
	}

	public String[] getMapAttributes() {
		return new String[] { "name", "mode", "select" };
	}	
	
	@Override
	public String toString() {
		return "Find template references";
	}

}
