package com.japisoft.editix.plugin;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.xmlpad.IXMLPanel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EditixDocumentModel {

	EditixDocumentModel() {
	}
	
	/** 
	 * @return The number of opened documents
	 */
	public int getDocumentCount() {
		return EditixFrame.THIS.getXMLContainerCount();
	}

	/**
	 * @param index A number between 0 and the number of opened documents - 1
	 * @return An access to the document at this index */
	public EditixDocument getDocument( int index ) {
		if ( index >= getDocumentCount() || index < 0 ) {
			throw new RuntimeException( "Invalid index, must be between [0-" + ( getDocumentCount() - 1 ) );
		}
		return new EditixDocument( index );
	}

	/**
	 * @return The current document or <code>null</code> if there's no current document
	 */
	public EditixDocument getCurrentDocument() {
		if ( EditixFrame.THIS.getCurrentXMLContainerIndex() == -1 )
			return null;
		return new EditixDocument( EditixFrame.THIS.getCurrentXMLContainerIndex() );
	}

	/**
	 * Create and add a new document
	 * @param type the document type XML, DTD, XSD, CSS...
	 * @return a new document
	 */
	public EditixDocument newDocument( String type ) {
		IXMLPanel c = EditixFactory.buildNewContainer( type, ( String )null );
		EditixFrame.THIS.addContainer( c );
		return new EditixDocument( getDocumentCount() - 1 );
	}
	
}
