package com.japisoft.editix.toolkit;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.TransferHandler;

import com.japisoft.editix.action.file.OpenAction;
import com.japisoft.editix.action.file.OpenProjectAction;
import com.japisoft.editix.document.DocumentModel;
import com.japisoft.editix.project.ProjectManager;
import com.japisoft.xmlpad.XMLDocumentInfo;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AddSystemFilesTransferHandler extends TransferHandler {
	
	public boolean canImport( JComponent comp, DataFlavor[] transferFlavors ) {			
		return transferFlavors[ 0 ].isFlavorJavaFileListType();
	}

	@Override
	public boolean canImport(TransferSupport support) {
		if (!support.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
            return false;
        }
        return true;
	}
	
	@Override
	public boolean importData(TransferSupport info) {
        if (!info.isDrop()) {
            return false;
        }

        // Check for FileList flavor
        if (!info.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
         
            return false;
        }

        // Get the fileList that is being dropped.
        Transferable t = info.getTransferable();
        
        try {
        	List<File> list = (List<File>)t.getTransferData(DataFlavor.javaFileListFlavor);
			for ( int i = 0; i < list.size(); i++ ) {
				String filePath = list.get( i ).toString();
				
				if ( filePath.endsWith( ".pre" ) ) {
					OpenProjectAction.openProject( false, filePath );
				}
				
				XMLDocumentInfo info2 = DocumentModel.getDocumentByFileName( filePath );
				OpenAction.openFile( info2.getType(), false, filePath, null, null );
			}
			return list.size() > 0;            
        }
        catch (Exception e) { return false; }

	}

	/*
	public boolean importData(JComponent arg0, Transferable arg1) {
        if (!arg1.isDrop()) {
            return false;
        }
		
		try {
			java.util.List list = ( java.util.List )arg1.getTransferData(
					DataFlavor.javaFileListFlavor
			);
			if ( list != null ) {
			} 
		}
		catch (UnsupportedFlavorException e) {}
		catch( IOException e ) {}
		return false;
	}
	*/

}
