package com.japisoft.editix.ui;

import java.io.File;
import java.util.Timer;
import java.util.TimerTask;

import com.japisoft.editix.main.EditixApplicationModel;
import com.japisoft.framework.xml.XMLFileData;
import com.japisoft.framework.xml.XMLToolkit;
import com.japisoft.xmlpad.IXMLPanel;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FileChangeChecker extends TimerTask {

	static Timer t = null;
	
	public static void start() {
		t = new Timer();
		t.schedule( 
				new FileChangeChecker(), 2000, 4000 
		);
	}

	public static void stop() {
		if ( t != null )
			t.cancel();
	}
	
	private int errorCounter = 0;

	public void run() {
		try {
			for ( int i = 0; i < EditixFrame.THIS.getXMLContainerCount(); i++ ) {
				IXMLPanel panel = EditixFrame.THIS.getIXMLPanel( i );
				if ( panel == null )
					continue;
				XMLContainer container = panel.getMainContainer();
				if ( container == null )
					continue;
				
				if ( container.hasProperty( "save.delegate" ) ) 
					continue;
				
				if ( container.hasProperty( "file.checker.ignore" ) )
					continue;
				
				String s = container.getCurrentDocumentLocation();
				if ( s != null ) {
					if ( s.indexOf( "://" ) == -1 ) {	
						File f = new File( s );
						long md = f.lastModified();
						if ( md != container.getDocumentInfo().getCurrentDocumentModifiedDate() ) {
							// Recharger le fichier ???
							if ( EditixFactory.buildAndShowConfirmDialog( "The file " + s + " has been modified, reload it ?" ) ) {
								if ( !panel.reload() ) {
									EditixFactory.buildAndShowErrorDialog( "Can't reload " + s );
								}
							}
							container.getDocumentInfo().setCurrentDocumentModifiedDate( md );
						}
					}
				}
			}
		} catch (RuntimeException e) {
			// This code mustn't create editix error
			EditixApplicationModel.debug( e );
			errorCounter++;
			if ( errorCounter > 5 ) // No more running, too dangerous
				stop();
		}
	}

}
