package com.japisoft.editix.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.awt.GraphicsDevice.WindowTranslucency;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;

import com.japisoft.p3.Manager;
import com.japisoft.xmlpad.XMLContainer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class Splashscreen extends JWindow {

	private JProgressBar pb = null;

	private Splashscreen() {
		super();
		
		try {
			GraphicsEnvironment ge = 
	            GraphicsEnvironment.getLocalGraphicsEnvironment();
	        GraphicsDevice gd = ge.getDefaultScreenDevice();
	        final boolean isTranslucencySupported = 
	            gd.isWindowTranslucencySupported(WindowTranslucency.TRANSLUCENT);		
	        if ( isTranslucencySupported ) {
	        	setOpacity( 0.9f );
	        }
		} catch( Throwable th ) {}
		
		JLabel l = ( JLabel )getContentPane().add(
				new JLabel(EditixFactory.getImageIcon("images/logo.png")));

		getContentPane().add( pb = new JProgressBar( ), BorderLayout.SOUTH );
		
		pb.setBorderPainted( false );
		pb.setForeground( new Color( Integer.parseInt( "3D4B85", 16 ) ) );
		
		if ( !Manager.isFree() ) {		
			int day = Manager.registeredDay();
			l.setBackground( Color.white );
			if (day > -1) {
				JLabel lbl = null;
				getContentPane().add(
					lbl = new JLabel(
						"Evaluation version : This version will expire in "
							+ Math.max( (Manager.compute( 20, 10 ) - day), 0 )
							+ " days"),
					BorderLayout.NORTH );
				lbl.setOpaque( false );
			} 
		} else {
			getContentPane().add(
					new JLabel(
						"Free Edition" ),BorderLayout.SOUTH );
		}

		pack();
	}

	static Splashscreen ss = null;

	public static void start() {
		if (ss == null) {
			ss = new Splashscreen();
			adapter = new SplashAdapter();
		}
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		ss.setLocation(
			(dim.width - ss.getWidth()) / 2,
			(dim.height - ss.getHeight()) / 2);
		ss.setBackground( Color.white );
		ss.setVisible(true);
	}
	
	public static void progress( int indice, int max ) {
		ss.pb.setMaximum( max );
		ss.pb.setValue( indice );
	}

	public static void restart() {
	}

	public static void stopNow() {
		try {
			ss.setVisible(false);
			ss.dispose();
		} catch (Throwable th) {
		}
	}

	static void hideSplashScreen() {
		ss.removeWindowListener(adapter);
		ss.setVisible(false);
		ss.dispose();
		adapter = null;
		ss = null;
		
		if ( EditixFrame.THIS == null )
			return;
		
		EditixFrame.THIS.toFront();
		EditixFrame.THIS.requestFocus();
		XMLContainer container = EditixFrame.THIS.getSelectedContainer();
		if (container != null)
			container.requestFocus();
	}

	public static void stop(boolean delayed) {
		if (delayed) {
			Timer t = new Timer();
			t.schedule(new TimerTask() {
				public void run() {
					ss.toFront();
				}
			}, 500);
			t.schedule(new TimerTask() {
				public void run() {
					hideSplashScreen();
				}
			}, 2500);
		} else
			hideSplashScreen();
	}

	static SplashAdapter adapter = null;

	static class SplashAdapter extends WindowAdapter {
		public SplashAdapter() {
			super();
		}
		public void windowDeactivated(WindowEvent e) {
			ss.toFront();
		}

		public void windowLostFocus(WindowEvent e) {
			ss.toFront();
		}
	}

}
