package com.japisoft.editix.ui.container.locationbar;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class LabelFactory implements Label {

	private static LabelFactory INSTANCE = null;

	public static LabelFactory getInstance() {
		if ( INSTANCE == null )
			INSTANCE = new LabelFactory();
		return INSTANCE;
	}

	private Map<String,Label> labels = null;

	private LabelFactory() {
		labels = new HashMap<String, Label>();
		labels.put( "default", new DefaultLabel() );
		labels.put( "XSLT", new XSLTLabel() );
		labels.put( "XSLT2", labels.get( "XSLT" ) );
		labels.put( "XSLT3", labels.get( "XSLT" ) );
		labels.put( "XSD", new XSDLabel() );
		labels.put( "RNG", new RNGLabel() );
		labels.put( "XHTML", new XHTMLLabel() );
		labels.put( "FO", new FOLabel() );
		labels.put( "SVG", new SVGLabel() );
	}

	public String getLabel( String type, FPNode node ) {
		Label lbl = getLabel( type );
		String tmp = lbl.getLabel( node );
		if ( tmp == null )
			return getLabel( node );
		return tmp;
	}

	public Color getColor( String type, FPNode node ) {
		Label lbl = getLabel( type );
		Color tmp = lbl.getColor( node );
		if ( tmp == null )
			tmp = getColor( node );
		return tmp;
	}

	public Color getColor(FPNode node) {
		return getColor( "default", node );
	}

	public String getLabel(FPNode node) {
		return getLabel( "default", node );
	}
		
	private Label getLabel( String type ) {
		Label lbl = labels.get( type );
		if ( lbl == null ) {
			lbl = labels.get( "default" );
		}
		return lbl;
	}

	// ---------------------------------------------------
	
	static Color DEF = new Color( 200, 200, 200 );
	
	static class DefaultLabel implements Label {
		public Color getColor(FPNode node) {
			return DEF;
		}
		public String getLabel(FPNode node) {
			return node.getNodeContent();
		}
	}

}
