package com.japisoft.editix.ui.container.xpath;

import java.util.ArrayList;
import java.util.List;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpression;

import com.japisoft.editix.ui.container.SerializeStateObject;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XPathEditorModel implements SerializeStateObject {

	private String xpath;

	public String getXPath() {
		return xpath;
	}

	public void setXPath( String xpath ) {
		this.xpath = xpath;
	}

	public void restoreState(String serialize) {
		String[] tmp = serialize.split( "" );
		xpath = tmp[ 0 ];
		columns = new ArrayList<XPathColumn>();
		for ( int i = 1; i < tmp.length; i++ ) {
			XPathColumn xpc = new XPathColumn();
			xpc.restoreState( tmp[ i ] );
			columns.add( xpc );
		}
	}

	public String serializeState() {
		if ( this.xpath == null )
			return null;
		StringBuffer sb = new StringBuffer( this.xpath );
		for ( int i = 0; i < getColumnCount(); i++ ) {
			sb.append( "" );
			sb.append( getColumn( i ).serializeState() );
		}
		return sb.toString();
	}

	private List<XPathColumn> columns = null;
	
	public void addColumn( String name, String xpath ) {
		if ( columns == null ) {
			columns = new ArrayList<XPathColumn>();
		}
		columns.add( new XPathColumn( name, xpath ) );
	}
	
	public int getColumnCount() {
		if ( columns == null ) {
			return 0;
		}
		return columns.size();
	}

	public XPathColumn getColumn( int index ) {
		return columns.get( index );
	}

	@Override
	public String toString() {
		return xpath;
	}	
	
	// ---------------------------------------------

	public static class XPathColumn implements SerializeStateObject {
		private String name;
		private String xpath;

		XPathColumn( String name, String xpath ) {
			this.name = name;
			this.xpath = xpath;
		}

		public XPathColumn() {}

		public void restoreState(String serialize) {
			String[] tmp = serialize.split( "" );
			this.setName( tmp[ 0 ] );
			this.setXpath( tmp[ 1 ] );
		}

		public String serializeState() {
			return getName() + "" + getXpath();
		}

		public String getName() {
			return name;
		}
		public void setName(String name) {
			this.name = name;
		}
		public String getXpath() {
			return xpath;
		}
		public void setXpath(String xpath) {
			this.xpath = xpath;
		}
		private XPathExpression xpe = null;
		public XPathExpression getXPathExpression( XPath xpath ) throws Exception {
			if ( xpe == null ) {
				xpe = xpath.compile( this.xpath );
			}
			return xpe;
		}
	}

}
