package com.japisoft.editix.ui.container.xpath;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XPathResultTableModel implements TableModel {

	private XPathEditorModel query;
	private Document doc;
	
	public XPathResultTableModel( 
		Document doc, 
		XPathEditorModel query ) {
		this.doc = doc;
		this.query = query;
	}

	public XPathEditorModel getXPathEditorModel() {
		return query;
	}
	
	public Document getSource() {
		return doc;
	}
	
	public void dispose() {
		this.query = null;
		this.doc = null;
		this.results = null;
		fireTableUpdate();
	}

	public void setErrorFound( String errorMessage ) {}

	private List<XPathRunnerResultObject[]> results = null;
	
	public void addResult( XPathRunnerResultObject[] objects ) {
		if ( results == null ) {
			results = new ArrayList<XPathRunnerResultObject[]>();
		}
		results.add( objects );
	}

	public void run() {}
	public void stop() {}

	private TableModelListener l;
	
	private TableModelEvent uniqueEvent = new TableModelEvent( this );
	
	void fireTableUpdate() {
		l.tableChanged( uniqueEvent );
	}
	
	public void addTableModelListener(TableModelListener l) {
		this.l = l;
	}

	public void removeTableModelListener(TableModelListener l) {
		this.l = null;
	}

	public Class<?> getColumnClass(int columnIndex) {
		return String.class;
	}

	public int getColumnCount() {
		return query.getColumnCount();
	}

	public String getColumnName(int columnIndex) {
		return query.getColumn( columnIndex ).getName();
	}

	public int getRowCount() {
		if ( results == null )
			return 0;
		return results.size();
	}
	
	private boolean modified = false;

	public void setValueAt(
		Object aValue, 
		int rowIndex, 
		int columnIndex ) {
		XPathRunnerResultObject[] obj = results.get( rowIndex );
		obj[ columnIndex ].setValue( ( String )aValue );
		this.modified = true;
	}

	public boolean isModified() {
		return modified;
	}
	
	public void setModified( boolean modified ) {
		this.modified = modified;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		XPathRunnerResultObject[] obj = results.get( rowIndex );
		return obj[ columnIndex ].getValue();
	}
	
	public Node getNodeAt( int rowIndex ) {
		XPathRunnerResultObject[] obj = results.get( rowIndex );
		return obj[ 0 ].getSourceNode();
	}

	public void removeNodeAt( int rowIndex ) {
		Node n = getNodeAt( rowIndex );
		n.getParentNode().removeChild( n );
		results.remove( rowIndex );
		this.modified = true;
		fireTableUpdate();
	}
	
	public void addNodeAt( int rowIndex, Node copy ) throws Exception {
		Node currentNode = getNodeAt( rowIndex );
		Node newNode = copy.cloneNode( true );
		XPath xpath = XPathFactory.newInstance().newXPath();
		XPathRunnerResultObject[] objs = XPathRunner.getResultObject( getXPathEditorModel(), xpath, newNode );
		results.add( rowIndex, objs );
		currentNode.getParentNode().insertBefore( newNode,currentNode );
		fireTableUpdate();
		this.modified = true;
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return true;
	}

}
