package com.japisoft.editix.ui.container.xpath;

import javax.swing.SwingUtilities;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.japisoft.editix.ui.container.xpath.XPathEditorModel.XPathColumn;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XPathRunner implements Runnable {

	private XPathResultTableModel model;
	
	public XPathRunner( XPathResultTableModel model ) {
		this.model = model;
	}

	public void run() {
		String globalXPath = this.model.getXPathEditorModel().getXPath();
		Document document = this.model.getSource();

		// XPathFactory factory = XPathFactory.newInstance();
		
		XPathFactory factory = new net.sf.saxon.xpath.XPathFactoryImpl(); 
		
		XPath xpath = factory.newXPath();
		
		try {
			NodeList nl = ( NodeList )xpath.evaluate( 
				globalXPath, 
				document, 
				XPathConstants.NODESET 
			);
			for ( int i = 0; i < nl.getLength(); i++ ) {
				addResult( xpath, nl.item( i ) );
			}
			SwingUtilities.invokeLater(
				new Runnable() {
					public void run() {
						model.fireTableUpdate();						
					}
				} 
			);
		} catch( Exception exc ) {
			model.setErrorFound( exc.getMessage() );
		}
		
	}

	public static XPathRunnerResultObject[] getResultObject( XPathEditorModel columns, XPath xpath, Node node ) throws Exception {
		XPathRunnerResultObject[] objects = new XPathRunnerResultObject[ columns.getColumnCount() ];
		for ( int i = 0; i < columns.getColumnCount(); i++ ) {
			XPathColumn xpc = columns.getColumn( i );
			XPathExpression xe = xpc.getXPathExpression( xpath );
			XPathRunnerResultObject re = new XPathRunnerResultObject( node, xe );
			objects[ i ] = re;
		}
		return objects;
	}

	private void addResult( XPath xpath, Node node ) throws Exception {
		XPathEditorModel columns = this.model.getXPathEditorModel();	
		model.addResult( getResultObject( columns, xpath, node ) );
	}

}
