package com.japisoft.editix.ui.panels.diff;

import java.awt.Component;

import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.tree.TreeCellRenderer;

import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class CustomTreeRenderer implements TreeCellRenderer {

	private JLabel lbl = new JLabel();

	public CustomTreeRenderer() {
		lbl.setOpaque( true );
	}
	
	public static int guessType( FPNode node ) {
		int type = -1;
		if ( "true".equals( node.getAttribute( "dfx:insert" ) ) )
			type = DiffResourceFactory.ADD_TYPE;
		else
		if ( "true".equals( node.getAttribute( "dfx:delete" ) ) ) {
			type = DiffResourceFactory.REMOVE_TYPE;
		}

		for ( int i = 0; i < node.getViewAttributeCount(); i++ ) {
			if ( node.getViewAttributeAt( i ).startsWith( "del:" ) ) {
				type = DiffResourceFactory.ATT_TYPE;
				break;
			}
		}
		return type;
	}
	
	public Component getTreeCellRendererComponent(
			JTree tree, 
			Object value,
			boolean selected, 
			boolean expanded, 
			boolean leaf, 
			int row,
			boolean hasFocus ) {

		if ( selected ) {
			lbl.setForeground( 
					UIManager.getColor( 
							"Tree.selectionForeground" ) );
			lbl.setBackground( 
					UIManager.getColor( 
							"Tree.selectionBackground" ) );
		} else {
			lbl.setBackground( tree.getBackground() );
			lbl.setForeground( tree.getForeground() );
		}

		if ( value instanceof FPNode ) {
			FPNode node = ( FPNode )value;
			
			int type = guessType( node );

			lbl.setIcon( DiffResourceFactory.getIconForType( type ) );
			lbl.setIcon( DiffResourceFactory.getIconForType( type ) );				

			lbl.setBackground( DiffResourceFactory.getBgColorForType( type ) );
			lbl.setText( node.getContent() );
			
			if ( selected ) {
				lbl.setBackground( UIManager.getColor( "Tree.selectionBackground" ) );
				lbl.setForeground( UIManager.getColor( "Tree.selectionForeground" ) );				
			} else
				lbl.setForeground( tree.getForeground() );
			
		}

		return lbl;
	}
	
}
