package com.japisoft.editix.ui.panels.project2.synchro;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.SwingUtilities;

import com.japisoft.editix.ui.panels.project2.Node;
import com.japisoft.framework.ApplicationModel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public abstract class AbstractSynchronizer implements Synchronizer, SynchronizerListener, Runnable {

	private Map<String,String> properties = null;
	
	public Set<String> getProperties() {
		if ( properties == null )
			return null;
		return properties.keySet();
	}

	public String getProperty(String name) {
		if ( properties == null )
			return null;
		return properties.get( name );
	}

	private Task task = null;
	
	public void upload(final File rootPath, final Node node) throws IOException {
		if ( task != null ) {
			info( "A task is running" );
			return;
		}		
		task = new Task() {
			public void run() throws Exception {			
				start();
				try {
					uploadIt(rootPath, node );
				} finally {
					stop();
				}
			}
		};
		new Thread( this ).start();
	}

	public void run() {
		try {
			info( "Starting synchronization" );
			task.run();
			info( "End of the synchronization" );
		} catch( Exception exc ) {
			info( "Error : " + exc.getMessage() );
		}
		task = null;
		listener = null;
	}

	public void download( final File rootPath, final Node node ) throws IOException {
		if ( task != null ) {
			info( "A task is running" );
			return;
		}
		task = new Task() {
			public void run() throws Exception {
				start();
				try {
					downloadIt(rootPath, node );
				} finally {
					stop();
				}				
			}
		};
		new Thread( this ).start();
	}

	protected abstract void uploadIt(File rootPath, Node node) throws IOException;
	
	protected abstract void downloadIt(File rootPath, Node node) throws IOException;
		
	public void setProperty(String name, String value) {
		if ( properties == null )
			properties = new HashMap<String, String>();
		properties.put( name, value );
	}
	
	public void info(final String message) {
		SwingUtilities.invokeLater(
			new Runnable() {
				public void run() {
					ApplicationModel.fireApplicationValue( "message", message );
				}
			}
		);
	}

	private SynchronizerListener listener = null;
	
	public void setListener( SynchronizerListener listener ) {
		this.listener = listener;
	}
	
	public void start() {
		if ( listener != null )
			listener.start();
	}

	public void stop() {
		if ( listener != null )
			listener.stop();		
	}

	protected String getRelativePath( File rootPath, File file ) {
		String startPart = rootPath.toString();
		startPart = startPart.replace( "\\", "/" );
		if ( !startPart.endsWith( "/" ) )
			startPart += "/";
		String docPart = file.toString();
		docPart = docPart.replace( "\\", "/" );		
		if ( docPart.startsWith( startPart ) ) {
			return docPart.substring( startPart.length() );
		}
		return docPart;
	}
	
	protected String getRelativePath( File rootPath, Node file ) {
		return getRelativePath( rootPath, file.getPath() );
	}
	
	@Override
	public String toString() {
		return getName();
	}
	
	interface Task {
		public void run() throws Exception;
	}
	
}
