package com.japisoft.editix.ui.panels.universalbrowser;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;

import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.preferences.Preferences;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FileBrowserPanel extends CommonUniversalBrowserPanel {

	protected String getTitle() {
		return "File Browser";
	}

	protected Action[] getBrowserActions() {
		return new Action[] {
			new ChangeDefaultDirectory(),
			new RefreshContent()
		};
	}	

	protected boolean storeLastBrowsedFileObject() {
		return false;
	}	
	
	class RefreshContent extends AbstractAction {
		
		public RefreshContent() {
			putValue( 
					Action.SMALL_ICON, 
					new ImageIcon( 
							ClassLoader.getSystemResource( 
									"images/folder_refresh.png" ) ) );
			putValue(
					Action.SHORT_DESCRIPTION,
					"Refresh the file content" );			
		}
		
		public void actionPerformed(ActionEvent e) {
			try {
				browserTree.refresh();
			} catch( Exception exc ) {
				EditixFactory.buildAndShowErrorDialog( "Can't refresh : " + exc.getMessage() );
			}
		}
	}

	class ChangeDefaultDirectory extends AbstractAction {

		ChangeDefaultDirectory() {
			putValue( 
					Action.SMALL_ICON, 
					new ImageIcon( 
							ClassLoader.getSystemResource( 
									"images/folder_view.png" ) ) );
			putValue(
					Action.SHORT_DESCRIPTION,
					"Change the default directory" );
		}

		public void actionPerformed(ActionEvent e) {
			JFileChooser cc = new JFileChooser();
			cc.setCurrentDirectory(					
					new File(
					Preferences.getPreference( 
							"filebrowser", 
							"defaultpath", 
							System.getProperty( "user.home" ) )					
					
					) );
			cc.setMultiSelectionEnabled( false );
			cc.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
			if ( cc.showOpenDialog( EditixFrame.THIS ) == 
				JFileChooser.APPROVE_OPTION ) {
				
				Preferences.setPreference( 
						"filebrowser", 
						"defaultpath", 
						cc.getSelectedFile().toString() );

				String uri = "file:///";
				uri += cc.getSelectedFile().toString();
				
				Preferences.setPreference( 
						Preferences.SYSTEM_GP, 
						FileBrowserPanel.class.getName(),
						uri );				

				browse(
						uri, 
						null, 
						null );

			}
		}

	}

}
