package com.japisoft.editix.ui.pathbuilder;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import com.japisoft.editix.action.file.BrowseZIPAction;
import com.japisoft.editix.ui.panels.project2.ProjectManager;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.application.descriptor.InterfaceBuilder;
import com.japisoft.framework.ui.text.PathBuilder;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.walker.TreeWalker;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
abstract class AbstractPathBuilder implements PathBuilder { 
	
	protected void completeForType( String type, List l ) {
		ProjectManager.getInstance().searchFilesForType( l, type );
	}
	
	private ArrayList l = null;
	
	private void addChoice( String path ) {
		if ( path != null && path.indexOf( "!" ) == -1 && !BrowseZIPAction.isFileArchive( path ) ) {
			if ( l == null )
				l = new ArrayList();
			if ( !l.contains( path ) ) {
				l.add( path );
			}
		}
	}

	public String[] buildPathsChoice() {
		String[] types = getTypes();
		addFromActiveOnes( types );
		if ( l == null )
			return null;
		else {
			String[] tmp = new String[ l.size() ];
			for ( int i = 0; i < l.size(); i++ )
				tmp[ i ] = ( String )l.get( i );
			return tmp;
		}
	}

	private void addFromActiveOnes( String[] types ) {
		// Search for opened
		for ( int i = 0; i < EditixFrame.THIS.getXMLContainerCount(); i++ ) {
			if ( EditixFrame.THIS.getXMLContainer( i ) == null )
				continue;
			for ( int j = 0; j < types.length; j++ ) {
				if ( types[ j ].equals(
						EditixFrame.THIS.getXMLContainer( i ).getDocumentInfo().getType() ) ) {
					addChoice( EditixFrame.THIS.getXMLContainer( i ).getCurrentDocumentLocation() );
				}
			}
		}
		// Search from project
		for ( int j = 0; j < types.length; j++ ) {
			ArrayList tempo = new ArrayList();
			completeForType( types[ j ], tempo );
			for ( int k = 0; k < tempo.size(); k++ )
				addChoice( ( String )tempo.get( k ) );
		}
		// Search from history
		InterfaceBuilder ib = EditixFrame.THIS.getBuilder();
		FPNode node = ib.getMenuNode( "openr" );
		if ( node != null ) {
			TreeWalker tw = new TreeWalker( node );
			Enumeration enume = tw.getTagNodeByName( "ui", true );
			if ( enume != null ) {
				enume.nextElement();	// Skip the menu ui
				while ( enume.hasMoreElements() ) {
					FPNode uiNode = ( FPNode )enume.nextElement();
					if ( uiNode.hasAttribute( "param2" ) ) {
						String type = uiNode.getAttribute( "param2" );
						for ( int j = 0; j < types.length; j++ ) {
							if ( type.equals( types[ j ] ) ) {
								if ( uiNode.hasAttribute( "param" ) ) {
									addChoice( uiNode.getAttribute( "param" ) );
								}
							}
						}
					}
				}
			}
		}
	}

	protected abstract String[] getTypes();
	
}
