package com.japisoft.editix.ui.xslt;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.util.ArrayList;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;

import com.japisoft.editix.ui.xslt.profiler.ProfilerElement;
import com.japisoft.framework.ui.table.ExportableTable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ProfilerContainer extends JPanel implements ListSelectionListener {

	private LineSelectionListener container;

	public ProfilerContainer( LineSelectionListener container ) {
		super();
		initUI();
		this.container = container;
	}

	private JTable table = null;
	private NodeRenderer nr = new NodeRenderer();

	private void initUI() {
		table = new ExportableTable() {
			public boolean isCellEditable(int row, int column) {
				return false;
			}
			public TableCellRenderer getCellRenderer(int row, int column) {
				return nr;
			}
		};
		DefaultTableModel dtm = new
			DefaultTableModel( new String[] { "Node", "Time", "Iter" }, 0 );
		table.setModel( dtm );
		table.getSelectionModel().setSelectionMode(
				ListSelectionModel.SINGLE_SELECTION );
		updateTableColumnSize();
		setLayout( new BorderLayout() );
		add( new JScrollPane( table ) );
	}

	private void updateTableColumnSize() {
		table.getColumnModel().getColumn( 0 ).setWidth( 150 );
		table.getColumnModel().getColumn( 0 ).setPreferredWidth( 150 );
		table.getColumnModel().getColumn( 2 ).setMaxWidth( 30 );		
	}

	public void addNotify() {
		super.addNotify();
		linkListeners();
	}

	public void removeNotify() {
		super.removeNotify();
		unlinkListners();
	}

	private void linkListeners() {
		table.getSelectionModel().addListSelectionListener( this );
	}
	private void unlinkListners() {
		table.getSelectionModel().removeListSelectionListener( this );
	}

	public void updateProfilerContext( ArrayList context ) {
		unlinkListners();
		
		double totalTime = 0;
		for ( int i = 0; i < context.size(); i++ ) {
			ProfilerElement pe = ( ProfilerElement )context.get( i );
			totalTime += pe.totalTime;
		}

		DefaultTableModel dtm = new
		DefaultTableModel( new String[] { "Node", "Time", "Iter" }, 0 );
				
		// Percent
		for ( int i = 0; i < context.size(); i++ ) {
			ProfilerElement pe = ( ProfilerElement )context.get( i );
			pe.timePercent = (int)Math.round( ( pe.totalTime / totalTime ) * 100.0 );			
			dtm.addRow( new Object[] { pe, pe, pe } );
		}

		table.setModel( dtm );
		updateTableColumnSize();
		table.getSelectionModel().setSelectionInterval( 0 , 0 );

		linkListeners();
	}

	public void valueChanged(ListSelectionEvent e) {
		if ( e.getFirstIndex() > -1 ) {
			ProfilerElement pe = ( ProfilerElement )table.getModel().getValueAt(
					e.getFirstIndex(), 
					0 
			);
			container.showXSLTLine( pe.uri, pe.line );
		}
	}

	class NodeRenderer implements TableCellRenderer {
		
		private JLabel label = new JLabel();
		private JProgressBar bar = new JProgressBar();

		private NodeRenderer() {
			label.setOpaque( true );
			bar.setForeground( new Color( 200, 250, 200 ) );
			bar.setMinimum( 0 );
			bar.setMaximum( 100 );
		}

		public Component getTableCellRendererComponent(
				JTable table, 
				Object value,
				boolean isSelected, 
				boolean hasFocus, 
				int row, 
				int column ) {

			if ( isSelected ) {
				label.setForeground( table.getSelectionForeground() );
				label.setBackground( table.getSelectionBackground() );
				bar.setBackground( table.getSelectionBackground() );				
			} else {
				label.setForeground( table.getForeground() );
				label.setBackground( table.getBackground() );
				bar.setBackground( table.getBackground() );				
			}

			ProfilerElement element = ( ProfilerElement )value;

			if ( column == 1 ) {
				bar.setValue( element.timePercent );
				return bar;
			}
			else
			if ( column == 0 ) {
				label.setHorizontalAlignment( SwingConstants.LEFT );
				label.setText( element.name );
			}
			else
			if ( column == 2 ) {
				label.setHorizontalAlignment( SwingConstants.CENTER );
				label.setText( "" + element.iteration );
			}
			
			return label;
			
		}
	}

}
