package com.japisoft.editix.ui.xslt.map;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;

import com.japisoft.framework.ui.table.ExportableTable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ElementTable extends ExportableTable {
	
	public void setElement( Element e ) {
		setModel( new ElementModel( e ) );
	}

	private ElementTableListener etl;

	public void setElementTableListener( ElementTableListener etl ) {
		this.etl = etl;
	}

	void fireElementTableUpdated() {
		if ( this.etl != null )
			this.etl.tableUpdated();
	}
	
	class ElementModel implements TableModel {

		private Element e;
		
		ElementModel( Element e ) {
			this.e = e;
		}
		
		private TableModelListener l;
		
		public void addTableModelListener( TableModelListener l ) {
			this.l = l;
		}

		public void removeTableModelListener( TableModelListener l ) {
			this.l = null;
		}

		public Class<?> getColumnClass( int columnIndex ) {
			return String.class;
		}

		public int getColumnCount() {
			return 2;
		}

		public String getColumnName( int columnIndex ) {
			if ( columnIndex == 0 )
				return "Attribute name";
			return "Attribute value";
		}

		public int getRowCount() {
			NamedNodeMap nnm = e.getAttributes();
			if ( nnm == null )
				return 1;
			return nnm.getLength() + 1;
		}

		public Object getValueAt( 
				int rowIndex, 
				int columnIndex ) {
			NamedNodeMap nnm = e.getAttributes();
			
			if ( rowIndex == nnm.getLength() ) {
				return "";
			}
			
			Attr a = ( Attr )nnm.item(rowIndex);
			if ( columnIndex == 0 )
				return a.getNodeName();
			return a.getNodeValue();
		}

		public boolean isCellEditable( 
				int rowIndex, 
				int columnIndex
		) {
			return true;
		}

		public void setValueAt(
				Object aValue, 
				int rowIndex, 
				int columnIndex ) {
			NamedNodeMap nnm = e.getAttributes();
			
			if ( rowIndex == nnm.getLength() ) {
				
				if ( columnIndex == 0 ) {
					e.setAttribute( ( String )aValue, "" );
					
					if ( l != null ) {
						l.tableChanged( new TableModelEvent( this ) );
					}
				}
				
			} else {
			
				Attr a = ( Attr )nnm.item(rowIndex);
				String name = a.getNodeName();
				
				if ( columnIndex == 1 ) {
					a.setNodeValue( ( String )aValue );
				} else {
					
					e.removeAttributeNode( a );
					if ( !"".equals( aValue ) ) {
						e.setAttribute( 
							name, 
							( String )aValue 
						);
					} else {
						
						if ( l != null ) {
							l.tableChanged( new TableModelEvent( this ) );
						}						

					}
					
				}
			
			}
			
			fireElementTableUpdated();
		}

	}

}
