package com.japisoft.editix.ui.xslt.map;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NamespaceSplitterSaxHandler extends DefaultHandler {

	private VirtualDomNode root = null;
	private Stack<VirtualDomNode> stack = null;
	private NodeList refNodes = null;
	private int nodeCounter;

	private String namespace;
	private boolean ignoreNamespace;

	private List<VirtualDomNode> list = null;

	private boolean skipRoot = false;

	public NamespaceSplitterSaxHandler( String namespace, boolean ignoreNamespace, Element refNode ) {
		this.namespace = namespace;
		this.ignoreNamespace = ignoreNamespace;

		root = new VirtualDomNode( refNode );
		stack = new Stack<VirtualDomNode>();
		stack.push( root );

		// Get all the nodes
		// refNodes = refNode.getElementsByTagNameNS( namespace, "*" );
		
		refNodes = refNode.getElementsByTagName( "*" );		
		nodeCounter = 0;
		list = new ArrayList<VirtualDomNode>();
		list.add( root );
	}

	public List<VirtualDomNode> getAll() { return list; }
	
	public VirtualDomNode getRoot() { return root; }
	
	@Override
	public void startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if ( !skipRoot ) {
			skipRoot = true;
			return;
		}
		
		if ( ( !ignoreNamespace && uri.equalsIgnoreCase( namespace ) ) || 
				( ignoreNamespace && !uri.equalsIgnoreCase( namespace ) ) ) {
			VirtualDomNode parent = stack.peek();
			VirtualDomNode node = new VirtualDomNode( refNodes.item( nodeCounter ) );
			parent.addChild( node );
			stack.push( node );
			list.add( node );
		}

		nodeCounter++;
		
	}

	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		if ( ( !ignoreNamespace && uri.equalsIgnoreCase( namespace ) ) || 
				( ignoreNamespace && !uri.equalsIgnoreCase( namespace ) ) ) {
			stack.pop();
		}
	}

}
