package com.japisoft.editix.ui.xslt.map;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class VirtualDomNode {

	private Node source;

	private VirtualDomNode parent;
	
	public VirtualDomNode( Node source ) {
		this.source = source;
		source.setUserData( "vdm", this, null );		
	}

	public Node getSource() { return source; }

	private List<VirtualDomNode> children = null;

	public void addChild( VirtualDomNode node ) {
		if ( children == null ) {
			children = new ArrayList<VirtualDomNode>();
		}
		node.parent = this;
		children.add( node );
	}

	public void removeChild( VirtualDomNode node ) {
		// Remove virtually
		children.remove( node );
		// Remove physically
		Node sn = node.getSource();
		sn.getParentNode().removeChild( sn );
	}
	
	public int getChildCount() {
		if ( children == null )
			return 0;
		return children.size();
	}
	
	public VirtualDomNode getParent() { return parent; }

	/** 
	 * @param ns The namespace to avoid
	 * @return
	 */
	public int getChildCount( String ns ) {
		NodeList nl = source.getChildNodes();
		int cpt = 0;
		for ( int i = 0; i < nl.getLength(); i++ ) {
			if ( nl.item( i ) instanceof Element ) {
				if ( !ns.equalsIgnoreCase( nl.item( i ).getNamespaceURI() ) )
					cpt++;
			}
		}
		return cpt;
	}

	/**
	 * @param ns The namespace to avoid
	 * @param index
	 * @return */
	public VirtualDomNode getChildAt( String ns, int index ) {
		NodeList nl = source.getChildNodes();
		int cpt = 0;
		for ( int i = 0; i < nl.getLength(); i++ ) {
			if ( nl.item( i ) instanceof Element ) {
				if ( !ns.equalsIgnoreCase( nl.item( i ).getNamespaceURI() ) ) {
					if ( cpt == index ) {
						return ( VirtualDomNode )nl.item( i ).getUserData( "vdm" );
					}
					cpt++;
				}
			}
		}
		return null;
	}

	public VirtualDomNode getChildAt( int index ) {
		return children.get( index );
	}
	
	public int getIndexOfChild( VirtualDomNode node ) {
		if ( node.children == null ) {
			return -1;
		}
		return node.children.indexOf( node );
	}
	
	@Override
	public String toString() {
		if ( source.getNodeType() == Node.ELEMENT_NODE ) {
			return source.getLocalName();
		}
		return source.getNodeValue();
	}

}
