 package com.japisoft.editix.xslt.debug;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Action;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;

import com.japisoft.editix.ui.EditixDialog;
import com.japisoft.editix.ui.EditixFactory;
import com.japisoft.editix.ui.EditixFrame;
import com.japisoft.framework.dialog.DialogManager;
import com.japisoft.framework.dialog.actions.AbstractDialogAction;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.bookmark.BookmarkModel;
import com.japisoft.xmlpad.bookmark.BookmarkPosition;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BreakpointsDialog extends EditixDialog implements ActionListener {
	
	private BookmarkModel model;
	
	private JButton btnRemove;
	private JButton btnSelect;

	public BreakpointsDialog( BookmarkModel model ) {
		super( 
				"Breakpoints List", 
				"Breakpoints", 
				"You can remove each breakpoint or select it",
				DialogManager.buildNewActionModel() );
		this.model = model;
		initUI();

		AbstractDialogAction a = new 
			AbstractDialogAction( 10, false ) {	};
		a.setActionDelegate( this );
		a.putValue( Action.NAME, "Remove" );
		a.putValue( Action.ACTION_COMMAND_KEY, "Remove" );
		actionModel.addDialogAction( a );

		a = new 
		AbstractDialogAction( 11, false ) {	};
		a.setActionDelegate( this );
		a.putValue( Action.NAME, "Select" );
		a.putValue( Action.ACTION_COMMAND_KEY, "Select" );
		actionModel.addDialogAction( a );
	}

	protected Dimension getDefaultSize() {
		return new Dimension( 350, 300 );
	}
	
	public void actionPerformed( ActionEvent e ) {
		String cmd = e.getActionCommand();
		if ( "Remove".equals( cmd ) ) {
			remove();
		} else
		if ( "Select".equals( cmd ) ) {
			select();
		}
	}
	
	private void remove() {
		int index = list.getSelectedIndex();
		if ( index == -1 ) {
			EditixFactory.buildAndShowErrorDialog( "No Selection" );
		} else {
			((DefaultListModel)list.getModel()).remove( index );
			BookmarkPosition position;
			model.removeBookmarkPosition(
					position = model.getBookmarkPositionAt( index ) );
			EditixFrame.THIS.getSelectedContainer().getEditor().getHighlighter().removeHighlight( position.getHighlightFlag() );
			EditixFrame.THIS.getSelectedContainer().getView().repaint();
		}
	}
	
	private void select() {
		int index = list.getSelectedIndex();
		if ( index == -1 ) {
			EditixFactory.buildAndShowErrorDialog( "No Selection" );
		} else {
			BookmarkPosition position = model.getBookmarkPositionAt( index );
			int offset = position.getOffset();
			XMLContainer container = EditixFrame.THIS.getSelectedContainer();
			container.getEditor().setCaretPosition( offset );
		}		
	}
	
	JList list = new JList();
	
	private void initUI() {
		DefaultListModel lm = new DefaultListModel();
		list.setCellRenderer( new SimpleRenderer() );
		for ( int i = 0; i < model.getBookmarkCount(); i++ ) {
			BookmarkPosition position = model.getBookmarkPositionAt( i );
			lm.addElement( "Breakpoint " + ( i + 1 ) );
		}
		list.setModel( lm );
		getContentPane().add( list );
	}

	ImageIcon icon = new ImageIcon( ClassLoader.getSystemResource( "images/breakpoint.png" ) );
	
	class SimpleRenderer extends DefaultListCellRenderer {
		
		public Component getListCellRendererComponent(
		        JList list,
		        Object value,
		        int index,
		        boolean isSelected,
		        boolean cellHasFocus) {
			JLabel lbl = ( JLabel )super.getListCellRendererComponent( list, value, index, isSelected, cellHasFocus );
			lbl.setIcon( icon );
			return lbl;
		}

	}
}
