package com.japisoft.editix.xslt.debug;

import net.sf.saxon.om.DocumentInfo;
import net.sf.saxon.om.SequenceIterator;
import net.sf.saxon.value.AtomicValue;
import net.sf.saxon.value.Closure;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class VariableImpl 
		implements Variable {

	private String name;
	private String type;
	private Object value;
	private int line;

	public VariableImpl(
			String name,
			String type,
			Object value,
			int line ) {
		this.name = name;
		this.type = type;
		this.value = value;
		this.line = line;
	}
	
	private NodeDebug node = null;
	
	public VariableImpl(
		NodeDebug node
	) {
		this.node = node;		
	}

	public String getName() {
		if ( node != null )
			return node.getVariableName();
		return name;
	}

	public String getType() {
		if ( node != null ) {
			Object value = node.getValue();
			return prettyType( value );
		}
		if ( type == null )
			return prettyType( value );
		return type;
	}

	public Object getValue() {
		if ( node != null )
			return prettyValue( node.getValue() );
		return prettyValue( value );
	}

	public int getLine() {
		return line;
	}

	private String prettyType( Object value ) {
		if ( value instanceof AtomicValue ) {
			AtomicValue av = ( AtomicValue )value;
			return av.getPrimitiveType().getDisplayName();
		} else
			return "node";
	}
	
	private String prettyValue( Object value ) {
		if( value instanceof Closure ) {
			try {
				Closure c = ( Closure )value;
				SequenceIterator si = c.iterate();
				int cpt = 0;
				while ( ( si.next() != null ) ) {
					cpt++;
				}
				return cpt + " item(s)";
			} catch( net.sf.saxon.trans.XPathException xe ) {
				return "Can't evaluate";
			}
		} else
		if ( value instanceof DocumentInfo ) {
			return "Document";
		}			
		return value.toString();
	}
	
}
