package com.japisoft.framework.application.descriptor.composer;

import java.util.HashMap;
import java.util.Map;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DescriptorTreeModel implements TreeModel {

	private TreeModelListener l;
	
	private Element root;
	
	private Map<String,Element> items = null;
	private Map<String,Boolean> legalElements = null;
	
	public DescriptorTreeModel( Element root ) {
		this.root = root;

		legalElements = new HashMap<String, Boolean>();
		
		String[] le = new String[] {
				"menuBar",
				"menu",
				"item",
				"itemRef",
				"separator",
				"toolBar",
				"popup"
		};
		
		for ( String l : le ) {
			legalElements.put( l, Boolean.TRUE );
		}
		
		// Store item by id for itemRef usage
		items = new HashMap<String, Element>();
		NodeList nl = root.getElementsByTagName( "item" );
		for ( int i = 0; i < nl.getLength(); i++ ) {
			Element item = ( Element )nl.item( i );
			items.put( item.getAttribute( "id" ), item );
		}
	}

	public void addTreeModelListener(TreeModelListener l) {
		this.l = l;
	}

	public void fireStructurChange( TreePath tp ) {
		if ( l != null )
			l.treeStructureChanged( new TreeModelEvent( this, tp ) );
	}
	
	public Object getChild(Object parent, int pindex) {
		int index = -1;
		NodeList nl = ( ( Element )parent ).getChildNodes();
		for ( int i = 0; i < nl.getLength(); i++ ) {
			Node n = nl.item( i );
			if ( n instanceof Element ) {
				if ( legalElements.containsKey( n.getNodeName() ) ) {
					index++;
					if ( index == pindex ) {
						if ( "itemRef".equals( 
								n.getNodeName() ) ) {
							return items.get( ( ( Element )n ).getAttribute( "ref" ) );
						}
						return n;
					}
				}
			}
		}
		return null;
	}

	public int getChildCount(Object parent) {
		int cpt = 0;
		NodeList nl = ( ( Element )parent ).getChildNodes();
		for ( int i = 0; i < nl.getLength(); i++ ) {
			Node n = nl.item( i );
			if ( n instanceof Element ) {
				if ( legalElements.containsKey( n.getNodeName() ) )
					cpt++;
			}
		}
		return cpt;
	}

	public int getIndexOfChild(Object parent, Object child) {
		int index = -1;
		NodeList nl = ( ( Element )parent ).getChildNodes();
		for ( int i = 0; i < nl.getLength(); i++ ) {
			Node n = nl.item( i );
			if ( n instanceof Element ) {
				if ( legalElements.containsKey( n.getNodeName() ) ) {
					index++;
					if ( n == child ) {
						break;
					}
				}
			}
		}
		return index;
	}

	public Object getRoot() {
		return root;
	}

	public boolean isLeaf(Object node) {
		Node n = ( Node )node;
		String name = n.getNodeName();
		if ( "item".equals( name ) )
			return true;
		if ( "itemRef".equals( name ) )
			return true;
		if ( "separator".equals( name ) )
			return true;
		return false;
	}

	public void removeTreeModelListener(TreeModelListener l) {
		this.l = null;
	}

	public void valueForPathChanged(TreePath path, Object newValue) {
	}

}
