package com.japisoft.framework.css;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ColorParser {

	public static Map<String,Color> predefined = new HashMap<String,Color>();
	
	static {

		Object[] table = {
				"aqua", "00FFFF",
				"green", "008000",
				"orange", "FFA500",
				"white", "FFFFFF",
				"black", "000000",
				"lime", "00FF00",
				"purple", "800080",	
				"yellow", "FFFF00",
				"blue", "0000FF",
				"maroon", "800000",
				"red", "FF0000",
				"fuschia", "FF00FF",
				"navy", "000080",
				"silver", "C0C0C0",
				"gray", "808080",
				"olive", "808000",
				"teal", "008080"	
		};

		for ( int i = 0; i < table.length; i += 2 ) {
			predefined.put(
				( String )table[ i ],
				new Color( 
					Integer.parseInt( 
						( String )table[ i + 1 ], 16 ) 
				) 
			);
		}

	}
	
	private ColorParser() {
	}
	
	private final static ColorParser INSTANCE = new ColorParser();
	
	public static ColorParser getInstance() { return INSTANCE; }

	public Color parseColor( String val ) {
		if ( val.startsWith( "#" ) ) {
			String rgb = val.substring( 1 );
			if ( rgb.length() == 3 ) {
				String r = Character.toString( rgb.charAt( 0 ) );
				String g = Character.toString( rgb.charAt( 1 ) );
				String b = Character.toString( rgb.charAt( 2 ) );
				rgb = r + r + g + g + b + b;
			}
			return new Color( Integer.parseInt( rgb, 16 ) );	
		} else {
			float r,g,b;
			float a = 1.0f;

			String values = null;
			
			if ( val.startsWith( "rgb" ) ) {
				int start = val.indexOf( "(" );
				int end = val.indexOf( ")" );
				String token = val.substring( start + 1, end );
				String[] parts = token.split( "," );

				r = ( float )( Float.parseFloat( parts[ 0 ] ) / 255.0 );
				g = ( float )( Float.parseFloat( parts[ 1 ] ) / 255.0 );
				b = ( float )( Float.parseFloat( parts[ 2 ] ) / 255.0 );
				
				if ( parts.length == 4 ) {
					a = Float.parseFloat( parts[ 3 ] );
				}
				
				return new Color( r, g, b, a );
			}
		}
		return predefined.get( val.toLowerCase() );
	}
	
	
}
