package com.japisoft.framework.css;

import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.util.HashMap;
import java.util.Map;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FontParser {

	public static Map<String,Font> predefined = new HashMap<String,Font>();

	static {

		predefined.put( "serif", new Font( Font.SERIF, Font.PLAIN, 14 ) );
		predefined.put( "sans-serif", new Font( Font.SANS_SERIF, Font.PLAIN, 14 ) );
		predefined.put( "monospace", new Font( Font.MONOSPACED, Font.PLAIN, 14 ) );
		predefined.put( "cursive", predefined.get( "sans-serif" ) );
		predefined.put( "fantasy", predefined.get( "sans-serif" ) );

		GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
		Font[] fonts = ge.getAllFonts();
		for ( Font f : fonts ) {
			
			f = f.deriveFont( Font.PLAIN, 14 );
			
			predefined.put(
					f.getFontName().toLowerCase(), f );
			
		}

	}

	private FontParser() {}

	private final static FontParser INSTANCE = new FontParser();

	public static FontParser getInstance() { return INSTANCE; }
	
	public Font parseFont( String val ) {
		String[] fonts = val.split( "," );
		for ( String f : fonts ) {
			f = f.trim().toLowerCase();
			if ( f.startsWith( "\"" ) ) 
				f = f.substring( 1, f.length() -1 );
			if ( predefined.containsKey( f ) )
				return predefined.get( f );
		}
		return null;
	}

	public Font getDefaultFont() { return predefined.get( "serif" ); }
	
	public static void main( String[] args ) {
		System.out.println( FontParser.getInstance().parseFont( "\"Lucida Console\", Courier, monospace" ) );
	}
}
