package com.japisoft.framework.css;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SimpleCSSDocument implements CSSDocument {

	private List<Rule> rules = null;

	public SimpleCSSDocument( String content ) {
		parse( content );
	}

	private void parse( String content ) {
		StringBuffer sb = new StringBuffer();
		SimpleRule currentRule = null;
		char lastC = 0;
		boolean commentMode = false;
		for ( int i = 0; i < content.length(); i++ ) {
			char c = content.charAt( i );
			if ( c == '*' && lastC == '/' ) {
				commentMode = true;
				sb.deleteCharAt( sb.length() - 1 );
			}
			if ( !commentMode ) {
				if ( c == '{' ) {
					currentRule = new SimpleRule();
					currentRule.setSelector( new SimpleSelector( sb.toString().trim() ) );
					sb.setLength( 0 );
				} else
				if ( c == '}' ) {
					String[] allProperties = sb.toString().split( ";" );
					for ( String property : allProperties ) {
						property = property.trim();
						if ( !"".equals( property ) )
							currentRule.addProperty( new SimpleProperty( property ) );
					}
					addRule( currentRule );
					sb.setLength( 0 );
				} else {
					sb.append( c );
				}
			} else {
				if ( c == '/' && lastC == '*' ) {
					commentMode = false;
				}
			}
			lastC = c;
		}
	}

	public void addRule( Rule r ) {
		if ( rules == null )
			rules = new ArrayList<Rule>();
		rules.add( r );
	}

	public List<Rule> getRules() {
		return rules;
	}

	public Rule getRule( String selector ) {
		if ( rules == null )
			return null;
		for ( Rule r : rules ) {
			if ( r.getSelector().toString().equals( selector ) )
				return r;
		}
		return null;
	}
	
	public Rule matchElement( Element element ) {
				
		if ( rules == null )
			return null;
		
		List<Rule> matchingRules = null;
		
		for ( Rule r : rules ) {
						
			if ( r.match( element ) ) {
				
				Selector oldSelector = r.getSelector();
				r = new SimpleRule( r );
				( ( SimpleRule )r ).setSelector( oldSelector );
				
				
				if ( matchingRules == null )
					matchingRules = new ArrayList<Rule>();
				
				boolean added = false;
				for ( int i = 0; i < matchingRules.size(); i++ ) {
					Rule r2 = matchingRules.get( i );
					if ( r2.getSelector().getSpecificity() <= r.getSelector().getSpecificity() ) {
						matchingRules.add( i, r );
						added = true;
						break;
					}		
				}
				if ( !added ) {
					matchingRules.add( r );
				}

			}
		}
		
		if ( matchingRules != null ) {
			Rule r = null;
			for ( Rule tmpr : matchingRules ) {
				if ( r == null )
					r = tmpr;
				else {

					// tmpr is ancestor, keep only some properties
					SimpleRule newR = new SimpleRule();
					for ( String toKeep : Toolkit.getInheritedProperties() ) {
						if ( tmpr.getProperty( toKeep ) != null )
							newR.addProperty( tmpr.getProperty( toKeep ) );
					}
					r = r.merge( newR );
				}
			}
			return r;
		}
		
		return null;
	}

}
