package com.japisoft.framework.css;

import java.awt.Color;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SimpleProperty implements Property {

	private String name;
	private Object value;
	
	public SimpleProperty( String name, Object value ) {
		this.name = name;
		this.value = value;
	}
	
	public SimpleProperty( String content ) {
		String[] part = content.split( ":" );
		if ( part.length >= 2 ) {
			this.name = part[ 0 ].trim().toLowerCase();
			this.value = part[ 1 ].trim();
			
			if ( name.startsWith("border" ) ) {
				this.value = BorderParser.getInstance().parse( 
					( String )this.value 
				);
			} else
			if ( name.contains( "color" ) ) {
				try {
					this.value = ColorParser.getInstance().parseColor( part[ 1 ] );
					if ( this.value == null )
						this.value = Color.BLACK;
				} catch( Exception exc ) {
					this.value = Color.BLACK;
				}
			} else
			if ( "font-family".equals( name ) ) {
				this.value = FontParser.getInstance().parseFont( part[ 1 ] );
				if ( this.value == null )
					this.value = FontParser.getInstance().getDefaultFont();				
			} else
			if ( "font-size".equals( name ) ) {
				try {
					String unit = null;
					if ( part[ 1 ].endsWith( "px" ) || part[ 1 ].endsWith( "pt" ) || part[ 1 ].endsWith( "em" ) ) {
						unit = part[ 1 ].substring( 
							0, 
							part[ 1 ].length() - 2 
						);
						if ( part[ 1 ].endsWith( "em" ) ) {
							unit = Integer.toString( ( int )Math.round( 12.0f * Float.parseFloat( unit ) ) );
						}
						this.value = Integer.parseInt( unit );
					} else
						this.value = Integer.parseInt( part[ 1 ] );
				} catch( Exception exc ) {
					this.value = null;
				}
			} else
			if ( this.value instanceof String ) {
				if ( ( ( ( String )this.value ) ).endsWith( "%" ) || 
					( ( ( String )this.value ) ).endsWith( "px" ) ) {
					this.value = new CSSDim( ( String )this.value );
				}
			}

		} else
			this.name = "?";
	}

	public String getName() {
		return name;
	}

	public Object getValue() {
		return value;
	}

	public static void main( String[] args ) {
		SimpleProperty sp = new SimpleProperty( "font-size:1d3" );
		System.out.println( sp.getValue() );
		
	}

}
