package com.japisoft.framework.dialog;

import java.awt.Dialog;
import java.awt.Frame;

import javax.swing.Icon;
import javax.swing.JComponent;

import com.japisoft.framework.dialog.actions.DialogActionModel;
import com.japisoft.framework.dialog.actions.OKAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicOKDialogComponent extends BasicDialogComponent {

	public BasicOKDialogComponent(
			Dialog owner,
			String dialogTitle,
			String title, 
			String comment, 
			Icon icon ) {
		super( owner, dialogTitle, true );
		init( title, comment, icon );
	}

	public BasicOKDialogComponent(
			Frame owner,
			String dialogTitle,
			String title, 
			String comment, 
			Icon icon ) {
		super( owner, dialogTitle, true );
		init( title, comment, icon );
	}

	DialogHeader header;
	DialogFooter footer;

	/** Defining the dialog actions */
	protected DialogActionModel prepareActionModel() {
		DialogActionModel actionModel = new DialogActionModel();
		actionModel.addDialogAction( new OKAction() );
		return actionModel;
	}

	private void init( String title, String comment, Icon icon ) {
		header = DialogManager.getDefaultDialogHeader();
		header.setComment( comment );
		header.setTitle( title );
		if  ( icon == null )
			icon = DialogManager.getDefaultDialogIcon();
		header.setIcon( icon );

		footer = DialogManager.getDefaultDialogFooter( true );
		DialogActionModel model = prepareActionModel();
		footer.setDialogTarget( this );
		footer.setModel( model );
	}

	private boolean first = true;
	
	public void addNotify() {
		super.addNotify();
		if ( !setUICalled && !first )
			throw new RuntimeException( "Invalid dialog usage, you must call setUI !" );
		first = false;
	}

	private boolean setUICalled = false;

	public JComponent setUI( JComponent pane ) {
		init( header, pane, footer );
		setUICalled = true;
		return pane;
	}
	
	public JComponent setUI( JComponent pane, boolean enabledHeader, boolean enabledFooter ) {
		init(
				enabledHeader ? header : null,
				pane,
				enabledFooter ? footer : null );
		setUICalled = true;
		return pane;
	}
	
}
