package com.japisoft.framework.dialog.help;

import java.awt.Dimension;
import java.awt.Frame;
import java.io.IOException;
import java.net.URL;

import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.dialog.BasicOKDialogComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ManualDialog extends BasicOKDialogComponent implements
		HyperlinkListener {

	/** This is a path from the classpath */
	public static String DEF_MANUAL_PATH = "doc/manual.html";

	private JEditorPane pane = null;

	static String getTITLE() {
		return "Manual of " + ApplicationModel.SHORT_APPNAME;
	}

	static String getCOMMENT() {
		if (DEF_MANUAL_PATH.startsWith("http://"))
			return "This manual is located at " + DEF_MANUAL_PATH;
		return null;
	}

	/** It uses the default location */
	public ManualDialog(Frame owner) {
		this( owner, null );
	}
	
	/**
	 * @param owner The owner frame
	 * @param location The location as a path or from the classpath
	 */
	public ManualDialog(Frame owner, String location ) {
		super( owner, "Manual", getTITLE(), getCOMMENT(), null );
		setModal( false );

		try {
			
			String tmp = location;
			if ( tmp == null )
				tmp = DEF_MANUAL_PATH;
			URL url = null;
			if ( tmp.startsWith( "http:" ) )
				url = new URL( tmp );
			else
				url = ClassLoader.getSystemResource( tmp );
			
			pane = new JEditorPane();
			//pane = new JEditorPane( url );
			pane.setEditable(false);
			pane.setText("Reading " + DEF_MANUAL_PATH + " ...");

			JScrollPane sp = new JScrollPane(pane);
			sp.setPreferredSize(new Dimension(650, 500));
			setUI(sp);

			load(url);

		} catch (IOException exc) {
		}
	}

	private void load(URL url) {
		final URL url2 = url;
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					pane.setPage(url2);
				} catch (Exception exc) {
					pane.setText("Can't read " + url2);
				}
			}
		});
	}

	public void addNotify() {
		super.addNotify();
		pane.addHyperlinkListener(this);
	}

	public void removeNotify() {
		super.removeNotify();
		pane.removeHyperlinkListener(this);
	}

	public void hyperlinkUpdate(HyperlinkEvent e) {
		if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			pane.scrollToReference(e.getDescription().substring(1));
		}
	}

}
