package com.japisoft.framework.dockable;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import javax.swing.UIManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicStatusBar extends JToolBar {

	public BasicStatusBar() {
		super();
		setFloatable( false );
	}

	public void prepare( JDock dock ) {
		prepare( dock, false );
	}
	
	public void prepare( JDock dock, boolean reset ) {
		if ( reset )
			removeAll();
		JComponent c = dock.getDockingView();
		ArrayList known = new ArrayList();
		
		for ( int i = 0; i < c.getComponentCount(); i++ ) {
			Component c2 = c.getComponent( i );
			Windowable w = dock.componentToWindowable( c2 );
			if ( w != null ) {
				addWindowable( dock, w );
				known.add( w.getId() );
			}
		}

		// Add for hidden windows
		
		if ( dock.hasHiddenWindows() ) {
			Enumeration enume = 
				dock.hiddenWindows();
			while ( enume.hasMoreElements() ) {
				Windowable w = ( Windowable )enume.nextElement();
				addWindowable( dock, w );
				//known.add( w.getId() );
			}
		}
		
		for ( int i = 0; i < getComponentCount(); i++ ) {
			JButton btn = ( JButton )getComponent( i );
			HideShowAction a = ( HideShowAction )btn.getAction();
			resetButtonUI( btn, !known.contains( a.id ) );
		}
	}

	void addWindowable( JDock dock, Windowable w ) {
		
		// Search for this windowable

		JButton btn = getButtonById( w.getId() );

		if ( btn == null ) {
			HideShowAction action = 
				new HideShowAction( w.getId(), dock );
			if ( w.getIcon() != null )
				action.putValue( Action.SMALL_ICON, w.getIcon() );
			if ( w.getTitle() != null )
				action.putValue( Action.SHORT_DESCRIPTION, w.getTitle() );
			action.putValue( Action.NAME, w.getId() );
	
			btn = ( JButton )add( action );
			
			if ( dock.isUIReady() ) {
				dock.getView().invalidate();
				dock.getView().validate();
				invalidate();
				validate();
				repaint();
			}
		}
	}

	void resetButtonUI( JButton button, boolean hidden ) {
		
		if ( button != null ) {
			if ( hidden ) {
				button.setBackground( Color.gray );
			}
			else
				button.setBackground( UIManager.getColor( "button.background" ) );
		}
	}

	public void dispose() {
		removeAll();
	}
	
	private JButton getButtonById( String id ) {
		for ( int i = 0; i < getComponentCount(); i++ ) {
			JButton btn = ( JButton )getComponent( i );
			HideShowAction a = ( HideShowAction )btn.getAction();
			if ( id.equals( a.id ) )
				return btn;
		}
		return null;
	}

	class HideShowAction extends AbstractAction {
		
		private String id;
		private JDock dock;

		public HideShowAction( String id, JDock dock ) {
			this.id = id;
			this.dock = dock;
		}

		public void actionPerformed( ActionEvent e ) {
			if ( dock.isInnerWindowHidden( id ) ) {
				dock.showInnerWindow( id );				
			} else {
				dock.hideInnerWindow( id );			
			}
		}
	}

}
