package com.japisoft.framework.dockable;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JToolBar;

import com.japisoft.framework.dockable.action.ActionModel;
import com.japisoft.framework.dockable.action.common.CommonAction;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
class DefaultDockedFrame extends JFrame implements WindowListener {

	private Windowable panel;
	private Action source;
	private JToolBar tb;
	
	public DefaultDockedFrame( Action source, Windowable panel ) {
		this.source = source;
		this.panel = panel;
		JComponent compo = ( JComponent )panel.getContentPane();
		panel.getView().remove( compo );
		getContentPane().add( compo );
		setDefaultCloseOperation( JFrame.HIDE_ON_CLOSE );
		setTitle( panel.getTitle() );
		setSize( new Dimension( 400, 400 ) );
		addWindowListener( this );
		panel.repaint();
		DockManager.storeDefaultDockedFrame( this );
		if ( panel.getFrameBounds() != null )
			setBounds( panel.getFrameBounds() );
		
		// Insert toolbar ?
		boolean toolbar = false;
		ActionModel model = panel.getActionModel();
		for ( int i = 0; i < model.getActionCount(); i++ ) {
			if  ( !( model.getAction( i ) instanceof CommonAction ) ) {
				toolbar = true;
				break;
			}
		}
		if ( toolbar ) {
			tb = new JToolBar();
			for ( int i = 0; i < model.getActionCount(); i++ ) {
				if ( !( model.getAction( i ) instanceof CommonAction ) ) 
					tb.add( ComponentFactory.getComponentFactory().buildButton( model.getAction( i ) ) );
			}
			getContentPane().add( tb, BorderLayout.NORTH );
		}
	}

	public void windowActivated(WindowEvent e) {
	}

	public void undock() {
		removeWindowListener( this );
		JComponent compo = ( JComponent )getContentPane().getComponent( 0 );
		getContentPane().remove( compo );
		panel.setContentPane( compo );
		if ( source != null )
			source.setEnabled( true );		

		panel.setFrameBounds( getBounds() );		
		DockManager.unstoreDefaultDockedFrame( this );
		dispose();
	}

	public void windowClosing(WindowEvent e) {
		undock();
	}
	
	public void windowClosed(WindowEvent e) {
	}

	public void windowDeactivated(WindowEvent e) {
	}

	public void windowDeiconified(WindowEvent e) {
	}

	public void windowIconified(WindowEvent e) {
	}

	public void windowOpened(WindowEvent e) {
	}

	public void dispose() {
		super.dispose();
		panel = null;
		source = null;
		if ( tb != null ) {
			tb.removeAll();
			tb = null;
		}
	}

}
