package com.japisoft.framework.dockable;

import java.util.EventObject;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JDockEvent extends EventObject {

	private int type;
	private String id;

	public static final int INNERWINDOW_SELECTED = 0;
	public static final int INNERWINDOW_MAXIMIZED = 1;
	public static final int INNERWINDOW_MOVED = 2;
	public static final int INNERWINDOW_REMOVED = 3;
	public static final int INNERWINDOW_HIDDEN = 4;
	public static final int INNERWINDOW_SHOWN = 5;
	public static final int INNERWINDOW_EXTRACTED = 6;
	public static final int INNERWINDOW_RESTORED = 7;
	
	/**
	 * @param source The JDock source
	 * @param type look at the INNERWINDOW_... const
	 * @param id The inner window id */
	public JDockEvent( Object source, int type, String id ) {
		super( source );
		this.type = type;
		this.id = id;
	}

	/** @return the JDock container */
	public JDock getJDockSource() { return ( JDock )getSource(); }

	/** @return the inner window id */
	public String getId() { return id; }

	/**
	 * @return <code>INNERWINDOW_SELECTED</code>,<code>INNERWINDOW_MAXIMIZED</code>,<code>INNERWINDOW_MOVED</code>... */
	public int getType() { 
		return type;
	}

	public String toString() {
		String msg = id + " ";
		switch( type ) {
			case INNERWINDOW_SELECTED : msg += "selected"; break;
			case INNERWINDOW_MAXIMIZED : msg += "maximized"; break;
			case INNERWINDOW_MOVED : msg += "moved"; break;
			case INNERWINDOW_REMOVED : msg += "removed"; break;
			case INNERWINDOW_HIDDEN : msg += "hidden"; break;
			case INNERWINDOW_SHOWN : msg += "shown"; break;
			case INNERWINDOW_EXTRACTED : msg += "extracted"; break;
			case INNERWINDOW_RESTORED : msg += "restored"; break;
		}
		return msg;
	}

}
