package com.japisoft.framework.dockable.action;

import java.util.ArrayList;

import javax.swing.Action;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicActionModel extends ArrayList implements ActionModel {

	/** Simple way to create an <code>ActionModel</code> instance */
	public static ActionModel getInstance( Action a ) {
		return getInstance( new Action[] { a } );
	}

	/** Simple way to create an <code>ActionModel</code> instance */
	public static ActionModel getInstance( Action[] a ) {
		return new BasicActionModel( a );
	}

	public BasicActionModel() {
		super();
	}
	
	/** Reset the model with the following actions */
	public BasicActionModel( Action[] actions ) {
		if ( actions == null )
			throw new RuntimeException( "Null actions ? " );
		for ( int i = actions.length - 1; i >= 0; i-- ) {
			addAction( actions[ i ] );
		}
	}
	
	/** @return an action matching this class */
	public Action getActionByClass(Class className ) {
		for ( int i = 0; i < size(); i++ ) {
			Object o = get( i );
			if ( o.getClass() == className )
				return (Action)o;
		}
		return null;
	}

	/** @return an action at this index */
	public Action getAction( int index ) {
		return ( Action )get( index );
	}

	/** @return the number of action */
	public int getActionCount() {
		return size();
	}

	/** Add a new action */
	public void addAction( Action a ) {
		add( a );
		fireStateChanged();
	}
	
	/** Add an action at this index starting from 0 */
	public void addAction( Action a, int index ) {
		add( index, a );
		fireStateChanged();
	}

	/** Remove an action */
	public void removeAction( Action a ) {
		remove( a );
		fireStateChanged();
	}

	public void removeAll() {
		super.removeAll( this );
		fireStateChanged();
	}

	private ArrayList listeners = null;

	/** Add a listener for knowing the action model state change */
	public void addModelStateListener( ModelStateListener listener ) {
		if ( listeners == null )
			listeners = new ArrayList();
		listeners.add( listener );
	}

	/** Remove a known listener */
	public void removeModelStateListener( ModelStateListener listener ) {
		if ( listeners != null )
			listeners.remove( listener );
	}

	private void fireStateChanged() {
		if ( listeners != null ) {
			for ( int i = 0; i < listeners.size(); i++ ) {
				ModelStateListener listener = ( ModelStateListener )listeners.get( i );
				listener.modelModified( this );
			}
		}
	}

	public boolean isSeparator(int index) {
		return getAction( index ) == SEPARATOR;
	}	

	public String toString() {
		String s = "[";
		for ( int i = 0; i < size(); i++ ) {
			if ( i > 0 )
				s += ",";
			s += get( i );
		}
		s += "]";
		return s;
	}

}
