package com.japisoft.framework.ui;

import java.awt.Component;
import java.awt.Insets;
import java.awt.Point;

import javax.swing.JViewport;
import javax.swing.SwingUtilities;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AutoScroll { 
    Component comp; 
    Insets insets; 
    Insets scrollUnits; 
 
    public AutoScroll(Component comp, Insets insets){ 
        this(comp, insets, insets); 
    } 
 
    public AutoScroll(Component comp, Insets insets, Insets scrollUnits){ 
        this.comp = comp; 
        this.insets = insets; 
        this.scrollUnits = scrollUnits; 
    } 
 
    public void autoscroll(Point cursorLoc){ 
        JViewport viewport = getViewport(); 
        if(viewport==null) 
            return; 
        Point viewPos = viewport.getViewPosition(); 
        int viewHeight = viewport.getExtentSize().height; 
        int viewWidth = viewport.getExtentSize().width; 

        if((cursorLoc.y-viewPos.y)<insets.top){ // scroll up 
            viewport.setViewPosition( 
                    new Point(viewPos.x, 
                            Math.max(viewPos.y-scrollUnits.top, 0))); 
        } else if((viewPos.y+viewHeight-cursorLoc.y)<insets.bottom){ // scroll down 
            viewport.setViewPosition( 
                    new Point(viewPos.x, 
                            Math.min(viewPos.y+scrollUnits.bottom, 
                                    comp.getHeight()-viewHeight))); 
        } else if((cursorLoc.x-viewPos.x)<insets.left){ // scroll left 
            viewport.setViewPosition( 
                    new Point(Math.max(viewPos.x-scrollUnits.left, 0), 
                            viewPos.y)); 
        } else if((viewPos.x+viewWidth-cursorLoc.x)<insets.right){ // scroll right 
            viewport.setViewPosition( 
                    new Point(Math.min(viewPos.x+scrollUnits.right, comp.getWidth()-viewWidth), 
                            viewPos.y)); 
        } 
    } 
 
    JViewport getViewport(){ 
        return (JViewport)SwingUtilities.getAncestorOfClass(JViewport.class, comp); 
    } 
} 	
