package com.japisoft.framework.ui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.UIManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FastLabel extends JComponent implements PropertyChangeListener {
	private String text;
	private boolean border;

	public FastLabel() {
		super();
		setForeground( UIManager.getColor( "Label.foreground" ) );
		setBackground( UIManager.getColor( "Label.background" ) );
		setFont( UIManager.getFont( "Label.font" ) );
		setOpaque( true );
		addPropertyChangeListener( this );
	}

	public FastLabel( boolean withBorder ) {
		this();
		this.border = withBorder;
	}

	private boolean center;

	public FastLabel( boolean withBorder, 
			boolean center ) {
		this( withBorder );
		this.center = center;
	}

	private boolean error;

	public FastLabel( boolean withBorder, 
			boolean center, 
			boolean error ) {
		this( withBorder, center );
		this.error = error;
	}

	public void removeNotify() {
		super.removeNotify();
		removePropertyChangeListener( this );
	}	

	public void propertyChange(PropertyChangeEvent evt) {
		if ( "font".equals( evt.getPropertyName() ) )
			computeInnerSize();
	}	

	public void setText( String text ) {
		this.text = text;
		computeInnerSize();
		if ( error )
			setToolTipText( text );	
		repaint();
	}

	public String getText() {
		return text;
	}

	private Icon icon;
	
	public void setIcon( Icon icon ) {
		this.icon = icon;
		computeInnerSize();
	}

	public Icon getIcon() {
		return icon;
	}

	int xtext = 4;
	int xicon = 0;
	int hc = 0;

	boolean underlineMode = false;
	
	public void setUnderlineMode( boolean underline ) {
		this.underlineMode = underline;
		repaint();
	}

	public void paintComponent( Graphics gc ) {
		super.paintComponent( gc );
		

		Graphics2D g2d = ( Graphics2D )gc;
		g2d.setRenderingHint( 
			RenderingHints.KEY_TEXT_ANTIALIASING, 
			RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB 
		);		
		
		
		if ( isOpaque() ) {
			gc.setColor( getBackground() );
			gc.fillRect( 0, 0, getWidth(), getHeight() );
		}

		if ( border ) {
			gc.setColor( Color.GRAY );
			gc.draw3DRect( 2, 2, getWidth() - 4, getHeight() - 4, true );
		}
		if ( icon != null ) {
			icon.paintIcon( this, gc, 0, 0 );
		}
		if ( text != null ) {
			gc.setColor( error ? Color.RED : getForeground() );
			gc.drawString( text, xtext,  hc );
		}

		if ( underlineMode ) {
			if ( underlineColor != null )
				gc.setColor( underlineColor );
			gc.drawLine( xtext, hc + 2, preferredSize.width, hc + 2 );
		}
	}

	private Color underlineColor = null;
	
	public void setUnderlineColor( Color color ) {
		this.underlineColor = color;
	}

	Dimension preferredSize = null;

	public Dimension getPreferredSize() {
		if ( preferredSize == null )
			return super.getPreferredSize();
		return preferredSize;
	}

	private void computeInnerSize() {
		if ( getFont() == null )
			return;
		FontMetrics metrics = getFontMetrics( getFont() );		
		int fw = 0;
		xtext = 0;
		
		if ( text != null )
			fw = metrics.stringWidth( text );

		if ( center && 
				text != null ) {
			xtext = 2 + ( getWidth() - ( fw / 2 ) );
		}	
		
		if ( icon != null )
			xtext += ( icon.getIconWidth() + 2 );

		hc = Math.max( metrics.getAscent(), icon != null ? icon.getIconHeight() : 0 ) + 2;
		int width = fw;
		
		preferredSize = new Dimension( xtext + fw, hc );

		hc = ( icon != null ? hc - ( icon.getIconHeight() - metrics.getDescent() ) / 2 : hc );
	}

	public void invalidate() {
	}
	public void validate() {
	}
	public void layout() {
	}

}
