package com.japisoft.framework.ui.table;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.TableCellRenderer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BooleanTableCellRenderer implements TableCellRenderer {

	private static JCheckBox cb = new JCheckBox();		

	public BooleanTableCellRenderer() {
		super();
		cb.setHorizontalAlignment( SwingConstants.CENTER );
	}
	
	private Color background, foreground, background2;

	public BooleanTableCellRenderer( Color background, Color foreground ) {
		this();
		this.background = background;
		this.foreground = foreground;
		this.background2 = this.background;
	}

	public BooleanTableCellRenderer( Color background, Color background2, Color foreground ) {
		this( background, foreground );
		this.background2 = background2;
	}

	public Component getTableCellRendererComponent(
			JTable table, 
			Object value,
			boolean isSelected, 
			boolean hasFocus, 
			int row, 
			int column ) {
		
		if ( isSelected ) {
			cb.setBackground( table.getSelectionBackground() );
			cb.setForeground( table.getSelectionForeground() );
		} else {
			
			if ( background != null ) {
				if ( row % 2 == 0 )
					cb.setBackground( background2 );
				else
					cb.setBackground( background );
			} else {
				cb.setBackground( background == null ? table.getBackground() : background );
				cb.setForeground( foreground == null ? table.getForeground() : foreground );
			}
		}

		if ( table.getModel() instanceof EnableTableModel ) {
			boolean enabled =
				( ( EnableTableModel )table.getModel() ).isEnabled( row );
			if ( !enabled ) {
				if ( !isSelected )
					cb.setBackground( SharedProperties.DISABLED_BGCOLOR );
			}
		}

		if ( table.getModel() instanceof ErrorTableModel ) {
			boolean error = 
				( ( ErrorTableModel )table.getModel() ).hasError( row );
			if ( error ) {
				if ( !isSelected )
					cb.setBackground( SharedProperties.ERROR_BGCOLOR );
			}
		}
		
		cb.setSelected( ( ( Boolean )value ).booleanValue() );
		return cb;
	}	

}
