package com.japisoft.framework.ui.table;

import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.print.PrinterException;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.Action;

import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.TableModel;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.spreadsheet.Spreadsheet;
import com.japisoft.framework.spreadsheet.SpreadsheetFactory;
import com.japisoft.framework.ui.toolkit.FileManager;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ExportableTable extends JTable implements MouseListener {

	private JPopupMenu menu = new JPopupMenu();

	public ExportableTable() {
		super();
		initPopup();		
	}

	public ExportableTable(TableModel model) {
		super(model);
		initPopup();
	}

	private void initPopup() {
		menu.add( new ExportAction() );
		menu.add( new PrintAction() );
	}

	@Override
	public void addNotify() {
		super.addNotify();
		addMouseListener( this );
	}

	@Override
	public void removeNotify() {
		super.removeNotify();
		removeMouseListener( this );
	}

	public void mouseClicked(MouseEvent e) {
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
		if ( e .isPopupTrigger() ) {
			menu.show( e.getComponent(), e.getX(), e.getY() );
		}		
	}

	public void mouseReleased(MouseEvent e) {
		if ( e.isPopupTrigger() ) {
			menu.show( e.getComponent(), e.getX(), e.getY() );
		}				
	}

	private class PrintAction extends AbstractAction {
		public PrintAction() {
			putValue( Action.NAME, "Print..." );
			putValue( Action.LONG_DESCRIPTION, "Print this table" );
		}
		public void actionPerformed(ActionEvent e) {
			try {
			    boolean complete = print();
			    if (complete) {
			    	ApplicationModel.fireApplicationValue( "info", "Print completed" );
			    } else {
			    }
			} catch (PrinterException pe) {
				ApplicationModel.fireApplicationValue( "error", "Can't print : " + pe.getMessage() );
			}			
		}
	}
	
	private class ExportAction extends AbstractAction {
		public ExportAction() {
			putValue( Action.NAME, "Export..." );
			putValue( Action.LONG_DESCRIPTION, "Export this table" );
		}
		public void actionPerformed(ActionEvent arg0) {
			
			File selectedFile = FileManager.getSelectedFile(
				false, 
				new String[] { 
					"xlsx","xls","csv" 
				}, 
				new String[] { 
					"Microsoft Excel 2007 and later (*.xlsx)", "Microsoft Excel (*.xls)", "Comma-separated values File (*.csv)" 
				}
			);			
			
			if ( selectedFile != null ) {
				
				if ( FileManager.hasFileExt( selectedFile, "xlsx","xls","csv" ) ) {				
					try {
						Spreadsheet spreadsheet = SpreadsheetFactory.getInstance().getSpreadsheet( selectedFile );
						spreadsheet.reset( getModel() );
						spreadsheet.write( new FileOutputStream( selectedFile ));
					} catch( Exception exc ) {
						ApplicationModel.fireApplicationValue( "error", exc.getMessage() );
					}
				} else {
					try {
						BufferedWriter bw = new BufferedWriter( 
							new FileWriter( selectedFile ) 
						);
						try {
							int row = getModel().getRowCount();
							int col = getModel().getColumnCount();
							for ( int i = 0; i < row; i++ ) {
								if ( i > 0 )
									bw.newLine();
								for ( int j = 0; j < col; j++ ) {
									Object value = getModel().getValueAt( i, j );
									if ( value != null ) {
										if ( j > 0 )
											bw.write( "\t" );
										bw.write( value.toString() );
									}
								}
							}
						} finally {
							try {
								bw.close();
							} catch( Exception exc ) {
							}
						}
					} catch( IOException exc ) {

					}
				}

			}
		}
	}

}
