package com.japisoft.framework.ui.text;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;


/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DefaultTextFieldHandler implements FileTextFieldHandler {

	public String selectResource(  
			FileTextField source, 
			String previousPath, 
			boolean directoryMode,
			boolean fileMode, 
			boolean openedMode,
			String[] fileExt2, 
			String currentDir) {

		if ( fileExt2 != null && 
				fileExt2.length > 0 && 
					fileExt2[ 0 ] == null )
			fileExt2 = null;

		final String[] fileExt = fileExt2;		
		JFileChooser fc = new JFileChooser();
		
		if ( source.isMultipleSelectionMode() ) {
			fc.setMultiSelectionEnabled( true );
		}
		
		if ( directoryMode && fileMode ) {
			fc.setFileSelectionMode( JFileChooser.FILES_AND_DIRECTORIES );
		} else
		if ( directoryMode )
			fc.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
		else
			fc.setFileSelectionMode( JFileChooser.FILES_ONLY );

		if ( fileExt != null )
			fc.setFileFilter(new FileFilter() {
				public String getDescription() {
					String res = "";
					for ( int i = 0; i < fileExt.length; i++ ) {
						if ( i > 0 )
							res += " ";
						res += "*." + fileExt[ i ];
					}
					return res;
				}
				public boolean accept(File f) {
					if ( f == null )
						return false;
					boolean ok = f.isDirectory();
					if ( !ok  ) {
						for ( int i = 0; i < fileExt.length; i++ ) {
							if ( fileExt[ i ] == null )
								continue;
							if ( 
									f.toString().toLowerCase().endsWith( fileExt[ i ] ) )
								return true;
						}	
					}
					return ok;
				}
			});

		if ( previousPath != null && previousPath.length() > 0) {
			if ( previousPath.indexOf( ";" ) > 0 )
				previousPath = previousPath.split( ";" )[ 0 ];
			File f = new File( previousPath ).getParentFile();
			if ( f != null )
				fc.setCurrentDirectory( f );
		}
		else
		if (currentDir != null) {
			fc.setCurrentDirectory(new File(currentDir));
		} 

		if (openedMode) {

			if (fc.showOpenDialog( source ) == JFileChooser.APPROVE_OPTION) {
				
				if ( fc.isMultiSelectionEnabled() ) {
					File[] res = fc.getSelectedFiles();
					StringBuffer sb = new StringBuffer();
					for ( File f : res ) {
						if ( sb.length() > 0 )
							sb.append( ";" );
						sb.append( f.toString() );
					}
					return sb.toString();
				}

				return fc.getSelectedFile().toString();
			} else
				return null;

		} else {

			if (fc.showSaveDialog( source ) == JFileChooser.APPROVE_OPTION) {
				return fc.getSelectedFile().toString();
			} else
				return null;
		}
	}

	public void createResource( String path ) {

		if ( wrongPathCheck( path ) )
			return;

		
		boolean res = new File( path ).mkdirs();
		if ( !res ) {

			if ( new File ( path ).exists() ) {
				JOptionPane.showMessageDialog( null, "This path " + path + " already exists " + path, "Wrong operation", JOptionPane.WARNING_MESSAGE );
			} else
				JOptionPane.showMessageDialog( null, "Cannot create " + path, "Wrong operation", JOptionPane.WARNING_MESSAGE );

		}

	}

	private boolean wrongPathCheck( String path ) {
		if ( path == null ) {
			JOptionPane.showMessageDialog( null, "No path", "Wrong operation", JOptionPane.WARNING_MESSAGE );
			return true;
		}
		return false;
	}
	
	public void deleteResource( String path ) {
		
		if ( wrongPathCheck( path ) )
			return;
		
		if ( JOptionPane.showConfirmDialog( null, "Delete " + path + " ?" ) == JOptionPane.YES_OPTION ) {
			
			File f = new File( path );
			
			if ( !f.exists() ) {
				JOptionPane.showMessageDialog( null, "Path " + path + " doesn't exist", "Wrong operation", JOptionPane.WARNING_MESSAGE );					
			} else {
				boolean res = f.delete();
				if ( ! res ) {

					if ( f.isDirectory() && f.list() != null ) {
						JOptionPane.showMessageDialog( null, "This path " + path + " is not empty", "Wrong operation", JOptionPane.WARNING_MESSAGE );							
					} else
						JOptionPane.showMessageDialog( null, "Cannot delete " + path, "Wrong operation", JOptionPane.WARNING_MESSAGE );
				}
			}
		}

	}

	public boolean isCreateResourceManaged() {
		return true;
	}
	
	public boolean isDeleteResourceManaged() {
		return false;
	}
	
}
