package com.japisoft.framework.ui.text;

import java.awt.Color;

import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class EmptyColoredTextField extends JTextField {
	
	private Color emptyColor = null;
	private Color okColor = null;
	
	public EmptyColoredTextField(
			Color emptyColor,
			Color okColor ) {
		this.emptyColor = emptyColor;
		this.okColor = okColor;
		updateColor();
		setDocument( new CustomPlainDocument() );
	}

	/** It will used the color from the keys of the UIManager : 
	 * - EmptyColoredTextField.emptyColor
	 * - EmptyColoredTextField.okColor
	 */
	public EmptyColoredTextField() {
		emptyColor = UIManager.getColor( "EmptyColoredTextField.emptyColor" );
		okColor = UIManager.getColor( "EmptyColoredTextField.okColor" );
		updateColor();
		setDocument( new CustomPlainDocument() );
	}

	private void updateColor() {
		if ( emptyColor == null || okColor == null )
			return;
		if ( getDocument().getLength() == 0 ) {
			if ( getBackground() != emptyColor )
				setBackground( emptyColor );
		}
		else {
			if ( getBackground() != okColor )
				setBackground( okColor );
		}
	}
	
	class CustomPlainDocument extends PlainDocument {
		public void insertString(int offs, String str, AttributeSet a)
				throws BadLocationException {
			super.insertString(offs, str, a);
			updateColor();
		}
		public void remove(int offs, int len) throws BadLocationException {
			super.remove(offs, len);
			updateColor();
		}
	}
}
