package com.japisoft.framework.ui.text;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.InvalidPathException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.TransferHandler;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.ui.TitleLabel;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FileTextField 
				extends JPanel 
					implements ActionListener, PopupMenuListener {

	public JComboBox combo;
	private TitleLabel lbl;
	private JButton btn;
	private JButton btn2;
	private JButton btn3;
	private String[] fileExt;
	private PathBuilder builder;
	private JPanel pnlBtns;

	/**
	 * @param label Label for the component
	 * @param filePath The initial file path set
	 * @param fileExt The initial file ext */	
	public FileTextField( 
		String label, 
		String filePath, 
		String fileExt ) {
		this( label, filePath, new String[] { fileExt } );
	}

	/**
	 * @param label Label for the component
	 * @param filePath The initial file path set
	 * @param fileExt The initial file exts */	
	public FileTextField( 
		String label, 
		String filePath, 
		String[] fileExt ) {
		this( 
			label, 
			new String[] { filePath }, 
			fileExt,
			null );
		combo.setSelectedIndex( 0 );
	}

	/**
	 * @param label Label for the component
	 * @param filePath The initial file path set
	 * @param fileExt The initial file ext
	 * @param builder A delegate for building the combo box content */		
	public FileTextField( 
		String label, 
		String[] filePath, 
		String fileExt,
		PathBuilder builder ) {
		this( label, filePath, new String[] { fileExt }, builder );
	}

	public void setDocument( Document document ) {
		( ( JTextComponent )combo.getEditor().getEditorComponent() ).setDocument( document );
	}

	/**
	 * @param label Label for the component
	 * @param filePath The initial file path set
	 * @param fileExt The initial file exts
	 * @param builder A delegate for building the combo box content */	
	public FileTextField( 
			String label, 
			String[] filePath, 
			String[] fileExt,
			PathBuilder builder ) {
		this.fileExt = fileExt;
		this.builder = builder;
		if ( label != null )
			lbl = new TitleLabel( label );

		String[] tmp = filePath;
		if ( builder != null ) {
			tmp = builder.buildPathsChoice();
			if ( tmp == null )
				tmp = filePath;
		}
		if ( tmp != null )
			combo = new JComboBox( tmp );
		else
			combo = new JComboBox();

		combo.setEditable( true );
		setLayout( new BorderLayout() );
		if ( label != null )
			add( lbl, BorderLayout.WEST );

		add( combo, BorderLayout.CENTER );
		
		setBorder( null );
		
		pnlBtns = new JPanel();
		pnlBtns.setLayout( new BoxLayout( pnlBtns, BoxLayout.X_AXIS ) );
		pnlBtns.setBorder( null );
		URL url = ClassLoader.getSystemResource( "images/folder.png" );
		if ( url == null ) {
			btn = new JButton( "..." );
			System.err.println( "Can't find images/folder.png" );
		} else		
			btn = new JButton( new ImageIcon( url ) );

		btn.setMargin( null );
		btn.setBorder( null );
		btn.setToolTipText( "Choose a file from the dialog filesystem" );

		pnlBtns.add( btn );
		add( pnlBtns, BorderLayout.EAST );

		if ( combo.getEditor().getEditorComponent() instanceof JComponent ) {
			( ( JComponent )combo.getEditor().getEditorComponent() ).setTransferHandler(
					new SystemTransferHandler() );
		}
		
		badColor = Preferences.getPreference( "interface", "invalidFileColor", new Color( 208, 144, 144 ) );
	}

	public FileTextField(String filePath, String[] fileExt) {
		this(null, filePath, fileExt);
	}

	public FileTextField(String filePath, String fileExt) {
		this(null, filePath, new String[] { fileExt } );
	}
	
	public void setFilePath(String path) {
		combo.setSelectedItem( path );
	}
	
	/** Builder for selecting easily a file */
	public void setPathBuilder( PathBuilder pb ) {
		this.builder = pb;
	}

	/** Override the current document model of the combo */
	public void overrideDocument( Document d ) {
		((JTextField)combo.getEditor().getEditorComponent()).setDocument( d );
	}

	private String prefGroup = null;
	private String prefName = null;
	
	public void setPreferenceBinding( String group, String name ) {
		this.prefGroup = group;
		this.prefName = name;
		if ( group != null && name != null ) {
			setCurrentDirectory( 
					Preferences.getPreference( group, name, (String)null ) );
		}
	}

	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		combo.setEnabled(enabled);
		btn.setEnabled(enabled);
		if ( btn2 != null )
			btn2.setEnabled( enabled );
		if ( btn3 != null )
			btn3.setEnabled( enabled );
	}

	private String currentDir;

	public void setCurrentDirectory(String loc) {
		this.currentDir = loc;
	}

	private boolean directoryMode = false;

	public void setDirectoryMode( boolean directoryMode ) {
		this.directoryMode = directoryMode;
		if ( directoryMode ) {
			URL	url = ClassLoader.getSystemResource( 
			"images/folder_new.png" );
			if ( url == null ) {
				btn2 = new JButton( "New" );
				System.err.println( "Can't find images/folder_new.png" );
			} else
				btn2 = new JButton( new ImageIcon( url ) );
			btn2.setMargin( null );
			btn2.setBorder( null );
			btn2.setToolTipText( "Create a directory" );
			
			url = ClassLoader
				.getSystemResource( "images/folder_delete.png" );
	
			if ( url == null ) {
				btn3 = new JButton( "Del" );
				System.err.println( "Can't find images/folder_delete.png" );
			}
			else
				btn3 = new JButton( new ImageIcon( url ) );

			btn3.setToolTipText( "Delete a file or an empty directory" );
			btn3.setMargin( null );
			btn3.setBorder( null );

			pnlBtns.add( btn2 );
			pnlBtns.add( btn3 );
		} 
	}

	private boolean fileMode = true;
	
	public void setFileMode( boolean fileMode ) {
		this.fileMode = fileMode;
	}
	
	// private ActionListener customListener;

	public void setActionListener( ActionListener customListener ) {
		// this.customListener = customListener;
		addActionListener( customListener );
	}

	public void addActionListener( ActionListener listener ) {
		if ( listeners == null ) {
			listeners = new ArrayList<ActionListener>();
		}
		listeners.add( listener );
	}
		
	public void removeActionListener( ActionListener listener ) {
		if ( listeners != null ) {
			listeners.remove( listener );
		}
	}
	
	private List<ActionListener> listeners = null;
	
	public void addNotify() {
		super.addNotify();
		btn.addActionListener( this );
		if ( btn2 != null )
			btn2.addActionListener( this );
		if ( btn3 != null )
			btn3.addActionListener( this );
		combo.addActionListener( this );
		combo.addPopupMenuListener( this );
	}

	public void removeNotify() {
		super.removeNotify();
		btn.removeActionListener( this );
		if ( btn2 != null )
			btn2.removeActionListener( this );
		if ( btn3 != null )
			btn3.removeActionListener( this );
		combo.removeActionListener( this );
		combo.removePopupMenuListener( this );
	}

	public void popupMenuCanceled(PopupMenuEvent e) {}
	public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {}

	public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
		if ( builder != null ) {		
			String[] choice = builder.buildPathsChoice();
			if ( choice != null ) {
				combo.setModel(
					new DefaultComboBoxModel( choice ) );
				if ( choice.length == 1 )
					combo.setSelectedIndex( 0 );
			}
		}
	}

	public void setFileExt( String[] ext ) {
		this.fileExt = ext;
	}

	public void setFileExt( String ext ) {
		this.fileExt = new String[] { ext };
	}
		
	/** @return the current text content */
	public String getText() {
		String tmp = ( String )combo.getSelectedItem();
		if ( "".equals( tmp ) )
			return null;
		return tmp;
	}
	
	public String getText( String defaultValue ) {
		String tmp = getText();
		if ( tmp == null )
			return defaultValue;
		else
			return null;
	}

	private boolean wsCheck = false;
	
	// Check for whitespace inside the path
	public void setWhitespaceChecker( boolean check ) {
		this.wsCheck = check;
	}

	public void setText( String text ) {
		if  ( !directoryMode && 
					fileExt != null && 
						fileExt.length > 0 && 
							fileExt[ 0 ] != null && 
								text != null && 
									!"".equals( text ) && 
										( text.indexOf( "." ) == -1 ) )
			text = text + "." + fileExt[ 0 ];

		if ( wsCheck ) {
			for ( int i = 0; i < text.length(); i++ ) {
				if ( Character.isWhitespace( text.charAt( i ) ) ) {
					JOptionPane.showMessageDialog( 
							ApplicationModel.MAIN_FRAME, 
							"Please don't use path with whitespaces inside", "Illegal character in your path", JOptionPane.WARNING_MESSAGE );
					return;
				}
			}
		}
		
		combo.setSelectedItem( text );
		// => actionPerformed event !
		
		// Force it due to a refresh bug ??
		combo.getEditor().setItem( text );

		if ( prefGroup != null && 
				prefName != null && text != null ) {
			File f = new File( text );
			if ( f.isFile() )
				f = f.getParentFile();
			Preferences.setPreference( prefGroup, prefName, f.toString() );
		}
		
		checkContent();
	}
	
	private Color defaultColor = null;
	private Color badColor = null;
	
	private boolean checkMode = true;
	
	public void checkMode( boolean checkMode ) {
		this.checkMode = checkMode;
	}
	
	private void checkContent() {
		if ( !checkMode ) return;
		
		if ( Preferences.getPreference( "interface", "coloredBadPath", true ) ) {		
			String text = getText();
			if ( text != null && !"".equals( text ) ) {
				if ( !text.startsWith( "http" ) ) {
					try {
						Path path = Paths.get( text );
	
						if ( fileMode ) {					
							if ( !Files.exists( path ) ) {
								showError( true );
							} else
								showError( false );
						}
					} catch( InvalidPathException exc ) {
						showError( true );
					}
				}
			}
		}
	}
	
	private void showError( boolean showMode ) {
		JComponent editor = ( JComponent )combo.getEditor().getEditorComponent();
		
		if ( defaultColor == null )
			defaultColor = editor.getBackground(); 
		
		if ( showMode ) {
			editor.setBackground( badColor );
		} else
			editor.setBackground( defaultColor );
	}
	
	public String getCurrentDirectory() {
		if ( ( combo.getSelectedItem() != null && 
				getText().length() == 0 ) || 
					getText() == null )
			return null;
		return new File( getText() ).getParent();
	}

	private boolean openedMode = true;

	public void setOpenedMode(boolean openedMode) {
		this.openedMode = openedMode;
	}

	private boolean multipleSelectionMode = false;
	
	public void setMultipleSelectionMode( boolean mode ) {
		this.multipleSelectionMode = mode;
	}

	public boolean isMultipleSelectionMode() {
		return this.multipleSelectionMode;
	}
	
	public void actionPerformed(ActionEvent e) {
		
		if ( e.getSource() == btn2 ) {	// New
			getFileTextFieldHandler().createResource( getText() );
			
		} else
		if ( e.getSource() == btn3 ) {	// Delete

			getFileTextFieldHandler().deleteResource( getText() );
			
		} else
		if ( e.getSource() == btn ) {

			String res = 
				getFileTextFieldHandler().selectResource(
					this,
					getText(),
					directoryMode,
					fileMode,
					openedMode,
					fileExt,
					currentDir );
			
			if ( res != null ) {
				// Avoid double actionPerformed
				combo.removeActionListener( this );
				setText( res );
				combo.addActionListener( this );
			}
		}
		
		if ( getText() != null && !"".equals( getText() ) ) {
			fireCustomEvent( e );
			checkContent();
		}
	}
	
	private void fireCustomEvent( ActionEvent e ) {
		/*
		if ( customListener != null ) {
			customListener.actionPerformed( 
					new ActionEvent( this, e.getID(), e.getActionCommand() ) );
		}
		*/
		
		if ( listeners != null ) {
			ActionEvent evt = new ActionEvent( this, e.getID(), e.getActionCommand() );
			for ( ActionListener al : listeners ) {
				al.actionPerformed( evt );
			}
		}
	}
	
	private void fireCustomEvent() {
		fireCustomEvent( new ActionEvent( this, 0, "" ) );
	}
	
	////////////////////////////////////////////

	FileTextFieldHandler handler = null;

	public void setFileTextFieldHandler( FileTextFieldHandler handler ) {
		this.handler = handler;
		btn2.setEnabled( 
				handler == null || 
				handler.isCreateResourceManaged() );
		btn3.setEnabled( handler == null || 
				handler.isDeleteResourceManaged() );	
	}

	public FileTextFieldHandler getFileTextFieldHandler() {
		if ( handler == null )
			handler = new DefaultTextFieldHandler();
		return handler;
	}
	
	class SystemTransferHandler extends TransferHandler {

		public boolean canImport( JComponent comp, DataFlavor[] transferFlavors ) {
			return transferFlavors[ 0 ].isFlavorJavaFileListType() || transferFlavors[ 0 ].isFlavorTextType();
		}

		public boolean importData(JComponent arg0, Transferable arg1) {
			try {

				try {
					String fileName = ( String  )arg1.getTransferData( DataFlavor.stringFlavor );
					if ( fileName != null && ( arg0 instanceof JTextComponent ) ) {
						setText( fileName );
						return true;
					}
				} catch( UnsupportedFlavorException e ) {
				}

				java.util.List list = ( java.util.List )arg1.getTransferData(
						DataFlavor.javaFileListFlavor
					);
				if ( list != null && list.size() == 1 ) {
					String fileName = list.get( 0 ).toString();
					if ( arg0 instanceof JTextComponent ) {
						setText( fileName );
					}
					return true;
				}
			} catch (UnsupportedFlavorException e) {
			} catch (IOException e) {
			}
			return false;
		}
		
	}

}
