package com.japisoft.framework.ui.toolkit;

import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public final class Toolkit {

	/** Convert a keystroke to a string like 'ctrl p' */
	public static String getKeyStrokeView( KeyStroke k ) {
		int modifier = k.getModifiers();
		char c = (char)k.getKeyCode();
		String result = KeyEvent.getKeyModifiersText( modifier ) + "-" + 
			c;
		return result;
	}
	
	/** Force a selection of the first child of the refNode */
	public static void selectFirstChild( JTree t, TreeNode refNode ) {

		if ( refNode != null ) {
			
			if ( refNode.getChildCount() > 0 ) {

				selectNode( t, refNode.getChildAt( 0 ) );

			}

		}

	}

	/** Simple tree node selection */
	public static void selectNode( JTree t, TreeNode n ) {

		ArrayList al = new ArrayList();
		while ( n != null ) {
			
			al.add(0,n);
			n = n.getParent();
			
		}

		t.setSelectionPath( new TreePath( al.toArray() ) );
		
	}

	public static void addNodeOrdering( DefaultMutableTreeNode parentNode, MutableTreeNode childNode ) {

		boolean inserted = false;
		
		// Lexically ordering

		for  ( int i = 0; i < parentNode.getChildCount(); i++ ) {
			
			DefaultMutableTreeNode child = 
				(DefaultMutableTreeNode)parentNode.getChildAt( i );
			String currentGroup = child.toString();
			if ( currentGroup.compareTo( childNode.toString()) > -1 ) {
				// Insert it
				parentNode.insert( 
						childNode, i );
				inserted = true;
				break;
			}
			
		}

		if ( !inserted )
			parentNode.add( 
					childNode );

	}
	
	public static TreeNode[] getPathToRoot(TreeNode aNode, int depth) {
	     TreeNode[] retNodes;
	     if(aNode == null) {
	         if(depth == 0)
	          return null;
	         else
	          retNodes = new TreeNode[depth];
	     }
	     else {
	         depth++;
	         retNodes = getPathToRoot(aNode.getParent(), depth);
	         retNodes[retNodes.length - depth] = aNode;
	     }
	     return retNodes;
	}	
	
	public static TreePath getPath( TreeNode aNode ) {
		return new TreePath( getPathToRoot( aNode, 0 ) );
	}
	

	
}
