package com.japisoft.framework.xml.grammar.xsd;

import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.japisoft.framework.xml.grammar.Grammar;
import com.japisoft.framework.xml.grammar.GrammarElement;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSDGrammar implements Grammar {

	private Document doc = null;
	private String location = null;
	
	public XSDGrammar( String location ) throws Exception {
		this.location = location;
		
		DocumentBuilderFactory dbf = 
			DocumentBuilderFactory.newInstance();

		dbf.setNamespaceAware( true );
		dbf.setIgnoringComments( true );
		
		DocumentBuilder db = 
			dbf.newDocumentBuilder();
				
		doc = db.parse( location );
		
		resolveIncludeRedefineImport();
	}

	private void resolveIncludeRedefineImport() {

	}

	private List<GrammarElement> cache = null;
	
	public List<GrammarElement> getGlobalElements() {
		if ( cache == null )
			cache = XSDGrammar.getGlobalComponent( 
				this, 
				doc, 
				"element" 
			);
		return cache;
	}

	public GrammarElement getGlobalElement(String name) {
		List<GrammarElement> l = getGlobalElements();
		if ( l != null ) {
			for ( GrammarElement ge : l ) {
				if ( ge.getName().equalsIgnoreCase( name ) )
					return ge;
			}
		}
		return null;
	}	
	
	public static List<GrammarElement> getGlobalComponent( 
			XSDGrammar grammar, 
			Document doc, 
			String matching ) {
		ArrayList<GrammarElement> 
			al = new ArrayList<GrammarElement>();
		Element root = 
			doc.getDocumentElement();
		NodeList nl = 
			root.getChildNodes();
		for ( int i = 0; i < nl.getLength(); i++ ) {
			Node n = nl.item( i );
			if ( n instanceof Element ) {
				String name = n.getNodeName();
				int j = name.indexOf( ":" );
				if ( j > -1 )
					name = name.substring( j + 1 );
				if ( matching.equals( name ) ) {
						al.add(
								new XSDGrammarElement(
									grammar,
									( Element )n 
								)
						);
				}
			}
		}
		return al;
	}

	public String getLocation() {
		return location;
	}

	private String name = null;
	
	public String getName() {
		if ( name == null ) {
			if ( location == null )
				name = "Schema";
			else {
				int i = location.lastIndexOf( "/" );
				if ( i == -1 )
					i = location.lastIndexOf( "\\" );
				if ( i > -1 ) {
					name = location.substring( i + 1 );
				} else
					name = location;
			}
		}
		return name;
	}
	
	public static void main( String[] args ) throws Exception {
		String p = "c:/travail/soft/japisoft-editix-2014/distrib/install-content/samples/xmlform/purchaseOrder.xsd";
		XSDGrammar g = new XSDGrammar( p );
		List<GrammarElement> le = g.getGlobalElements();
		for ( GrammarElement ge : le ) {
			( ( XSDGrammarNode )ge ).dump();
		}
	}

}
