package com.japisoft.framework.xml.parser;

import org.xml.sax.*;

import com.japisoft.framework.collection.FastVector;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AttributesSax2Impl implements Attributes {
    public AttributesSax2Impl() {
	super();   
    }

    private FastVector vURI = new FastVector();
    private FastVector vLocalName = new FastVector();
    private FastVector vType = new FastVector();
    private FastVector vQName = new FastVector();
    private FastVector vValue = new FastVector();

    /** Add a new attribute */
    public void addAttribute( String prefix, String localName, String uri, String type, String value ) {
	vURI.add( uri );
	vLocalName.add( localName );
	vType.add( type );
	if ( prefix != null ) {
	    vQName.add( prefix + ":" + localName );
	} else
	    vQName.add( localName );
	vValue.add( value );
    }

    public int getLength () {
	return vURI.size();
    }

    public String getURI ( int index ) { 
	return (String)vURI.get( index );
    }

    public String getLocalName (int index) {
	return (String)vLocalName.get( index );
    }

    public String getQName (int index) {
	return (String)vQName.get( index );
    }

    public String getType (int index) {
	return (String)vType.get( index );
    }

    public String getValue (int index) {
	return (String)vValue.get( index );
    }

    public int getIndex (String uri, String localName) {
	for ( int i = 0; i < getLength(); i++ ) {
	    if ( uri.equals( vURI.get( i ) ) && 
		 localName.equals( vLocalName.get( i ) ) )
		return i;
	}
	return -1;
    }

    public int getIndex (String qName) {
	return vQName.indexOf( qName );
    }

    public String getType (String uri, String localName) {
	int i = getIndex( uri, localName );
	if ( i > -1 )
	    return (String)vType.get( i );
	return null;
    }

    public String getType (String qName) {
	int i = getIndex( qName );
	if ( i > -1 )
	    return (String)vType.get( i );
	return null;
    }

    public String getValue (String uri, String localName) {
	int i = getIndex( uri, localName );
	if ( i > -1 )
	    return (String)vValue.get( i );
	return null;
    }

    public String getValue (String qName) {
	int i = getIndex( qName );
	if ( i > -1 )
	    return (String)vValue.get( i );
	return null;
    }

    public String toString() {
	StringBuffer sb = new StringBuffer();
	sb.append( "[" );
	for ( int i = 0; i < getLength(); i++ ) {
	    if ( i > 0 )
		sb.append( "," );
	    sb.append( "{" ).append( "uri=" ).append( getURI( i ) ).append( ",local=" ).append( getLocalName( i ) );
	    sb.append( ",qname=" ).append( getQName( i ) ).append( ",type=" ).append( getType( i ) );
	    sb.append( ",value=" ).append( getValue( i ) ).append( "}" );
	}
	sb.append( "]" );
	return sb.toString();
    }

}

// AttributesImpl ends here
