package com.japisoft.framework.xml.parser.document;

import java.util.List;

import com.japisoft.framework.collection.FastVector;
import com.japisoft.framework.xml.parser.FPParser;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.document.DocumentBuilderException;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface DocumentBuilder {

	/** By default to false, if true, a list will available
	 * node is built. This is a flat view of the tree. It can be
	 * useful for searching node on location
	 * @param listMode */
	public void setFlatView( boolean mode );
	
	/** @return a flat view of the current tree */
	public List<FPNode> getFlatView();

    /** Check the closing node : <code>false</code> should be used for always well formed document */
    public void setCheckForCloseTag( boolean check );

    /** Reset the current node */
    public Object openNode( FPParser parser, String prefix, String prefixURI, String tag ) throws DocumentBuilderException;

    /** Close the current node, an exception is thrown for invalid tag name */
    public Object closeNode( FPParser parser, String prefix, String tag ) throws DocumentBuilderException;

    /** Close the last open node */
    public void closeNode( FPParser parser ) throws DocumentBuilderException;

    /** Reset attribute for the current node, an exception is thrown for no current node */
    public void setAttribute( String prefix, String prefixURI, String att, String value ) throws DocumentBuilderException;

    /** Set the namespace prefix and its URI */
    public void setNameSpace( String prefix, String prefixURI );

    /** Add a new node for the current node */
    public void addCommentNode( FPParser parser, String comment );

    /** Add a new text node for the current node */
    public Object addTextNode( FPParser parser, String text ) throws DocumentBuilderException;

    /** @return current document */
    public Document getDocument();

    /** Remove start and end white space for text */
    public void trimTextNode( boolean trim );
    
    /** @return <code>false</code> if the root element has not been closed */
    public boolean isTerminated();
    
    /** Called at the end of the parsing */
    public void dispose();
}

// DocumentBuilder ends here
