package com.japisoft.framework.xml.parser.dom;

import org.w3c.dom.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DocumentImpl extends NodeImpl implements org.w3c.dom.Document {
	com.japisoft.framework.xml.parser.document.Document d;

	private Element root;

	public DocumentImpl(com.japisoft.framework.xml.parser.document.Document d) {
		super();
		this.d = d;
		if ( d.getRoot() instanceof Element ) {
			root = ( Element )d.getRoot();
		}
	}

	public DocumentImpl( Element root ) {
		this.root = root;
	}

	@Override
	public boolean isLeaf() {
		return false;
	}
	
	com.japisoft.framework.xml.parser.document.Document getInnerDocument() {
		if ( d == null ) {
			d = new com.japisoft.framework.xml.parser.document.Document();
			d.setNodeFactory( new DomNodeFactory() );
		}
		return d;
	}

	/**
	 * The Document Type Declaration (see <code>DocumentType</code>) associated 
	 * with  this document. For HTML documents as well as XML documents without 
	 * a document type declaration this returns <code>null</code>. The DOM Level
	 *  1 does not support editing the Document Type Declaration, therefore 
	 * <code>docType</code> cannot be altered in any way.
	 */
	public DocumentType getDoctype() {
		return new DocumentTypeImpl();
	}

	public short getNodeType() {
		return Node.DOCUMENT_NODE;
	}

	/**
	 * The <code>DOMImplementation</code> object that handles this document. A 
	 * DOM application may use objects from multiple  implementations.
	 */
	public DOMImplementation getImplementation() {
		return new DOMImplementationImpl();
	}

	/**
	 * This is a convenience attribute that allows direct access to the child 
	 * node that is the root element of  the document. For HTML documents, this 
	 * is the element with the tagName "HTML".
	 */
	public Element getDocumentElement() {
		if ( root != null )
			return (Element)root;
		if ( d != null )
			return (Element) d.getRoot();
		return null;
	}

	public Node getFirstChild() {
		return getDocumentElement();
	}

	public Node getNextSibling() {
		return null;
	}

	/**
	 * Creates an element of the type specified. Note that the instance returned 
	 * implements the Element interface, so attributes can be specified 
	 * directly  on the returned object.
	 * @param tagName The name of the element type to instantiate. For XML, this 
	 *   is case-sensitive. For HTML, the  <code>tagName</code> parameter may 
	 *   be provided in any case,  but it must be mapped to the canonical 
	 *   uppercase form by  the DOM implementation. 
	 * @return A new <code>Element</code> object.
	 * @exception DOMException
	 *   INVALID_CHARACTER_ERR: Raised if the specified name contains an 
	 *   invalid character.
	 */
	public Element createElement(String tagName) throws DOMException {
		return (Element) (getInnerDocument().getNodeFactory().getTagNode(tagName));
	}

	/**
	 * Creates an empty <code>DocumentFragment</code> object. 
	 * @return A new <code>DocumentFragment</code>.
	 */
	public DocumentFragment createDocumentFragment() {
		return new DocumentFragmentImpl();
	}

	/**
	 * Creates a <code>Text</code> node given the specified string.
	 * @param data The data for the node.
	 * @return The new <code>Text</code> object.
	 */
	public Text createTextNode(String data) {
		return (Text) getInnerDocument().getNodeFactory().getTextNode(data);
	}

	/**
	 * Creates a <code>Comment</code> node given the specified string.
	 * @param data The data for the node.
	 * @return The new <code>Comment</code> object.
	 */
	public Comment createComment(String data) {
		return (Comment) getInnerDocument().getNodeFactory().getCommentNode(data);
	}

	/**
	 * Creates a <code>CDATASection</code> node whose value  is the specified 
	 * string.
	 * @param data The data for the <code>CDATASection</code> contents.
	 * @return The new <code>CDATASection</code> object.
	 * @exception DOMException
	 *   NOT_SUPPORTED_ERR: Raised if this document is an HTML document.
	 */
	public CDATASection createCDATASection(String data) throws DOMException {
		throw new DOMExceptionImpl(
			DOMException.NOT_SUPPORTED_ERR,
			"Not supported");
	}

	/**
	 * Creates a <code>ProcessingInstruction</code> node given the specified 
	 * name and data strings.
	 * @param target The target part of the processing instruction.
	 * @param data The data for the node.
	 * @return The new <code>ProcessingInstruction</code> object.
	 * @exception DOMException
	 *   INVALID_CHARACTER_ERR: Raised if an invalid character is specified.
	 *   <br>NOT_SUPPORTED_ERR: Raised if this document is an HTML document.
	 */
	public ProcessingInstruction createProcessingInstruction(
		String target,
		String data)
		throws DOMException {
		throw new DOMExceptionImpl(
			DOMException.NOT_SUPPORTED_ERR,
			"Not supported");
	}

	/**
	 * Creates an <code>Attr</code> of the given name. Note that the 
	 * <code>Attr</code> instance can then be set on an <code>Element</code> 
	 * using the <code>setAttribute</code> method. 
	 * @param name The name of the attribute.
	 * @return A new <code>Attr</code> object.
	 * @exception DOMException
	 *   INVALID_CHARACTER_ERR: Raised if the specified name contains an 
	 *   invalid character.
	 */
	public Attr createAttribute(String name) throws DOMException {
		return new AttrImpl(name, null);
	}

	/**
	 * Creates an EntityReference object.
	 * @param name The name of the entity to reference. 
	 * @return The new <code>EntityReference</code> object.
	 * @exception DOMException
	 *   INVALID_CHARACTER_ERR: Raised if the specified name contains an 
	 *   invalid character.
	 *   <br>NOT_SUPPORTED_ERR: Raised if this document is an HTML document. 
	 */
	public EntityReference createEntityReference(String name)
		throws DOMException {
		throw new DOMExceptionImpl(
			DOMException.NOT_SUPPORTED_ERR,
			"Not supported");
	}

	/**
	 * Returns a <code>NodeList</code> of all the <code>Element</code>s with a 
	 * given tag name in the order in which they would be encountered in a 
	 * preorder traversal of the <code>Document</code> tree. 
	 * @param tagname The name of the tag to match on. The special value "*" 
	 *   matches all tags.
	 * @return A new <code>NodeList</code> object containing all the matched 
	 *   <code>Element</code>s.
	 */
	public NodeList getElementsByTagName(String tagname) {
		if ( d == null || root == null )
			return null;
		return root.getElementsByTagName( tagname );
	}

	public Node importNode(Node importedNode, boolean deep)
		throws DOMException {
		NodeImpl n = (NodeImpl) importedNode.cloneNode(deep);
		n.setOwnerDocument(this);
		return n;
	}

	public Element createElementNS(String namespaceURI, String qualifiedName)
		throws DOMException {
		NodeImpl n = (NodeImpl) createElement(qualifiedName);
		n.setNamespaceURI(namespaceURI);
		return (Element) n;
	}

	public Attr createAttributeNS(String namespaceURI, String qualifiedName)
		throws DOMException {
		NodeImpl n = (NodeImpl) createAttribute(qualifiedName);
		n.setNamespaceURI(namespaceURI);
		return (Attr) n;
	}

	public NodeList getElementsByTagNameNS(
		String namespaceURI,
		String localName) {
		throw new RuntimeException("Not supported");
	}

	public Element getElementById(String elementId) {
		return null;
	}

	public short compareDocumentPosition(Node other) throws DOMException {
		// TODO Auto-generated method stub
		return 0;
	}

	public String getBaseURI() {
		// TODO Auto-generated method stub
		return null;
	}

	public Object getFeature(String feature, String version) {
		// TODO Auto-generated method stub
		return null;
	}

	public String getTextContent() throws DOMException {
		// TODO Auto-generated method stub
		return null;
	}

	public Object getUserData(String key) {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean isDefaultNamespace(String namespaceURI) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isEqualNode(Node arg) {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean isSameNode(Node other) {
		// TODO Auto-generated method stub
		return false;
	}

	public String lookupNamespaceURI(String prefix) {
		// TODO Auto-generated method stub
		return null;
	}

	public String lookupPrefix(String namespaceURI) {
		// TODO Auto-generated method stub
		return null;
	}

	public void setTextContent(String textContent) throws DOMException {
		// TODO Auto-generated method stub
		
	}

	public Object setUserData(String key, Object data, UserDataHandler handler) {
		// TODO Auto-generated method stub
		return null;
	}

	public Node adoptNode(Node source) throws DOMException {
		// TODO Auto-generated method stub
		return null;
	}

	public String getDocumentURI() {
		// TODO Auto-generated method stub
		return null;
	}

	public DOMConfiguration getDomConfig() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getInputEncoding() {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean getStrictErrorChecking() {
		// TODO Auto-generated method stub
		return false;
	}

	public String getXmlEncoding() {
		// TODO Auto-generated method stub
		return null;
	}

	public boolean getXmlStandalone() {
		// TODO Auto-generated method stub
		return false;
	}

	public String getXmlVersion() {
		// TODO Auto-generated method stub
		return null;
	}

	public void normalizeDocument() {
		// TODO Auto-generated method stub
		
	}

	public Node renameNode(Node n, String namespaceURI, String qualifiedName) throws DOMException {
		// TODO Auto-generated method stub
		return null;
	}

	public void setDocumentURI(String documentURI) {
		// TODO Auto-generated method stub
		
	}

	public void setStrictErrorChecking(boolean strictErrorChecking) {
		// TODO Auto-generated method stub
		
	}

	public void setXmlStandalone(boolean xmlStandalone) throws DOMException {
		// TODO Auto-generated method stub
		
	}

	public void setXmlVersion(String xmlVersion) throws DOMException {
		// TODO Auto-generated method stub
		
	}

}

// DocumentImpl ends here
