package com.japisoft.framework.xml.parser.dom;

import java.util.Vector;

import com.japisoft.framework.xml.parser.node.*;

import org.w3c.dom.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NodeListImpl implements NodeList {
	private FPNode n;

	/** @param n reference node */
	public NodeListImpl(FPNode n) {
		super();
		this.n = n;
	}

	private String filter;

	/** @param n reference node
	 * @param filter tag name to find */
	public NodeListImpl(FPNode n, String filter) {
		this(n);
		this.filter = filter;
		if  ( filter != null )
			prepareChildren();
	}

	private Vector children = null;

	private void prepareChildren() {
		if ( children == null )
			children = new Vector();
		prepareChildren( n );
	}

	private void prepareChildren( FPNode n ) {
		for ( int i = 0; i < n.childCount(); i++ ) {
			FPNode n2 = n.childAt( i );
			if ( n2.isTag() && filter.equals( n2.getNodeContent() ) )
				children.addElement( n2 );
		}
		for ( int i = 0; i < n.childCount(); i++ ) {
			FPNode n2 = n.childAt( i );
			if ( n2.isTag() )
				prepareChildren( n2 );		
		}
	}

	/** @return a node at index or <code>null</code> */
	public Node item(int index) {
		if (filter == null)
			return (Node) n.childAt(index);
		else
			return ( Node )children.elementAt( index );
	}

	/** @return the total number of node */
	public int getLength() {
		if (filter == null)
			return n.childCount();
		else {
			return children.size();
		}
	}

}

// NodeListImpl ends here
