package com.japisoft.framework.xml.parser.node;

import java.util.Iterator;

import com.japisoft.framework.xml.parser.document.Document;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface MutableNode {
	/** @return the current namespace URI */
	public String getNameSpaceURI();
    /** @return the current nameSpace prefix */
    public String getNameSpacePrefix();
    /** This node is a part of this nameSpace for this prefix and this URI */
    public void setNameSpace( String prefix, String uri );
    /** Add nameSpace support for this prefix and this URI */
    public void addNameSpaceDeclaration( String prefix, String uri );
	/** Check if this namespace prefix is declared in this node */
	public boolean isNamespaceDeclared( String prefix );
	/** @return a list of namespace prefix declaration */
	public Iterator<String> getNameSpaceDeclaration();
    /** Remove nameSpace for this prefix */
    public void removeNameSpaceDeclaration( String prefix );
    /** Reset the default namespace, using xmlns= value */
    public void setDefaultNamespace( String defaultNamespace );
    /** Return the default namespace declaration */
    public String getDefaultNamespace();
    /** Reset the node content, for tag node it means the tag name */
    public void setNodeContent( String content );
    /** @return the node content, for tag node it means the tag name */
    public String getNodeContent();
    /** Reset the node parent */
    public void setNodeParent( MutableNode node );
    /** Reset the node attribute */
    public void setNodeAttribute( String name, String value );
    /** Add a new child */
    public void addNode( MutableNode node );
    /** Reset the current starting tag line */
    public void setStartingLine( int line );
    /** Reset the current stopping line */
    public void setStoppingLine( int line );
    /** Reset the current staring offset */
    public void setStartingOffset( int offset );
    /** Reset the current stopping offset */
    public void setStoppingOffset( int offset );
    /** Particular way to know the tag declaration is in the &lt;A/&gt; form rather than &lt;A&gt;&lt;/A&gt; */
	public void setAutoClose( boolean closedLeaf );
	public void setDocument(Document doc);
	public boolean hasAttribute( String name );

}

// MutableNode ends here
