package com.japisoft.framework.xml.parser.tools;

import org.xml.sax.*;
import javax.xml.parsers.*;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class FPSAXParserFactory extends SAXParserFactory {
	public FPSAXParserFactory() {
		super();
	}

	public boolean getFeature(String name)
		throws
			ParserConfigurationException,
			SAXNotRecognizedException,
			SAXNotSupportedException {
		if ("http://xml.org/sax/features/namespaces".equals(name))
			return true;
		if ("http://xml.org/sax/features/namespace-prefixes".equals(name))
			return true;

		return false;
	}

	public void setFeature(String name, boolean value)
		throws
			ParserConfigurationException,
			SAXNotRecognizedException,
			SAXNotSupportedException {
		throw new SAXNotSupportedException("Unknown feature " + name);
	}

	public static SAXParserFactory newInstance() {
		return new FPSAXParserFactory();
	}

	public SAXParser newSAXParser()
		throws ParserConfigurationException, SAXException {
		return new CustomSAXParser();
	}

	///////////////////////////////////////

	public class CustomSAXParser extends SAXParser {
		public CustomSAXParser() {
			super();
		}

		public Parser getParser() {
			return new com.japisoft.framework.xml.parser.sax.SaxParser();
		}

		public XMLReader getXMLReader() {
			return new com.japisoft.framework.xml.parser.sax.Sax2Parser();
		}

		public Object getProperty(String name)
			throws SAXNotRecognizedException, SAXNotSupportedException {
			return null;
		}

		public boolean isNamespaceAware() {
			return true;
		}

		public boolean isValidating() {
			return false;
		}

		public void setProperty(String name, Object value)
			throws SAXNotRecognizedException, SAXNotSupportedException {
			throw new SAXNotSupportedException("Not support for " + name);
		}
	}

}
