package com.japisoft.framework.xml.parser.tools;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TraceContentHandler implements ContentHandler {
	Locator l;

	public void setDocumentLocator(Locator locator) {
		this.l = locator;
	}

	public void startDocument() throws SAXException {
		System.out.println("- start document");
	}

	public void endDocument() throws SAXException {
		System.out.println("- end document");
	}

	public void startPrefixMapping(String prefix, String uri)
		throws SAXException {
		System.out.println("- startPrefixMapping " + prefix + " / uri=" + uri);
	}

	public void endPrefixMapping(String prefix) throws SAXException {
		System.out.println("- endPrefixMapping " + prefix);
	}

	public void startElement(
		String uri,
		String localName,
		String qname,
		Attributes atts)
		throws SAXException {
		System.out.println(
			"* start tag uri="
				+ uri
				+ " localname="
				+ localName
				+ " qname="
				+ qname
				+ " "
				+ atts
				+ " at line:"
				+ l.getLineNumber()
				+ " col:"
				+ l.getColumnNumber());
	}

	public void endElement(String uri, String localName, String qname)
		throws SAXException {
		System.out.println(
			"* end tag uri="
				+ uri
				+ " localname="
				+ localName
				+ " qname="
				+ qname);
	}

	public void characters(char ch[], int start, int length)
		throws SAXException {
		System.out.println("+ text [" + new String(ch) + "]");
	}

	public void ignorableWhitespace(char ch[], int start, int length)
		throws SAXException {
	}

	public void processingInstruction(String target, String data)
		throws SAXException {
		System.out.println("! instruction " + target + " " + data);
	}

	public void skippedEntity(String name) throws SAXException {
		System.out.println( "Skipped entity :" + name );
	}

}
