package com.japisoft.framework.xml.parser.tools;

import java.util.List;
import java.util.Vector;

import javax.swing.tree.TreePath;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.collection.FastVector;
import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public final class XMLToolkit {

	/**
	 * @param doc
	 *            the current XML document
	 * @param line
	 *            the current line
	 * @return the best node for a line. It will work only for a flatView mode
	 */
	public static FPNode getNodeForLine(Document doc, int line) {
		List<FPNode> v;
		if ((v = doc.getFlatNodes()) == null)
			throw new RuntimeException(
					"Invalid usage for getNodeForLine : No flatView mode ");

		int i = 0;
		FPNode r = null;

		while (true) {
			FPNode n = (FPNode) v.get(i);
			if (n.getStartingLine() > line)
				break;
			if (line >= n.getStartingLine() && line <= n.getStoppingLine())
				r = n;
			else {
			}
			i++;
			if (i >= v.size()) {
				if (r == null)
					r = n;
				break;
			}
		}
		return r;
	}

	/**
	 * @param doc
	 *            the current XML document
	 * @param line
	 *            the document offset
	 * @return the best node for a document location. It will work only for a
	 *         flatView mode
	 */
	public static FPNode getNodeForOffset(Document doc, int offset) {
		List<FPNode> v;
		
		if (doc == null)
			return null;
		
		if ((v = doc.getFlatNodes()) == null) {
			throw new RuntimeException(
					"Invalid usage for getNodeForLine : No flatView mode ");
		}
		
		int i = 0;
		FPNode r = null;
		FPNode root = (FPNode) doc.getRoot();
		
		if (root.startingOffset > offset
				|| root.stoppingOffset < offset) {
			return null;
		}
	
		int delta = 0;
		
		while (true) {
			FPNode n = (FPNode) v.get(i);
			if (n.startingOffset > offset)
				break;
			
			if ( n.isText() )
				delta = 1;
			else
				delta = 0;
			if (offset > ( n.startingOffset - delta )
					&& offset <= n.stoppingOffset)
				r = n;
			else {
			}
			i++;
			if (i >= v.size()) {
				if (r == null)
					r = n;
				break;
			}
		}
		return r;
	}

	public static String[] getQualifiedPath(FPNode node) {
		if (node == null)
			return null;
		Vector v = new Vector();
		while (node != null) {
			v.insertElementAt(node.getQualifiedContent(), 0);
			node = node.getFPParent();
		}
		String[] r = new String[v.size()];
		v.copyInto(r);
		return r;
	}

	/** @return a swing tree path matching this node */
	public static TreePath getTreePath(FPNode n) {
		Vector v = new Vector();
		while (n != null) {
			v.insertElementAt(n, 0);
			n = n.getFPParent();
		}
		if (v.size() == 0)
			return null;
		return new TreePath(v.toArray());
	}

	/** @return the first child matching at least one tag from the parent */
	public static FPNode getMatchingNode(FPNode parent, String[] tags) {
		for (int i = 0; i < parent.childCount(); i++) {
			FPNode child = parent.childAt(i);
			if (child.isTag()) {
				String tag = child.getNodeContent();
				for (int j = 0; j < tags.length; j++) {
					if (tag.equals(tags[j])) {
						return child;
					}
				}
			}
		}
		return null;
	}

	public static String resolveCharEntities(String value) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < value.length(); i++) {
			char c = value.charAt(i);
			if (c == '<')
				sb.append("&lt;");
			else if (c == '>')
				sb.append("&gt;");
			else if (c == '\"')
				sb.append("&quot;");
			else if (c == '\'')
				sb.append("&apos;");
			else if (c == '&'
					&& !(i + 1 < value.length() && value.charAt(i + 1) == '#'))
				sb.append("&amp;");
			else if (c == '\n')
				sb.append("&#10;");
			else
				sb.append(c);
		}
		return sb.toString();
	}

}

