package com.japisoft.framework.xml.refactor;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.refactor.elements.AttributeRefactor;
import com.japisoft.framework.xml.refactor.elements.CommentRefactor;
import com.japisoft.framework.xml.refactor.elements.ElementRefactor;
import com.japisoft.framework.xml.refactor.elements.EntityRefactor;
import com.japisoft.framework.xml.refactor.elements.NamespaceRefactor;
import com.japisoft.framework.xml.refactor.elements.PIRefactor;
import com.japisoft.framework.xml.refactor.elements.PrefixRefactor;
import com.japisoft.framework.xml.refactor.elements.RefactorAction;
import com.japisoft.framework.xml.refactor.elements.RefactorObj;
import com.japisoft.framework.xml.refactor.ui.RefactorTable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public final class RefactorManager {
    private static ArrayList refactors;
    private static HashMap mapNameRefactorClass;

    static {
        refactors = new ArrayList();
        mapNameRefactorClass = new HashMap();
        addRefactor( new ElementRefactor() );
        addRefactor( new PrefixRefactor() );
        addRefactor( new NamespaceRefactor() );
        addRefactor( new AttributeRefactor() );
        addRefactor( new EntityRefactor() );
        addRefactor( new CommentRefactor() );
        addRefactor( new PIRefactor() );
    }

    public static void addRefactor( RefactorObj ro ) {
    	mapNameRefactorClass.put(
    			ro.getName(),
    			ro.getClass() );
        refactors.add( ro );
    }

    public static void addRefactorAt( RefactorObj ro, int index ) {
    	mapNameRefactorClass.put(
    			ro.getName(),
    			ro.getClass() );
    	refactors.add( index, ro );
    }

    public static void removeNonDefaultRefactors() {
    	Iterator it = refactors.iterator();
    	while ( it.hasNext() ) {
    		RefactorObj obj = ( RefactorObj )it.next();
    		if ( !obj.isDefault() )
    			it.remove();
    	}
    }

    public static void removeRefactor( RefactorObj ro ) {
        refactors.remove( ro );
    }

    public static void removeRefactorByName( String name ) {
    	for ( int i = 0; i < getRefactorCount(); i++ ) {
    		if ( getRefactor( i ).getName().equals( name ) ) {
    			refactors.remove( getRefactor( i ) );
    			break;
    		}
    	}
    }

    public static int getRefactorCount() {
        return refactors.size();
    }

    public static RefactorObj getRefactor( int i ) {
        return ( RefactorObj )refactors.get( i );
    }

    public static String getRefactorName( int i ) {
        return getRefactor( i ).getName();
    }
    
    public static RefactorObj getRefactor( String name ) {
    	for ( int i = 0; i < getRefactorCount(); i++ ) {
    		if ( name.equals( getRefactorName( i ) ) )
    			return getRefactor( i );
    	}
    	return null;
    }
    
    public static RefactorObj[] getRefactors() {
        RefactorObj[] ro = new RefactorObj[ refactors.size() ];
        for ( int i = 0; i < ro.length; i++ )
            ro[ i ] = ( RefactorObj )refactors.get( i );
        return ro;
    }
    
    public static String[] getRefactorActions( String name ) {
        for ( int i = 0; i < getRefactorCount(); i++ ) {
            RefactorObj ro = getRefactor( i );
            if ( ro.getName().equals( name ) )
                return ro.getActions();
        }
        return null;
    }

    public static RefactorObj buildRefactorObj( 
    		String name, 
    		RefactorAction action ) throws Exception {
    	Class cl = ( Class )mapNameRefactorClass.get( name );
    	if ( cl == null )
    		throw new RuntimeException( "Invalid class for name " + name );
    	RefactorObj ro = ( RefactorObj )cl.newInstance();
    	ro.setRefactorAction( action );
    	return ro;
    }

}
