package com.japisoft.framework.xml.refactor.elements;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.refactor.ui.RefactorTable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class NamespaceRefactor extends AbstractRefactor {
	private static final String RENAME_ACTION = "(V1) RENAME TO (V2)";
	private static final String DELETE_ACTION = "DELETE (V1)";

	public static String[] ACTIONS = new String[] {
		RENAME_ACTION,
		DELETE_ACTION,
	};

	public NamespaceRefactor() {
		super( ANY );
	}

        public String[] getActions() {
            return ACTIONS;
        }

        public String getName() {
            return "Namespace";
        }

	protected Node refactorIt( Node node, RefactorAction ra ) {

		if ( RENAME_ACTION.equals( ra.getAction() ) ) {

			if ( node.getNodeType() == Node.ATTRIBUTE_NODE ) {
				
				if ( node.getNodeName().startsWith( "xmlns:" ) || node.getNodeName().equals( "xmlns" ) ) {
					
					if ( ra.matchOldValue( node.getNodeValue() ) ) {
						
						if ( !ra.isNewValueEmpty() )
							
							node.setNodeValue( ra.getNewValue() );
						
					}

				}
				
			}
			
		} else
		if ( DELETE_ACTION.equals( ra.getAction() ) ) {

			if ( node.getNodeType() == Node.ATTRIBUTE_NODE ) {
			
				if ( node.getNodeName().equals( "xmlns" ) ) {
	
					if ( ra.matchOldValue( node.getNodeValue() ) )
						return null;
					
				} 
			
			}

			if ( node.getNodeType() == Node.ELEMENT_NODE ) {

				// Check for attributs
				
				NamedNodeMap map = node.getAttributes();
				if ( map != null ) {
					for ( int i = 0; i < map.getLength(); i++ ) {
						
						Node attNode = map.item( i );
						
						if ( attNode.getNodeName().startsWith( "xmlns:" ) ) {
							
							if ( ra.matchOldValue( attNode.getNodeValue() ) ) {

								( ( Element )node ).setAttribute( attNode.getNodeName(), null );
								map.removeNamedItem( attNode.getNodeName() );
								
								// Prefix name

								int j = attNode.getNodeName().indexOf( ":" );
								String prefix = attNode.getNodeName().substring( j + 1 );

								if ( prefix.equals( node.getPrefix() ) ) {
									// Remove it
									node.setPrefix( null );
								}
									
								// Check for descendant
								Element e = ( Element )node;
								NodeList nl = e.getElementsByTagName( "*" );

								for ( int k = 0; k < nl.getLength(); k++ ) {
									Node n = ( Node )nl.item( k );
									if ( prefix.equals( n.getPrefix() ) ) {
										n.setPrefix( null );
									}
									cleanAttributesWithPrefix( n, prefix );
								}
								
							}
							
						}
						
					}
				}
				
			}
						
		}

		return node;
	}

	private void cleanAttributesWithPrefix( Node n, String prefix ) {

		NamedNodeMap nnm = n.getAttributes();
		for ( int i = 0; i < nnm.getLength(); i++ ) {
			
			Node n2 = nnm.item( i );
			if ( n2.getNodeName().startsWith( prefix + ":" ) ) {

				String value = n2.getNodeValue();
				nnm.removeNamedItem( n2.getNodeName() );
				( ( Element )n ).setAttribute( n2.getNodeName().substring( prefix.length() + 1 ), value ); 

			}
			
		}
		
	}

	public void initTable( RefactorTable table, FPNode context ) {
		if ( context.hasAttribute( "xmlns" ) ) {
			table.init( 0, context.getAttribute( "xmlns" ) );
		}
		if ( context.getNameSpaceURI() != null ) {
			table.init( 0, context.getNameSpaceURI() );
		}
	}	

}
