package com.japisoft.framework.xml.refactor2.elements.xslfo;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.refactor2.AbstractRefactor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class PageMasterNameRefactor extends AbstractRefactor {

	public boolean requireNewValue() {
		return true;
	}

	public boolean process(FPNode node) {

		if ( node.hasAttribute( "master-reference" ) ) {
			oldValue = node.getAttribute( "master-reference" );
			return true;
		}

		if ( node.hasAttribute( "master-name" ) ) {
			oldValue = node.getAttribute( "master-name" );
			return true;
		}

		return false;
	}

	public String getTitle(FPNode node) {
		return "The page master '" + oldValue + "'";
	}

	public void startElement(
			String uri, 
			String localName, 
			String qName, 
			Attributes atts ) throws SAXException {

		if ( atts != null && 
					atts.getLength() > 0 && 
						atts.getValue( "master-name" ) != null && 
					!"".equals( atts.getValue( "master-name" ) ) ) {
			atts = new AttributesProxy( atts, "master-name" );
		}

		if ( atts != null && 
				atts.getLength() > 0 && 
					atts.getValue( "master-reference" ) != null && 
						!"".equals( atts.getValue( "master-reference" ) ) ) {
			atts = new AttributesProxy( atts, "master-reference" );
		}

		super.startElement(uri, localName, qName, atts);
	}

	// ---------------------------------------------------------------
	
	class AttributesProxy implements Attributes {

		private Attributes ref;
		private String attName;
		
		AttributesProxy( Attributes ref, String attName ) {
			this.ref = ref;
			this.attName = attName;
		}

		public int getIndex(String uri, String localName) {
			return ref.getIndex( uri, localName );
		}

		public int getIndex(String qName) {
			return ref.getIndex( qName );
		}

		public int getLength() {
			return ref.getLength();
		}

		public String getLocalName(int index) {
			return ref.getLocalName( index );
		}

		public String getQName(int index) {
			return ref.getQName( index );
		}

		public String getType(int index) {
			return ref.getType( index );
		}

		public String getType(String uri, String localName) {
			return ref.getType( uri, localName );
		}

		public String getType(String qName) {
			return ref.getType( qName );
		}

		public String getURI(int index) {
			return ref.getURI( index );
		}

		private String processValue( String name, String value ) {
			if ( attName.equals( name ) && 
					value.equals( oldValue ) )
				return newValue;
			return value;
		}

		public String getValue(int index) {
			return processValue( ref.getQName( index ), ref.getValue( index ) );
		}

		public String getValue(String uri, String localName) {
			return processValue( localName, ref.getValue( uri, localName ) );
		}

		public String getValue(String qName) {
			return processValue( qName, ref.getValue( qName ) );
		}

	}

}
