package com.japisoft.stylededitor.model;

import javax.swing.text.AttributeSet;
import javax.swing.text.Document;
import javax.swing.text.Element;

import org.w3c.dom.Node;

import com.japisoft.framework.css.Property;
import com.japisoft.framework.css.Rule;
import com.japisoft.framework.css.Toolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class BasicNodeElement implements NodeElement {

	protected Node node = null;
	protected Document document = null;
	private int startOffset = 0, endOffset = 0;
	
	public BasicNodeElement( 
			Document document, 
			Node node, 
			int startOffset ) {
		this.document = document;
		this.node = node;
		this.startOffset = startOffset;
		this.endOffset = startOffset;		
	}

	public AttributeSet getAttributes() {
		return null;
	}

	public Document getDocument() {
		return document;
	}

	public Node getNode() {
		return node;
	}

	public Element getElement( int index ) {
		return null;
	}

	public int getElementCount() {
		return 0;
	}

	public int getElementIndex( int offset ) {
		return -1;
	}

	public int getStartOffset() {
		return startOffset;
	}

	public void setStartOffset( int offset ) {
		this.startOffset = offset;
	}
	
	public void setEndOffset( int offset ) {
		this.endOffset = offset;
	}

	public int getEndOffset() {
		return endOffset;
	}
	
	public String getName() {
		return "<" + node.getNodeName() + ">";
	}

	public Element getParentElement() {
		return document.getDefaultRootElement();
	}

	public boolean isLeaf() {
		return true;
	}
	
	public void setData( String name, Object value ) {
		node.setUserData( name, value, null );
	}

	public Object getData( String name ) {
		return node.getUserData( name ); 
	}

	public boolean isDOMElement() {
		return node instanceof org.w3c.dom.Element;
	}

	public Object getCSSProperty( String name, Object defaultValue ) {
		Rule r = ( Rule )node.getUserData( "css" );
		if ( r == null && Toolkit.isInheritedProperty( name ) ) {
			Node tmpNode = node;
			while ( !( tmpNode instanceof Document ) ) {
				tmpNode = tmpNode.getParentNode();
				if ( tmpNode == null )
					break;
				Rule rTmp = ( Rule )tmpNode.getUserData( "css" );
				if ( rTmp != null ) {
					if ( r == null )
						r = rTmp;
					else
						r = r.merge( rTmp );
				}
			}
			if ( r == null )
				r = Rule.EMPTY_RULE;
			node.setUserData( "css", r, null );
		}
		if ( r == null )
			r = Rule.EMPTY_RULE;
		Property p = r.getProperty( name );
		if ( p == null ) {
			return defaultValue;
		}
		return p.getValue();
	}

}
