package com.japisoft.stylededitor.model;

import java.util.ArrayList;
import java.util.List;

import javax.swing.text.AttributeSet;
import javax.swing.text.Document;
import javax.swing.text.Element;

import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.TreeWalker;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DocumentElement extends BasicNodeElement {

	private List<NodeElement> children = null;

	public DocumentElement( 
			Document document, 
			org.w3c.dom.Document root ) {
		super( document, root, 0 );
		resetContent( root );
	}

	private int endOffset;
	
	private NodeElement rootElement;
	
	public NodeElement getRootElement() {
		return rootElement;
	}

	public void resetContent( org.w3c.dom.Document doc ) {
		children = null;
		rootElement = null;

		int startOffset = 0;
		DocumentTraversal traversal = ( DocumentTraversal )doc;

		TreeWalker tw = traversal.createTreeWalker( 
				doc.getDocumentElement(), 
				NodeFilter.SHOW_ALL, 
				null, 
				false );

		Node n = null;

		// Split text node for carriage return

		ArrayList<Text> splittable = null;
		ArrayList<Text> stripSpace = null;
		
		while ( ( n = tw.nextNode() ) != null ) {
			if ( n instanceof Text ) {
				String value = n.getNodeValue();
				
				// Trim empty space
				if ( "".equals( value.trim() ) ) {
					if ( !( n.getParentNode().getLastChild() == n && n.getParentNode().getFirstChild() == n ) ) { 
						if ( stripSpace == null ) {
							stripSpace = new ArrayList<Text>();
						}
						stripSpace.add( ( Text )n );
						continue;
					}
				}

				if ( value.contains( "\n" ) && !"\n".equals( value )) {
					if ( splittable == null ) {
						splittable = new ArrayList<Text>();
					}
					splittable.add( ( Text )n );
				}
			}
		}
		
		if ( splittable != null ) {
			for ( Text toSplit : splittable ) {
				org.w3c.dom.Element parent = 
					( org.w3c.dom.Element )toSplit.getParentNode();

				String value = toSplit.getNodeValue();
				String part = "";

				char c = 0;
				
				for ( int i = 0; i < value.length(); i++ ) {
					c = value.charAt( i );
					if ( c == '\n' ) {
						Text newText = doc.createTextNode( part ); 	// + "\n" );
						parent.insertBefore( newText, toSplit );
						part = "";
					} else {
						part += c;
					}
				}

				if ( !"".equals( part ) ) {
					Text newText = doc.createTextNode( part );
					parent.insertBefore( newText, toSplit );					
				} else {
					// Text newText = doc.createTextNode( "" );
					// parent.insertBefore( newText, toSplit );
				}

				toSplit.getParentNode().removeChild( toSplit );
			}
		}

		if ( stripSpace != null ) {
			for ( Text t : stripSpace ) {
				t.getParentNode().removeChild( t );
			}
		}

		/////////////////////////////////////////////////////////////////////

		tw = traversal.createTreeWalker( 
			doc, 
			NodeFilter.SHOW_ALL, 
			null, 
			false 
		);		

		n = null;

		while ( ( n = tw.nextNode() ) != null ) {
			NodeElement element = null;
			if ( n instanceof Text ) {
				element = new TextElement(
					document, 
					( Text )n, 
					startOffset 
				);	
			} else {
				element = new BasicNodeElement(
					document,
					n,
					startOffset
				);
				
				if ( n instanceof org.w3c.dom.Element ) {
					if ( rootElement == null )
						rootElement = element;
				}
				
			}
			addChild( element );
			
			if ( element.getEndOffset() > startOffset ) {
				startOffset = element.getEndOffset() + 1;
			}

			// For the document
			endOffset = element.getEndOffset();
		}
	}

	public void addChild( NodeElement e ) {
		if ( children == null )
			children = new ArrayList<NodeElement>();
		children.add( e );
		
		e.getNode().setUserData( "element", e, null );
	}

	public AttributeSet getAttributes() {
		return null;
	}

	public Document getDocument() {
		return document;
	}

	public Element getElement( int index ) {
		if ( children == null || index < 0 )
			return null;
		return children.get( index );
	}

	public int getElementCount() {
		if ( children == null )
			return 0;
		return children.size();
	}

	public int getElementIndex( int offset ) {
		int result = -1;
		for ( int i = 0; i < getElementCount(); i++ ) {
			Element child = getElement( i );
			if ( !( child instanceof TextElement ) ) {
				continue;
			}
			if ( offset >= child.getStartOffset() && 
					offset <= child.getEndOffset() ) {				
				result = i;
				break;
			} else
				if ( child.getStartOffset() > offset ) {
					break;
				}
		}
		return result;
	}

	public int getStartOffset() {
		return 0;
	}

	public int getEndOffset() {
		return this.endOffset;
	}

	public String getName() {
		return "root";
	}

	public Element getParentElement() {
		return null;
	}

	public boolean isLeaf() {
		return children != null && children.size() > 0;
	}
	
}
