package com.japisoft.xflows;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.framework.xml.parser.FPParser;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.framework.xml.parser.walker.NodeNameCriteria;
import com.japisoft.framework.xml.parser.walker.TreeWalker;
import com.japisoft.xflows.task.Task;
import com.japisoft.xflows.task.ui.builder.ScenarioBuilder;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XFlowsApplicationModel extends ApplicationModel {
	public static XFlowsApplicationModel ACCESSOR = null;

	XFlowsApplicationModel() {
		ACCESSOR = this;
	}

	private ArrayList list;

	public ArrayList getTasks() {
		if ( list == null )
			list = new ArrayList();
		return list;
	}

	private LoggerModel loggerModel;
	
	public LoggerModel getLogger() {
		if ( loggerModel == null ) {
			loggerModel = new LoggerModel();
		}
		return loggerModel;
	}

	public void setLogger( LoggerModel model ) {
		this.loggerModel = model;
	}

	private static boolean modified = false;
	
	public static void setModified() {
		modified = true;
	}

	public static boolean isModified() {
		return modified;
	}

	public void reload() {
		if ( lastComponent != null )
			setCurrentApplicationComponent( lastComponent );
	}

	public void newProject() {
		currentProjectFile = null;
		list = new ArrayList();
		loggerModel = new LoggerModel();
		reload();
	}

	private ApplicationComponent lastComponent = null;

	public void setCurrentApplicationComponent( 
			ApplicationComponent component ) {
		this.lastComponent = component;
		component.stopEditing();
		component.setApplicationModel( this );
		modified = false;
	}

	private File currentProjectFile = null;

	public File getCurrentProjectFile() { return currentProjectFile; }
	
	public void store( File file ) throws IOException {
		
		
	}
	
	public void store() throws IOException {
		if ( currentProjectFile == null )
			throw new RuntimeException( "Invalid usage" );
		store( currentProjectFile );
	}

	public void read( File file ) throws Exception {
		this.currentProjectFile = file;
		FPParser p = new FPParser();
		FPNode root = ( FPNode )p.parse(new FileReader( file )).getRoot();
		TreeWalker walker = new TreeWalker( root );
		Enumeration enu = walker.getTagNodeByName( "task", false );
		list = new ArrayList();
		while ( enu.hasMoreElements() ) {
			FPNode child = ( FPNode )enu.nextElement();
			Task t = new Task();
			t.updateFromXML( child );
			list.add( t );
		}
		
		FPNode logger = walker.getOneNodeByCriteria( 
				new NodeNameCriteria( "logger" ), false );

		if ( logger != null ) {
			loggerModel = new LoggerModel();
			loggerModel.updateFromXML( logger );
		}
		reload();
		modified = false;
	}
	
	public void init() {
		if ( Preferences.getPreference(
				"interface",
				"restoreLastProject",
				true ) ) {
			String lastProject = 
				Preferences.getPreference(
					Preferences.SYSTEM_GP,
					"lastProjectFile",
					( String )null );
			if ( lastProject != null ) {
				File f = new File( lastProject );
				if ( f.exists() ) {
					try {
						read( f );
					} catch (Exception e) {
					}
				}
			}
		}
	}

	public void stop() {
		if ( currentProjectFile != null )
			Preferences.setPreference(
				Preferences.SYSTEM_GP,
				"lastProjectFile",
				currentProjectFile.toString() );
	}

}
