package com.japisoft.xflows.task;

import java.io.File;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface TaskContext {

	public Task currentTask();
	
	/** Enabled/Disabled the information output */
	public void setQuietInfoMode( boolean quiet );

	/** Enabled/Disabled the warning output */
	public void setQuietWarningMode( boolean quiet );

	/** Enabled/Disabled the error output */
	public void setQuietErrorMode( boolean quiet );

	/** Add a warning message in the log */
	public void addWarning( String message );

	/** Add an information in the log */
	public void addInfo( String message );

	public boolean hasErrorFound();

	/** Add an error message in the log */
	public void addError( String message );

	/** @return a parameter value from the UI part */
	public String getParam( String name );

	/** 
	 * @return a parameter value from the UI part */
	public String getParam( String name, String def );

	/** @return a parameter with a directory path */
	public String getParamForPath( String name );
		
	public boolean hasParam( String name );
	
	public TaskParams getParams();
	
	/** Interrupt the task ? */
	public void interrupt();
	
	public boolean isInterrupted();
	
	/** Reset the current source file */
	public void setCurrentSourceFile( File f );

	/** Update the default encoding for the task */
	public void setDefaultEncoding( String encoding );
	
	public String getDefaultEncoding();

	/** Reset the current target file */
	public void setCurrentTargetFile( File f );
	
	/** Store a task result */
	public void setTaskResult( String result );
	
	/** Get a task result */
	public String getTaskResult();

	/** Store a task source */
	public void setTaskSource( String source );
	
	/** Get a task source */
	public String getTaskSource();
	
	/** @return the current source file */
	public File getCurrentSourceFile();
	
	/** @return the current target file */
	public File getCurrentTargetFile();

}
