package com.japisoft.xflows.task.concat;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import com.japisoft.framework.ui.text.FileTextField;
import com.japisoft.xflows.task.AbstractTaskUI;
import com.japisoft.xflows.task.FilesTaskRunner;
import com.japisoft.xflows.task.ui.XFlowsFactory;
import com.japisoft.xflows.task.xslt.XSLTParametersDialog;


/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ConcatUI extends AbstractTaskUI {

	public static final String SOURCEPATH = FilesTaskRunner.SOURCEPATH;
	public static final String SOURCEFILTER = FilesTaskRunner.SOURCEFILTER;
	public static final String TARGETPATH = FilesTaskRunner.TARGETPATH;
	public static final String ROOTTAG = "roottag";

	public ConcatUI() {
		super( "concat" );
		try {
			jbInit();

			tfSourcePath.setName(SOURCEPATH);
			cbSourceFilter.setName(SOURCEFILTER);
			tfTargetPath.setName(TARGETPATH);
			tfRootTag.setName( ROOTTAG );

			tfSourcePath.setDirectoryMode( true );
			tfTargetPath.setDirectoryMode( true );
			tfTargetPath.setOpenedMode( false );
			
			tfSourcePath.setPreferenceBinding( "xflows", "defaultSourcePath" );
			tfTargetPath.setPreferenceBinding( "xflows", "defaultTargetPath" );

		} catch (Exception ex) {
			ex.printStackTrace();
		}		
	}

	TitledBorder titledBorder1;
	JLabel lblSource = new JLabel();
	FileTextField tfSourcePath = new FileTextField(null, "xml");
	JLabel lblSourceFilter = new JLabel();
	JComboBox cbSourceFilter = XFlowsFactory.getSourceFilter("xml");
	JLabel lblParams = new JLabel();
	JLabel lblTarget = new JLabel();
	JLabel lblRootTag = new JLabel( "Root tag" );
	FileTextField tfTargetPath = new FileTextField(null, ( String )null);
	JTextField tfRootTag = new JTextField();
	GridBagLayout gridBagLayout1 = new GridBagLayout();

	public void actionPerformed(ActionEvent e) {
		XSLTParametersDialog dialog = new XSLTParametersDialog(params);
		dialog.setVisible(true);
		dialog.dispose();
	}

	void jbInit() throws Exception {
		titledBorder1 = new TitledBorder(BorderFactory.createLineBorder(
				new Color(153, 153, 153), 2), "Concat XML");
		lblSource.setText("XML Source path");
		this.setBorder(titledBorder1);
		this.setLayout(gridBagLayout1);
		tfSourcePath.setText("");
		lblSourceFilter.setText("XML Source filter");
		cbSourceFilter.setEditable(true);
		lblParams.setText("Parameters");
		lblTarget.setToolTipText("");
		lblTarget.setText("XML Target file");
		tfTargetPath.setText("");
		this.add(lblSource, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2,
						17, 0, 0), 0, 0));
		this.add(tfSourcePath, new GridBagConstraints(1, 0, 1, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(2, 9, 0, 14), 434, 4));
		this.add(lblSourceFilter, new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						20, 17, 0, 0), 0, 0));
		this.add(cbSourceFilter, new GridBagConstraints(1, 1, 1, 1, 1.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
				new Insets(16, 9, 0, 14), 406, -1));

		this.add(lblTarget, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						18, 17, 0, 31), 0, 0));
		this.add(tfTargetPath, new GridBagConstraints(1, 2, 1, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(14, 9, 0, 14), 434, 4));
		
		this.add(lblRootTag, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
						18, 17, 0, 31), 0, 0));

		this.add(tfRootTag, new GridBagConstraints(1, 3, 1, 1, 1.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(14, 9, 0, 14), 434, 4));		
		
	}	

}
