package com.japisoft.xflows.task.exp.json;

import java.io.File;
import java.io.FileInputStream;

import org.json.JSONObject;
import org.json.XML;

import com.japisoft.framework.toolkit.FileToolkit;
import com.japisoft.framework.xml.XMLFileData;
import com.japisoft.framework.xml.XMLToolkit;
import com.japisoft.xflows.task.TaskContext;
import com.japisoft.xflows.task.TaskRunner;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JSonFileRunner implements TaskRunner  {

	public boolean run( TaskContext context ) {
		context.addInfo( 
			"Exporting " + 
				context.getCurrentSourceFile() );

		File source = context.getCurrentSourceFile();
		File target = context.getCurrentTargetFile();

		try {
			String xmlData = context.getTaskSource();
			
			if ( xmlData == null ) {				
				XMLFileData xfd = XMLToolkit.getContentFromInputStream( 
					new FileInputStream( source ), 
					null 
				);
				xmlData = xfd.getContent();
			}
			
			JSONObject json = XML.toJSONObject( xmlData );
			String result = json.toString();
			FileToolkit.writeFile( 
				target, 
				result, 
				context.getDefaultEncoding() 
			);

		} catch( Throwable exc ) {
			context.addError(
				exc.getMessage() 
			);
			return TaskRunner.ERROR;
		}

		return TaskRunner.OK;
	}	

}
