package com.japisoft.xflows.task.jsx;

import java.io.File;

import javax.script.Bindings;
import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptException;

import com.japisoft.editix.editor.jsx.action.NodeFactory;
import com.japisoft.editix.editor.jsx.domapi.Document;
import com.japisoft.editix.javascript.JavaScriptFactory;
import com.japisoft.framework.toolkit.FileToolkit;
import com.japisoft.xflows.task.TaskContext;
import com.japisoft.xflows.task.TaskRunner;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class JSXTFileRunner implements TaskRunner {

	private TaskContext context = null;

	public boolean run(TaskContext context) {
		this.context = context;
		try {
			String fstylesheet = context.getParam(JSXUI.JSX);
			context.addInfo("Updating " + context.getCurrentSourceFile());
			return applyTransformation( context, context
					.getCurrentSourceFile(), fstylesheet, context
					.getCurrentTargetFile());
		} finally {
			this.context = null;
		}
	}

	public static boolean applyTransformation(
			TaskContext context, File data, String jsx, File res) {

		ScriptEngine engine = JavaScriptFactory.newFactory().engine();
		Document doc = null;
		
		try {
			Bindings b = engine.getBindings(ScriptContext.GLOBAL_SCOPE);
			doc = NodeFactory.newInstance().getDocument( data.toString() );					
			b.put( "document", doc );
		} catch( Exception exc ) {
			context.addError( "Can't parse this XML document " + data + " ?" );
			return ERROR;
		}

		try {
			String jsxContent = FileToolkit.getContentFromFileName( new File( jsx ), "UTF-8" );
			engine.eval( jsxContent );
			doc.save();
		} catch( ScriptException exc ) {
			context.addError( "Invalid Script JSX : " + exc.getMessage() );
			return ERROR;
		} catch( Throwable th ) {
			context.addError( "Can't load the JSX file " + jsx + " ?" );
			return ERROR;
		}
		
		return OK;
	}

}
