package com.japisoft.xflows.task.xquery;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamResult;

import net.sf.saxon.Configuration;
import net.sf.saxon.query.DynamicQueryContext;
import net.sf.saxon.query.StaticQueryContext;
import net.sf.saxon.query.XQueryExpression;
import net.sf.saxon.trans.XPathException;

import com.japisoft.xflows.task.TaskContext;
import com.japisoft.xflows.task.TaskRunner;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XQueryFileRunner implements TaskRunner {

	private TaskContext context = null;

	public boolean run( TaskContext context ) {
		this.context = context;
		try {
			File fxquery = new File( context.getParam( XQueryUI.XQUERY ) );		
			context.addInfo( "Transforming " + context.getCurrentSourceFile() );
			return applyTransformation( context, context.getCurrentSourceFile(), fxquery, context.getCurrentTargetFile() );
		} finally {
			this.context = null;
		}
	}

	public static boolean applyTransformation( TaskContext context, File data, File xquery, File res ) {
		
		Configuration config = new Configuration();
		StaticQueryContext staticContext = 
		        new StaticQueryContext( config );

		try {
			XQueryExpression exp = 
			        staticContext.compileQuery( new FileReader( xquery ) );
			
			DynamicQueryContext dynamicContext = 
			    new DynamicQueryContext( config );
			dynamicContext.setContextNode(
					staticContext.buildDocument(
							new SAXSource(
									new org.xml.sax.InputSource( new FileReader( data ) ) )				
					) 
			);

			FileWriter buffer = new FileWriter( res );
			StreamResult result = new StreamResult( buffer );			
			Properties props = new Properties();

			if ( "xml".equals( context.getParam( XQueryUI.OUTPUT ) ) ) {
				props.setProperty(OutputKeys.METHOD, "xml");
				props.setProperty(OutputKeys.INDENT, "yes");
			} else {
				props.setProperty(OutputKeys.METHOD, "text");
			}

			exp.run( 
					dynamicContext, 
					result, 
					props );
			
		} catch ( XPathException e ) {
			context.addError( e.getMessageAndLocation() );
			return ERROR;
		} catch ( FileNotFoundException e ) {
			context.addError( e.getMessage() );
			return ERROR;
		} catch ( IOException e ) {
			context.addError( e.getMessage() );
			return ERROR;
		}

		return OK;
	}

}
