package com.japisoft.xflows.task.xslt;

import java.awt.Dimension;
import java.util.Iterator;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import com.japisoft.framework.ApplicationModel;
import com.japisoft.framework.dialog.BasicOKCancelDialogComponent;
import com.japisoft.framework.preferences.Preferences;
import com.japisoft.xflows.task.TaskParams;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XSLTParametersDialog extends BasicOKCancelDialogComponent {

	private TaskParams params;
	private JTable tb;

	public XSLTParametersDialog( TaskParams params ) {
		super(
			ApplicationModel.MAIN_FRAME,
			"XSLT Option",
			"XSLT Parameters",
			"Choose a set of name and value for using it inside your stylesheet with the <param> elements", 
			null );

		this.params = params;
		tb = new JTable();
		DefaultTableModel model = new DefaultTableModel(
				new String[] { "Name", "Value" },
				0 );
		tb.setModel( model );
		JScrollPane sp = null;
		setUI( sp = new JScrollPane( tb ) );
		sp.setPreferredSize( new Dimension( 400, 500 ) );
	}

	protected void beforeClosing() {
		super.beforeClosing();
		TableModel model = tb.getModel();
		for ( int i = 0; i < model.getRowCount(); i++ ) {
			String name = ( String )model.getValueAt( i, 0 );
			String value = ( String )model.getValueAt( i, 1 );
			if ( "".equals( value ) )
				value = null;
			params.setParam( "param_" + name, value, TaskParams.XSLTPARAMS );
		}
	}
	
	protected void beforeShowing() {
		super.beforeShowing();
		for ( Iterator it = params.getParams(); it.hasNext(); ) {
			String name = ( String )it.next();
			if ( params.getParamType( name ) == TaskParams.XSLTPARAMS ) {
				if ( params.hasParamValue( name ) ) {
					( ( DefaultTableModel )tb.getModel() ).addRow(
							new Object[] { 
									name.substring( 6 ), 
									params.getParamValue( name ) } );
				}
			}
		}
		for ( int i = tb.getModel().getRowCount(); 
					i < Preferences.getPreference( "xslt", "parameter", 20 ); 
						i++ ) {
			((DefaultTableModel)tb.getModel()).addRow(
					new Object[] { "", "" } );
		}
	}

}
