package com.japisoft.xmlform.component;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.border.Border;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.japisoft.xmlform.component.container.GridComponent;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLSerializer {

	public static Element serialize( 
			Document doc,
			Element parent,
			AbstractXMLFormComponent component ) {

		component.resolveNextSibling();

		Element field = doc.createElementNS(
				null,
				"field" );
		field.setAttribute( 
				"class", 
				component.getClass().getName() );
		Element p = doc.createElementNS(
				null,
				"properties" );
		field.appendChild( p );
		if ( parent != null )
			parent.appendChild( field );
		
		HashMap<String, PropertyChangeEvent> mapProperties = 
			component.getModifiedProperties();
		if ( mapProperties != null ) {
			Iterator<String> it = 
				mapProperties.keySet().iterator();
			while ( it.hasNext() ) {
				String key = it.next();
				PropertyChangeEvent e = mapProperties.get( key );
				Element property = 
					serialize( doc, e );
				if ( property != null )
					p.appendChild( property );
			}
		}

		// Add children
		for ( int i = 0; i < component.getComponentCount(); i++ ) {

			Component c = component.getComponent( i );
			
			// Process the grid
			if ( c instanceof GridComponent ) {
				
				GridComponent gc = ( GridComponent )c;
				for  ( int j = 0; j < gc.getComponentCount(); j++ ) {
					Component c2 = gc.getComponent( j );
					if ( c2 instanceof AbstractXMLFormComponent ) {
						// Add it
						field.appendChild(
								serialize( 
										doc, 
										field, 
										( AbstractXMLFormComponent )c2 ) );

					}
				}

			}

			// Process the other component
			if ( c instanceof AbstractXMLFormComponent ) {
				// Add it
				field.appendChild(
						serialize( 
								doc, 
								field, 
								( AbstractXMLFormComponent )c ) );
			}

		}

		return field;
	}

	private static Element serialize(
			Document doc,
			PropertyChangeEvent e ) {

		Object val = e.getNewValue();
		if ( val == null )
			return null;
		if ( val instanceof Border ) {
			return null;
		}

		Element field = doc.createElementNS( null, "property" );
		field.setAttribute( "name", e.getPropertyName() );
		field.setAttribute( "class", val.getClass().getName() );
		field.setAttribute( "value", valueToString( val ) );
		return field;

	}

	public static String valueToString( Object value ) {
		if ( value instanceof String ) {
			return value.toString();
		} else
		if ( value instanceof Rectangle ) {
			Rectangle r = ( Rectangle )value;
			return r.x + "," + r.y + "," + r.width + "," + r.height;
		} else
		if ( value instanceof Color ) {
			Color c = ( Color )value;
			return c.getRed() + "," + c.getGreen() + "," + c.getBlue();
		} else 
		if ( value instanceof Font ) {
			Font f = ( Font )value;
			return f.getFamily() + "," + f.getStyle() + "," + f.getSize();
		} else
		if ( value instanceof String[] ) {
			String[] ss = ( String[] )value;
			StringBuffer sb = new StringBuffer();
			for ( String s : ss ) {
				if ( sb.length() > 0 )
					sb.append( "~~" );
				sb.append( s );
			}
			return sb.toString();
		} else
		if ( value instanceof HashMap ) {
			StringBuffer sb = new StringBuffer();
			HashMap<String,String> map = ( HashMap<String, String> )value;
			for ( String key : map.keySet() ) {
				if ( sb.length() > 0 )
					sb.append( "~~" );				
				sb.append( key );
				sb.append( "~~" );
				sb.append( map.get( key ) );
			}
			return sb.toString();
		}

		return value.toString();
	}

}
