package com.japisoft.xmlform.component.editable;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import org.w3c.dom.Node;
import org.w3c.dom.Text;

import com.japisoft.framework.internationalization.Traductor;
import com.japisoft.framework.xml.grammar.GrammarNode;
import com.japisoft.framework.xml.grammar.GrammarText;
import com.japisoft.framework.xml.grammar.GrammarType;
import com.japisoft.xmlform.component.ComponentContext;
import com.japisoft.xmlform.component.container.GridComponent;
import com.japisoft.xmlform.designer.data.GrammarNodeTreeNode;
import com.japisoft.xmlform.designer.properties.PropertyDescriptor;
import com.japisoft.xmlform.designer.properties.PropertyDescriptorImpl;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLEnumComponent extends XMLEditableComponent 
		implements ActionListener {

	private JComboBox cb = null;
	
	public XMLEnumComponent( 
			boolean designMode, 
			ComponentContext context ) {	
		super( designMode, context );
		cb = new JComboBox();
		
		if ( !designMode ) // For forcing best height
			add( new JLabel( " " ), BorderLayout.NORTH );

		add( cb, BorderLayout.CENTER );

		setPreferredSize( new Dimension( 
				200, 
				4 * GridComponent.getGridSize() ) );		

		// checkEnumValues();
	}

	@Override
	public void setTooltip(String tooltip) {
		super.setTooltip(tooltip);
		cb.setToolTipText( tooltip );
	}
	
	private void checkEnumValues() {
	
		if ( designMode && cb != null ) {
			GrammarNode n = node.getSource();
			GrammarType t = n.getType();
			if ( t != null ) {
				List<GrammarNode> nodes = 
					t.getValues();
				ArrayList<String> vs = new ArrayList<String>();
				for ( GrammarNode gn : nodes ) {
					if ( gn instanceof GrammarText ) {
						GrammarText gt = ( GrammarText )gn;
						vs.add( gt.getValue() );
					}
				}
				String[] r = new String[ vs.size() ];
				for ( int i = 0; i < vs.size(); i++ )
					r[ i ] = 
						vs.get( i );
				setValues( r );
			}			
		}		

	}
	
	@Override
	public void setGrammarNode(GrammarNodeTreeNode node) {
		super.setGrammarNode(node);
		checkEnumValues();
	}
	
	@Override
	protected void setDOM( Node newDom ) {
		if ( newDom instanceof Text ) {
			newDom = newDom.getParentNode();
		}
		super.setDOM( newDom );
		domText = null;
		Node t = getDOMText();
		
		String value = t.getNodeValue();
		if ( dictionnary != null )
			value = dictionnary.get( value );
		
		cb.setSelectedItem( value );
	}

	public void addNotify() {
		super.addNotify();
		if ( designMode ) {
			cb.addMouseListener( this );
			cb.addMouseMotionListener( this );
		} else
			cb.addActionListener( this );
	}
	
	@Override
	public void removeNotify() {
		super.removeNotify();
		if ( designMode ) {
			cb.removeMouseListener( this );
			cb.removeMouseMotionListener( this );
		} else
			cb.removeActionListener( this );
	}

	private String[] values = null;
	
	public String[] getValues() {
		return values;
	}
	
	public void setValues( String[] values ) {
		firePropertyChange( "values", this.values, values );
		this.values = values;
		cb.removeAllItems();
		cb.addItem( Traductor.traduce( "nos", "No Selection" ) );
		for ( String v : values ) {
			
			if ( dictionnary != null )
				v = dictionnary.get( v );

			cb.addItem( v );
		}
	}

	private HashMap<String,String> dictionnary = null;
	
	public HashMap<String,String> getDictionnary() {
		if ( dictionnary == null ) {
			dictionnary = new HashMap<String, String>();
		}

		if ( values != null ) {
			for ( String value : values ) {
				if ( !dictionnary.containsKey( value ) ) {
					dictionnary.put( value, value );
				}
			}
		}
		return dictionnary;
	}
	
	public void setDictionnary( HashMap<String,String> dictionnary ) {
		firePropertyChange( "dictionnary", null, dictionnary );
		this.dictionnary = dictionnary;
		
		if ( values != null ) {
			//for ( int i = 0; i < values.length; i++ ) {
			//	values[ i ] = dictionnary.get( values[ i ] );
			//}
			
			DefaultComboBoxModel model = new DefaultComboBoxModel();			
			model.addElement( cb.getItemAt( 0 ) );

			for ( String value : values ) {

				if ( dictionnary.containsKey( value ) )
					model.addElement( 
							dictionnary.get( value ) );
				else
					model.addElement( value );

			}

			cb.setModel( model );
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		int index = cb.getSelectedIndex();
		if ( index == 0 )
			deleteDOMText();
		else {
			
			String value = ( String )cb.getSelectedItem();

			if ( dictionnary != null ) {
				// Translate it

				for ( Entry<String,String> entry : dictionnary.entrySet() ) {
					
					if ( value.equals( entry.getValue() ) ) {
						value = entry.getKey();
						break;
					}
					
				}
			}

			getDOMText().setNodeValue( value );
		}
	}

	protected void prepareProperties( 
			ArrayList<PropertyDescriptor> l ) throws Exception {
		super.prepareProperties( l );
		l.add( new PropertyDescriptorImpl( 
			"dictionnary", 
			HashMap.class, 
			this ) );
	}

}
