package com.japisoft.xmlform.designer.data;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;

import javax.swing.Icon;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.tree.TreeCellRenderer;

import com.japisoft.framework.app.toolkit.Toolkit;
import com.japisoft.framework.ui.FastLabel;
import com.japisoft.framework.xml.grammar.GrammarNode;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DataTreeRenderer implements TreeCellRenderer {

	private BoundImageIcon elementOpened = null;
	private BoundImageIcon elementClosed = null;	
	private BoundImageIcon text = null;	
	private FastLabel label = null;

	public DataTreeRenderer() {
		elementOpened = 
			new BoundImageIcon( Toolkit.getImageIcon( 
					"images/folder.png" ) );
		elementClosed = 
			new BoundImageIcon( Toolkit.getImageIcon( 
					"images/folder_closed.png" ) );		
		text = 
			new BoundImageIcon( Toolkit.getImageIcon( 
					"images/text.png" ) );
		label = 
			new FastLabel();
		label.setOpaque( true );
	}

	public Component getTreeCellRendererComponent(
			JTree tree, 
			Object value,
			boolean selected, 
			boolean expanded, 
			boolean leaf, 
			int row,
			boolean hasFocus ) {
		
		if ( value instanceof GrammarNodeTreeNode ) {
			GrammarNodeTreeNode node = ( GrammarNodeTreeNode )value;
			GrammarNode gn = node.getSource();
			
			elementOpened.setBounded( node.getUserObject() != null );
			elementClosed.setBounded( node.getUserObject() != null );
			text.setBounded( node.getUserObject() != null );
			
			if ( "complex".equals( gn.getType().getType() ) ) {
				if ( expanded )
					label.setIcon( elementOpened );
				else
					label.setIcon( elementClosed );
			} else
				label.setIcon( text );
			label.setText( node.toString() );
			
			if ( !selected ) {
				label.setBackground( tree.getBackground() );
				label.setForeground( tree.getForeground() );
			} else {
				label.setBackground( UIManager.getColor( "Tree.selectionBackground" ) );
				label.setForeground( UIManager.getColor( "Tree.selectionForeground" ) );
			}
			
		} else {
			label.setText( null );
		}

		return label;
	}

	class BoundImageIcon implements Icon {

		private Icon ref = null;
		
		BoundImageIcon( Icon ref ) {
			this.ref = ref;
		}
		
		public int getIconHeight() {
			return ref.getIconHeight();
		}

		public int getIconWidth() {
			return ref.getIconWidth();
		}

		boolean bounded = false;
		
		public void setBounded( boolean bounded ) {
			this.bounded = bounded;
		}
		
		public void paintIcon( Component c, Graphics g, int x, int y ) {
			ref.paintIcon( c, g, x, y );
			if ( bounded ) {
				g.setColor( Color.DARK_GRAY );
				g.fill3DRect( 2, 2, 4, 4, true );
			}
		}

	}

}
