package com.japisoft.xmlpad;

import com.japisoft.xmlpad.editor.*;
import com.japisoft.xmlpad.elementview.ElementView;
import com.japisoft.xmlpad.elementview.table.TableElementView;
import com.japisoft.xmlpad.error.DefaultErrorView;
import com.japisoft.xmlpad.error.ErrorView;
import com.japisoft.xmlpad.helper.ui.BasicTitledPanelHelper;
import com.japisoft.xmlpad.helper.ui.TitledPanelHelper;

import javax.swing.*;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class ComponentFactory {
	
	protected ComponentFactory() {
		super();
	}
	
	static ComponentFactory FACTORY;
	
	/** @return the current factory */
	public static ComponentFactory getFactory() {
		if ( FACTORY == null )
			FACTORY = new ComponentFactory();
		return FACTORY;
	}
	
	/** Override the default factory by a subclass */
	public static void setFactory( ComponentFactory factory ) {
		FACTORY = factory;
	}

	/** create a scrollpane for editor or tree */
	public JScrollPane getNewScrollPane() {
		return new JScrollPane();
	}

	/** create a panel for editor and tree */
	public JPanel getNewPanel() {
		return new JPanel();
	}

	/** create a vertical oriented splitpane */
	public JSplitPane getNewVerticalSplitPane() {
		JSplitPane sp = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
		// sp.setOneTouchExpandable( true );
		return sp;
	}

	/** create an horizontal oriented splitpane */
	public JSplitPane getNewHorizontalSplitPane() {
		JSplitPane sp = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
		// sp.setOneTouchExpandable( true );
		return sp;
	}

	/** create a default tee */
	public JTree getNewTree() {
		return new JTree();
	}

	/** create a default toolBar */
	public JToolBar getNewToolBar() {
		return new JToolBar();
	}
	
	/** create a default toolBar for the tree */
	public JToolBar getNewTreeToolBar() {
		return new JToolBar();
	}

	/** create a default XMLEditor */
	public XMLEditor getNewXMLEditor( EditorContext context ) {
		XMLEditor e = new XMLEditor( context );
		return e;
	}

	/** create a default popupMenu */
	public JPopupMenu getNewPopupMenu() {
		return new JPopupMenu();
	}

	/** @return a new table element view */
	public ElementView getNewElementView( XMLContainer container ) {
		return new TableElementView( container );
	}
	
	/** @return a component for showing the helper's title */
	public TitledPanelHelper getNewTitledPanelHelper() {
		return new BasicTitledPanelHelper();
	}

	/** @return a component for showing the helper's title */
	public TitledPanelHelper getNewFooterPanelHelper() {
		return null;
	}

	/** @return a view for this container */
	public IView getUIContainer( XMLContainer container ) {
		return new DefaultView( container );
	}

	/** @return the default error view used by the XML container */
	public ErrorView getDefaultErrorView() {
		return new DefaultErrorView();
	}
}

// ComponentFactory ends here
