package com.japisoft.xmlpad;

import java.net.URL;

import com.japisoft.framework.xml.SchemaLocator;


/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public interface SchemaAccessibility {

	/** @return the current DTD location */
	public String getCurrentDTD();
	
	/** @return the current DTD root */	
	public String getCurrentDTDRoot();

	/** Reset the default document DTD location
	 * @param dtdRoot The DTD Root element
	 * @param dtd The DTD path or URL */
	public void setDefaultDTD(String dtdRoot, String dtd);

	/** Reset the default document DTD location
	 * @param dtdRoot The DTD Root element
	 * @param dtd The DTD inputStream content
	 */	
	public void setDefaultDTD( String dtdRoot, URL dtd );

	/** Reset the current dtd root and dtd location by this one. This is done automatically for 
	 * a new document and for parsing action. If you want default dtd, call rather the
	 * <code>setDefaultDTD</code> method */
	public void setDTD(String dtdRoot, String dtdLocation, int dtdDeclarationLine );

	/** Reset a local DTD. The SchemaLocator object will contain a path or a stream towards a full
	 * DTD. This is used for sample for a local DTD declaration. Don't use this method if
	 * your know the DTD location or URL.
	 * @param dtdRoot the dtd root element
	 * @param locator  The dtd content
	 * @exception Exception if a problem occurs when using this locator
	 * */
	public void setDTD( String dtdRoot, SchemaLocator locator ) throws Exception;

	/** @return the current schema location */
	public String getCurrentSchema();

	/** @return the current schema root */
	public String getCurrentSchemaRoot();

	/** Reset the current schema location 
	 * @param root Schema root tag
	 * @param namespace list of namespace bound to the schema location
	 * @param schemaLocation Schema URL or current document relative location 
	 * @param schemaDeclarationLine The location of the declaration inside the Document, you can ignore this value and put -1
	 * */
	public void setSchema(String schemaRoot, String[] namespace, String[] schemaLocation, int schemaDeclarationLine );

	/** @return the current DTD root element */
	public String getDefaultDTDRoot();
	
	/** @return the current DTDLocation built with the current document location */
	public String getDefaultDTDLocation();
	
	/** 
	 * @param resolve if <code>true</code> the location is built using the current document location
	 * @return the current DTD location. This DTD location can take into account relative document location 
	 * */
	public String getDTDLocation(boolean resolve);
	
	/** @return the default schema file/url location */
	public String getDefaultSchemaLocation();
	
	/** @return the default schema root tag */
	public String getDefaultSchemaRoot();
	
	/** Reset the default schema root tag and document location (url or file path) */
	public void setDefaultSchema(String root, String location);
	
	/** Reset the current RelaxNG document path for validating the current document. At this moment, 
	 * there's no way to embedded th RelaxNG usage inside the XML document like for the DTD or the
	 * schema, so you Must specify the relaxNG location each time you need it for parsing. Note that to 
	 * remove this option, you have to call it with a <code>null</code> value */
	public void setRelaxNGValidationLocation(String location);

	/** Reset the current RelaxNG document path for validating the current document */	
	public void setRelaxNGValidationLocation( SchemaLocator locator );

	/** @return the current RelaxNG documant location */
	public SchemaLocator getRelaxNGValidationLocation();
	
	/** 
	 * @param resolve if <code>true</code> the location is built using the current document location
	 * @return the current Schema location. This schema location can take into account relative document location
	 */
	public String getSchemaLocation(boolean resolve) ;

	/** This is only for inner usage, it mustn't be called by the user */
	void dispose();

}
