package com.japisoft.xmlpad;

import java.awt.Color;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import org.xml.sax.EntityResolver;

import com.japisoft.framework.app.toolkit.Toolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SharedProperties {

	/** Line number color for the selected node */
	public static Color LINE_NUMBER_COLOR_SELECTED = Color.DARK_GRAY;

	/** Line number color for the non selected part */
	public static Color LINE_NUMBER_COLOR = new Color(150, 150, 150);
		
	/** This is the maximum visible characters in the tree for text and attribute value*/
	public static int VISIBLE_TREENODE_TEXTE = 15;

	/** Use the resource bundle for finding the localized message. 
	 * Disable it will improve the loading performance. By default <code>true</code> */
	public static boolean LOCALIZED_MESSAGE = true;

	/** Delay before an popup helper is shown in ms */
	public static int HELPER_DELAY = 400;
	
	/** This is a property for activating a line wrapping. By default <code>false</code>. It must be called before using the XMLContainer class */
	public static boolean WRAPPED_LINE = false;

	/** This is a property for switching to a full text view. It must be called before using the XMLContainer class */
	public static boolean FULL_TEXT_VIEW = true;
	
	/** This is a property for caching a SCHEMA inside the content assistant. By default <code>false</code> */
	public static boolean SCHEMA_CACHING = false;

	/** Entity resolver for various entities. Useful for XML catalogs */
	public static EntityResolver DEFAULT_ENTITY_RESOLVER = null;

	/** Property for activating/disactivacting tooltip for each tree node */
	public static boolean TOOLTIP_TREE = true;

	/** Property for defining the default directory when loading a file */
	public static String DEFAULT_LOAD_DIRECTORY = null;
	
	/** Highligh syntax errors inside the editor on-the-fly */
	public static boolean HIGHLIGHT_ERROR_ONTHEFLY = false;

	/** Maximal size for the entity name and value inside the helper */
	public static int MAX_ENTITY_VISIBLE_SIZE_HELPER = 10;
	
	/** Add an helper panel for each content assistant element */
	public static boolean HELPER_PANEL = true;

	private static ImageIcon miniErrorIcon = null;

	/** Display the line number beside the text */
	public static boolean EDITOR_LINE_NUMBER = true;
	
	/** @return a minimal icon superposed in the tree or column bar for each error */
	public static ImageIcon getBugLittleIcon() {
		if ( miniErrorIcon == null )
			miniErrorIcon = (ImageIcon)Toolkit.getImageIcon( "images/little_bug_red.png" );
		return miniErrorIcon;
	}

	static Icon SYSTEM_ICON;
	static Icon ENTITY_ICON;

	/** Icon for content assistant */
	public static Icon getDefaultSystemHelperIcon() {
		if ( SYSTEM_ICON == null )
			SYSTEM_ICON = Toolkit.getImageIcon( "images/elements2.png" );
		return SYSTEM_ICON;
	}	

	/** Icon for content assistant */	
	public static Icon getDefaultEntityHelperIcon() {
		if ( ENTITY_ICON == null )
			ENTITY_ICON = Toolkit.getImageIcon( "images/element3.png" );
		return ENTITY_ICON;
	}	
	
}
