package com.japisoft.xmlpad.action.file;

import java.io.*;
import java.net.URL;
import javax.swing.*;

import com.japisoft.xmlpad.Debug;
import com.japisoft.xmlpad.SharedProperties;
import com.japisoft.xmlpad.UIFactory;
import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.XMLAction;
import com.japisoft.xmlpad.toolkit.XMLFileData;
import com.japisoft.xmlpad.toolkit.XMLToolkit;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class LoadAction extends XMLAction {
	
	public static final String ID = LoadAction.class.getName();
	
	public LoadAction() {
		super();
	}

	static XMLFileData getContentFromFileName(String fileName) throws Throwable {
		InputStream input = null;
		if (fileName.indexOf("://") > -1) {
			// URL case
			URL url = new URL(fileName);
			input = url.openStream();
		} else {
			input = new FileInputStream( fileName );
		}
		return XMLToolkit.getContentFromInputStream( input, null );
	}

	/** Reset the container with this fileName content 
	 * @param fileName a file system path or URL */
	public static void loadInBuffer(XMLContainer container, String fileName)
		throws Throwable {
		XMLFileData content = getContentFromFileName(fileName);
		container.setCurrentDocumentLocation(fileName);
		container.setText(content.getContent());
		Debug.debug( "Read with " + content.getEncoding() );
		container.getDocumentInfo().setEncoding( content.getEncoding() );
	}

	public boolean notifyAction() {
		JFileChooser chooser = UIFactory.getInstance().getOpenFileChooser();
		chooser.setFileFilter(container.getDocumentInfo().getFileFilter());
		if (container.getDocumentInfo().getWorkingDirectory() != null)
			chooser.setCurrentDirectory(
				new File(container.getDocumentInfo().getWorkingDirectory()));
		else {
			if ( SharedProperties.DEFAULT_LOAD_DIRECTORY != null ) {
				chooser.setCurrentDirectory(
						new File( SharedProperties.DEFAULT_LOAD_DIRECTORY ) );				
			}
		}

		int returnVal = chooser.showOpenDialog(editor);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			boolean rtSupport = editor.isEnabledRealTimeStructureChanged();
			editor.setEnabledRealTimeStructureChanged(false);
			String fileName = "" + chooser.getSelectedFile();
			try {
				loadInBuffer(container, fileName);
				//container.searchAndParseDTD();
			} catch (Throwable th) {
				JOptionPane.showMessageDialog(
					editor,
					th.getMessage(),
					"Error",
					JOptionPane.ERROR_MESSAGE);
				th.printStackTrace();
				return INVALID_ACTION;
			}
			editor.setEnabledRealTimeStructureChanged(rtSupport);
			editor.notifyStructureChanged();
		}
		return VALID_ACTION;
	}

}
