package com.japisoft.xmlpad.editor;

import java.util.Hashtable;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class XMLEntityResolver extends Hashtable {

    public static String LT_ENTITY = "lt";
    public static String GT_ENTITY = "gt";
    public static String QUOTE_ENTITY = "quote";
    public static String AMP_ENTITY = "amp";

    public XMLEntityResolver() {
	super();
	setEnableDefaultXMLEntities( true );
    }

    /** Supports for standard XML Entities : '<,>,",&' */
    public void setEnableDefaultXMLEntities( boolean enable ) {
	if ( enable ) {
	    storeEntityValue( '<', LT_ENTITY );
	    storeEntityValue( '>', GT_ENTITY );
	    storeEntityValue( '"', QUOTE_ENTITY );
	    storeEntityValue( '&', AMP_ENTITY );
	} else {
	    removeEntityValue( '<' );
	    removeEntityValue( '>' );
	    removeEntityValue( '"' );
	    removeEntityValue( '&' );
	}
    }

    /** Store the tied entity for the character value */
    public void storeEntityValue( char value, String entityName ) {
	put( new TemporaryCharacter( value ), entityName );
	storeEntity( entityName, value );
    }

    public void removeEntityValue( char value ) {
	TemporaryCharacter tc = new TemporaryCharacter( value );
	String entity = ( String )get( tc );
	if ( entity != null ) {
	    remove( tc );
	    removeEntity( entity );
	}
    }

    private TemporaryCharacter tc = new TemporaryCharacter();

    /** @return an entity for the char value */
    public String getEntityValue( char value ) {
	tc.setValue( value );
	return ( String )get( tc );
    }

    /** @return <code>true</code> if value is known for an entity */
    public boolean hasEntityValue( char value ) {
	return ( getEntityValue( value ) != null );
    }

    private Hashtable htEntityValue = null;

    /** Store an entity value */
    public void storeEntity( String name, char value ) {
	if ( htEntityValue == null )
	    htEntityValue = new Hashtable();
	htEntityValue.put( name, new TemporaryCharacter( value ) );
    }

    /** Remove an entity value */
    public void removeEntity( String name ) {
	if ( htEntityValue == null )
	    return;
	htEntityValue.remove( name );
    }

    /** @return an entity value for the entity name */
    public char getEntityResolution( String entityName ) throws UnknownEntity {
	if ( htEntityValue == null )
	    throw new UnknownEntity();
	TemporaryCharacter tc = ( TemporaryCharacter )htEntityValue.get( entityName );
	if ( tc == null )
	    throw new UnknownEntity();
	return tc.getValue();
    }

    ////////////////////////////////////////////////////////////////////////////////////

    // Better than character 
    public class TemporaryCharacter {
	private char value;

	public TemporaryCharacter( char value ) {
	    setValue( value );
	}

	public void setValue( char value ) {
	    this.value = value;
	}

	public char getValue() { 
	    return value;
	}

	public TemporaryCharacter() {
	    super();
	}

	public boolean equals( Object obj ) {
	    if ( obj instanceof TemporaryCharacter ) {
		return ( ( ( TemporaryCharacter )obj ).hashCode() == hashCode() );
	    } else
		return super.equals( obj );
	}

	public int hashCode() {
	    return ( int )value;
	}
    }
}

// XMLEntityResolver ends here
