package com.japisoft.xmlpad.helper.handler.schema.dtd;

import com.japisoft.dtdparser.node.DTDNode;
import com.japisoft.dtdparser.node.ElementDTDNode;
import com.japisoft.dtdparser.node.ElementRefDTDNode;
import com.japisoft.dtdparser.node.ElementSetDTDNode;
import com.japisoft.xmlpad.helper.model.SchemaNode;
import com.japisoft.xmlpad.helper.model.SchemaNodeProducer;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DTDToSchemaNode implements SchemaNodeProducer {

	public SchemaNode getSchemaNode( Object element ) {
		ElementDTDNode e = ( ElementDTDNode ) element;
		SchemaNode root = new SchemaNode( SchemaNode.ROOT );

		if ( !e.isEmptyElement() )
			buildSchema( e, root );

		return root;
	}
	
	private void buildSchema(
			ElementDTDNode elementParent,
			SchemaNode schemaParent ) {

		for ( int i = 0; i < elementParent.getDTDNodeCount(); i++ ) {

			DTDNode node = elementParent.getDTDNodeAt( i );

			if ( node.isElementSet() ) {
				ElementSetDTDNode set = ( ElementSetDTDNode ) node;
				addElementSet( set, schemaParent );
			}

		}
	}

	private void addElementSet( ElementSetDTDNode set, SchemaNode schemaParent ) {

		SchemaNode setNode = new SchemaNode(
				set.getType() == ElementSetDTDNode.CHOICE_TYPE ? 
						SchemaNode.OP_OR
						: SchemaNode.OP_AND );

		// Complete the setNode
		
		for ( int j = 0; j < set.getDTDNodeCount(); j++ ) {
			
			DTDNode node2 = set.getDTDNodeAt( j );

			if ( node2.isElementRef() ) {
				
				addElementRef( 
						( ElementRefDTDNode )node2,
						setNode 
				);
				
			} else
			if ( node2.isElementSet() ) {
				
				addElementSet( 
						( ElementSetDTDNode )node2, 
						setNode );

			}

		}

		// Manage the operator

		int op = set.getOperator();

		if ( op == ElementDTDNode.ONE_ITEM_OPERATOR ) {
			
			// Ok

			schemaParent.addNext( setNode );
						
		} else 
        if ( op == ElementDTDNode.ONE_MORE_ITEM_OPERATOR ) {

			// +
		
        	schemaParent.addNext( setNode );
    		setNode.addNext( setNode );

		} else
		if ( op == ElementDTDNode.ZERO_MORE_ITEM_OPERATOR ) {
			
			// *

			SchemaNode or = new SchemaNode( SchemaNode.OP_OR );
			or.addNext( setNode );			
			or.addNext( 
					new SchemaNode( SchemaNode.EMPTY ) );
			setNode.addNext( setNode );
			schemaParent.addNext( or );
			
		} else
		if ( op == ElementDTDNode.ZERO_ONE_ITEM_OPERATOR ) {

			// ?

			SchemaNode or = new SchemaNode( SchemaNode.OP_OR );
			or.addNext( setNode );
			or.addNext( new SchemaNode( SchemaNode.EMPTY ) );
			schemaParent.addNext( or );

		}
		
	}

	private void addElementRef( ElementRefDTDNode node, SchemaNode schemaParent ) {

		// Search the element

		ElementDTDNode target = node.getReferenceNode();

		if ( target == null )
			return;	// Error ?

		SchemaNode elementNode = new SchemaNode( 
				new DTDTagDescriptor( target ) );

		// Manage the operator

		int op = node.getOperator();

		if ( op == ElementDTDNode.ONE_ITEM_OPERATOR ) {
			
			// Ok
			
			schemaParent.addNext( elementNode );

		} else 
        if ( op == ElementDTDNode.ONE_MORE_ITEM_OPERATOR ) {

			// +

    		schemaParent.addNext( elementNode );
        	        	
        	elementNode.addNext( elementNode );

		} else
		if ( op == ElementDTDNode.ZERO_MORE_ITEM_OPERATOR ) {

			// *

			SchemaNode or = new SchemaNode( SchemaNode.OP_OR );
			or.addNext( new SchemaNode( SchemaNode.EMPTY ) );
			or.addNext( elementNode );
			or.addNext( or );
			schemaParent.addNext( or );

		} else
		if ( op == ElementDTDNode.ZERO_ONE_ITEM_OPERATOR ) {

			// ?

			SchemaNode or = new SchemaNode( SchemaNode.OP_OR );

			or.addNext( elementNode );
			or.addNext( new SchemaNode( SchemaNode.EMPTY ) );
						
			schemaParent.addNext( or );

		}

	}

}
