package com.japisoft.xmlpad.helper.handler.system;

import java.util.List;

import javax.swing.Icon;

import com.japisoft.framework.xml.parser.document.Document;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.SharedProperties;
import com.japisoft.xmlpad.editor.XMLPadDocument;
import com.japisoft.xmlpad.helper.model.AbstractDescriptor;
import com.japisoft.xmlpad.helper.model.TagDescriptor;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class DefaultSystemHandler extends AbstractSystemHandler {

	protected void installDescriptors( 
			FPNode currentNode,
			XMLPadDocument document, 
			int offset,
			String addedString ) {

		// Complete content from the current text sequence
		
		if ( document.isInProlog( offset )) {
			( ( AbstractDescriptor )addDescriptor( 
				new TagDescriptor( 
						addedString + "DOCTYPE tag SYSTEM \"\">", 
						null, 
						true, 
						true ) ) ).setComment( "DTD declaration part" );
		} else {
			
			if ( !document.hasSchema() ) {
				// All the available element

				if ( currentNode != null ) {
					Document doc = currentNode.getDocument();
					if ( doc != null ) {
						List<String> l = doc.getOrderedTags();
						for ( int i = 0; i < l.size(); i++ ) {
							addDescriptor( 
								new TagDescriptor(
									l.get( i ), 
									"<" + l.get( i ) + "></" + l.get( i ) + ">" ) 
							);
						}
					}
				}
			}

			( ( AbstractDescriptor )addDescriptor(
					new TagDescriptor( 
						addedString + "[CDATA[ ]]>", 
						null, 
						true, 
						true ) ) ).setComment( "Section CDATA for text" );			
		}
		
		( ( AbstractDescriptor )addDescriptor( 
				new TagDescriptor( 
						addedString + "-- -->", 
						null, 
						true, 
						true ) ) ).setComment( "XML comment" );		
	}

	protected String getActivatorSequence() {
		return "<!";
	}	

	public boolean haveDescriptors(
			FPNode currentNode,
			XMLPadDocument document, 
			boolean insertBefore, 
			int offset, 
			String activatorString ) {

		if ( activatorString != null && 
				!"!".equals( activatorString ) )
			return false;

		if ( document.isInsideTag( offset ) )
			return false;
		
		return match( 
				document, 
				offset, 
				activatorString, 
				"<!" );
	}

	protected Icon getDefaultIcon() {
		return SharedProperties.getDefaultSystemHelperIcon();
	}

}
