package com.japisoft.xmlpad.helper.model;

import javax.swing.DefaultListModel;

import com.japisoft.framework.job.Job;
import com.japisoft.framework.job.JobAdapter;
import com.japisoft.framework.job.JobManager;
import com.japisoft.framework.xml.parser.node.FPNode;
import com.japisoft.xmlpad.editor.XMLPadDocument;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class AttributeHelper extends AbstractHelper {

	private TagDescriptor helper = null;

	public AttributeHelper( TagDescriptor helper ) {
		this.helper = helper;
	}

	/** Wait for a tag descriptor */
	public void prepare( Object loc ) {
		helper = ( TagDescriptor ) loc;
	}

	public String getTitle() {
		return helper.getName();
	}

	protected boolean hasElements() {
		return helper != null && helper.getAtts() != null
				&& helper.getAtts().length > 0;
	}

	public AttDescriptor[] getAttributes( FPNode node ) {
		if ( helper == null )
			return null;

		AttDescriptor[] atts = helper.getAtts();
		if ( atts != null ) {
			// Disable known attributes

			if ( node != null ) {
				for ( int i = 0; i < atts.length; i++ ) {
					AttDescriptor _ = atts[i];
					String name = _.getName();
					if ( node.hasAttribute( name ) )
						_.setEnabled( false );
					else
						_.setEnabled( true );
				}
			}
		}
		return atts;
	}
	
	public AttDescriptor getAttribute( String name ) {
		AttDescriptor[] atts = helper.getAtts();
		if ( atts != null ) {
			for ( int i = 0; i < atts.length; i++ ) {
				if ( name.equals( atts[ i ].getName() ) )
					return atts[ i ];
			}
		}
		return null;
	}

	protected void insertResult( 
			XMLPadDocument doc, 
			int offset, 
			String result,
			Descriptor descriptor, String added ) {

		AttDescriptor att = ( AttDescriptor )descriptor;
		
		String name = att.getName();
		if ( added != null )
			name = name.substring( added.length() );
				
		doc.insertStringWithoutHelper( 
				offset, 
				name,
				null );

		offset += name.length();

		doc.insertStringWithoutHelper( offset, "=", null );
		offset++;
		int storedOffset = 0;
		if ( att.hasEnumValues() ) {
			storedOffset = offset;
		}
		if ( !att.hasEnumValues() )
			doc.insertStringWithoutHelper( offset, "\"" + att.getDefaultValue() + "\"", null );
		else
			doc.insertStringWithoutHelper( offset, "\"" + "\"", null );

		if ( storedOffset > 0 ) {
			final int ii = storedOffset;
			final XMLPadDocument doc2 = doc;
			Job newJob = new JobAdapter() {
				public void run() {
					doc2.manageCompletion( false, ii, "\"" );
				}
			};
			JobManager.addJob( newJob );
		}
	}
	
	protected void fillList( FPNode node, DefaultListModel model ) {
		AttDescriptor[] as = getAttributes(node);
		all: for (int i = 0; i < as.length; i++) {
			for (int j = 0; j < model.getSize(); j++) {
				AttDescriptor ad = ( AttDescriptor ) model.get(j);
				if (ad.getName() == null)
					continue all;
				if ( ad.getName().compareTo( as[ i ].getName() ) > 0 ) {
					model.insertElementAt( as[ i ], j );
					continue all;
				}
			}
			model.addElement( as[ i ] );
		}
	}

}
