package com.japisoft.xmlpad.helper.model;

import java.util.ArrayList;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class TagDescriptor extends AbstractDescriptor {
	public String name;
	private String nameHelper;
	private AttDescriptor[] atts;
	public boolean empty;
	private boolean raw;
	private boolean enabled = true;
	private int maxOcc = -1;
	private int minOcc = -1;
	private boolean choice = false;
	public String namespace = null;
	
	public TagDescriptor( 
			String name,
			boolean empty ) {
		this.name = name;
		this.empty = empty;
	}

	public TagDescriptor( 
			String name,
			String raw ) {
		this( name, false );
		this.raw = true;
		this.rawContent = raw;
	}
		
	public TagDescriptor( 
			String name, 
			AttDescriptor[] atts, 
			boolean empty ) {
		this.name = name;
		this.atts = atts;
		this.empty = empty;
	}
	
	public TagDescriptor( 
			String name, 
			AttDescriptor[] atts, 
			boolean empty, 
			boolean raw ) {
		this.name = name;
		this.atts = atts;
		this.empty = empty;
		this.raw = raw;
	}

	public TagDescriptor( 
			String name, 
			String nameHelper, 
			AttDescriptor[] atts, 
			boolean empty, 
			boolean raw ) {
		this.name = name;
		this.nameHelper = nameHelper;
		this.atts = atts;
		this.empty = empty;
		this.raw = raw;
	}

	public void setAttDescriptor( AttDescriptor[] atts ) {
		this.atts = atts;
	}
	
	public void setEmpty( boolean empty ) {
		this.empty = empty;
	}
	
	public void setMaxOcc( int occ ) {
		this.maxOcc = occ;
	}
	
	public int getMacOcc() {
		return maxOcc;
	}
	
	public void setMinOcc( int occ ) {
		this.minOcc = occ;
	}
	
	public int getMinOcc() {
		return minOcc;
	}

	public void setChoice( boolean choice ) {
		this.choice = choice;
	}
	
	public boolean isChoice() {
		return choice;
	}
	
	public String getName() { return name; }
	
	public void setName( String name ) { this.name = name; }
	public void setNameHelper( String nameHelper ) {
		this.nameHelper = nameHelper;
	}
	
	public String getNameForHelper() {
		if ( nameHelper == null ) {
			nameHelper = getName().replace( '', ' ' );
			if  ( nameHelper.length() > 40 ) {
				nameHelper = nameHelper.substring( 0, 40 ) + "...";
			}
		}
		return nameHelper;
	}

	public AttDescriptor[] getRequiredAtt() { return atts; }
	public AttDescriptor[] getAtts() { return atts; }

	public void addAttDescriptor( AttDescriptor ad ) {
		if ( atts == null )
			atts = new AttDescriptor[] { ad };
		else {
			AttDescriptor[] oldAtts = atts;
			atts = new AttDescriptor[ oldAtts.length + 1 ];
			System.arraycopy( oldAtts, 0, atts, 0, oldAtts.length );
			atts[ oldAtts.length ] = ad;
		}
	}

	public boolean isEmpty() { return empty; }
	public boolean isRaw() { return raw; }
	public boolean isEnabled() { return enabled; }

	public void setEnabled( boolean enabled ) {
		this.enabled = enabled;
		for  ( int i = 0; i < getSynonymousTagDescriptorCount(); i++ ) {
			getSynonymousTagDescriptor( i ).setEnabled( enabled );
		}
	}

	public String getStartingTag() {
		StringBuffer sb = new StringBuffer( "<" );
		sb.append( name );
		if ( atts != null ) {
			for ( int i = 0; i < atts.length; i++ ) {
				if ( atts[ i ].isRequired() ) {
					sb.append( " ");
					sb.append( atts[ i ].getName() );
					sb.append( "=\"" );
					sb.append( atts[ i ].getDefaultValue() );
					sb.append( "\"" );
				}
			}
		}
		sb.append( ">" );		
		return sb.toString();
	}
	
	public String getEndingTag() {
		StringBuffer sb = new StringBuffer( "</" ).append( name ).append( ">" );
		return sb.toString();
	}
	
	private String endAddedPart;
	
	public void setEndAddedPart( String endAddedPart ) {
		this.endAddedPart = endAddedPart;
	}

	private String rawContent = null;
	
	public void setRawContent( String rawContent ) {
		this.rawContent = rawContent;
	}
	
	public String toExternalForm() {

		if ( rawContent != null ) {
			return rawContent;
		}
		
		StringBuffer sb = new StringBuffer();
		if ( addedPart != null )
			sb.append( addedPart );
		sb.append( name );

		boolean cursorAdded = false;
		
		if ( !raw ) {
		
			if ( atts != null ) {
				for ( int i = 0; i < atts.length; i++ ) {
					if ( atts[ i ].isRequired() ) {
						sb.append( " ");
						sb.append( atts[ i ].getName() );
						sb.append( "=\"" );
						if ( !cursorAdded ) {
							cursorAdded = true;
							sb.append( "" );
						}
						sb.append( atts[ i ].getDefaultValue() );
						sb.append( "\"" );
					}
				}
			}
			if ( empty )
				sb.append( "/>");
			else {
				sb.append( ">" );
				if ( !cursorAdded ) {
					sb.append( "" );
				}
				sb.append( "</").append( endAddedPart == null ? "" : endAddedPart ).append( name ).append( ">" );
			}
		
		}
		
		return sb.toString();	
	}
	
	public String getBuiltTag() {				
		return "<" + toExternalForm();
	}

	public String toString() {
		return getNameForHelper();
	}

	public boolean equals( Object o ) {
		if ( o instanceof TagDescriptor ) {
			TagDescriptor td = ( TagDescriptor )o;
			return td.toString() != null
				&& td.toString().equals( toString() );
		} else
			return super.equals( o );
	}
	
	private ArrayList secondaryTagDescriptor = null;

	// For the substitutionGroup feature
	public void addSynonymousTagDescriptor( TagDescriptor td ) {
		if ( secondaryTagDescriptor == null )
			secondaryTagDescriptor = new ArrayList();
		secondaryTagDescriptor.add( td );
	}

	public int getSynonymousTagDescriptorCount() {
		if ( secondaryTagDescriptor == null )
			return 0;
		return secondaryTagDescriptor.size();
	}

	public TagDescriptor getSynonymousTagDescriptor( int i ) {
		if ( secondaryTagDescriptor != null )
			return ( TagDescriptor )secondaryTagDescriptor.get( i );
		return null;
	}

}
