package com.japisoft.xmlpad.toolkit;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.JFrame;

import com.japisoft.xmlpad.XMLContainer;
import com.japisoft.xmlpad.action.file.LoadAction;
/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class SingleDocumentEditor extends JFrame {
	private XMLContainer container;

	public final static boolean NEW_DOCUMENT = true;
	public final static boolean CURRENT_DOCUMENT = !NEW_DOCUMENT;

	private SingleDocumentEditor() {
		super( "Single editor" );
		getContentPane().add( ( container = new XMLContainer() ).getView() );
		setSize( 550, 400 );
		setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
	}

	/** @return the current XMLContainer */
	public XMLContainer getXMLContainer() { return container; }

	/** Edit the provided file by opening a new Frame with the XMLPad container.
	 * @param fileName The XML document to edit
	 * @param newOne true if the file can be created or false for an existing one
	 * @return The new SingleDocumentEditor
	 * @throws FileNotFoundException If the file cannot be found
	 * @throws IOException if the document cannot be loaded
	 */
	public static SingleDocumentEditor showEditor( String fileName, boolean newOne ) throws FileNotFoundException, IOException  {
		SingleDocumentEditor sde = new SingleDocumentEditor();
		sde.setTitle( fileName );
		if ( !newOne || new File( fileName ).exists() ) {
			try {
				LoadAction.loadInBuffer( sde.container, fileName );
			} catch( Throwable th ) {
				if ( th instanceof FileNotFoundException )
					throw (FileNotFoundException)th;
				else
					throw new IOException( th.getMessage() );
			}
		}
		else
			sde.container.getDocumentInfo().setCurrentDocumentLocation( fileName );
		sde.setVisible( true );
		return sde;
	}

	public static void main( String[] args ) throws Throwable {
		SingleDocumentEditor.showEditor( "/home/japisoft/phoneBook/project/toto.xml", false );
	}
}
