package com.japisoft.xmlpad.xml.validator;

import com.japisoft.xmlpad.XMLContainer;

import java.io.StringReader;

import javax.xml.parsers.SAXParser;
import com.sun.msv.verifier.ErrorInfo;
import com.sun.msv.verifier.ValidityViolation;
import org.iso_relax.verifier.*;
import org.xml.sax.*;
import org.xml.sax.helpers.XMLFilterImpl;

/**
EditiX Open source XML Editor
https://www.editix.com


Sources Edition :

Our software is available under both a commercial and a non-commercial license :

- If you are a non-commercial user, you are free to use the software for any non-commercial purpose under the terms of the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International Public License. 

- If you are a commercial user and you want to use the software for commercial purposes, you must purchase a commercial license. The details are inside the license-commercial.txt file.



Binaries Edition :

- You can purchase a license for binaries usage using the purchasing page at https://www.editix.com. Each binary version can be downloaded on https://www.editix.com.



Please contact us at https://www.editix.com for more information on purchasing a commercial license.


@author (c) 2010-2023 Alexandre Brillant



*/
public class RelaxNGValidator implements Validator {

	XMLContainer container;
	
	public int validate(XMLContainer container, boolean silentMode ) {
		this.container = container;
		try {
			return validateRelaxNG();
		} finally {
			container = null;
		}

	}
	
	private int validateRelaxNG() {

		try {
			
			VerifierFactory factory = new com.sun.msv.verifier.jarv.TheFactoryImpl();

			// compile a schema and gets a verifier.
			final Verifier verifier = factory.newVerifier( 
					new InputSource(
							container.getSchemaAccessibility().getRelaxNGValidationLocation().getReader() ) );

			// create a SAX parser			
			SAXParser parser = XMLPadSAXParserFactory.getNewSAXParser( false );
			XMLReader reader = parser.getXMLReader();

			Interceptor interceptor = new Interceptor();

			interceptor.setParent(reader);
			interceptor.setContentHandler(verifier.getVerifierHandler());

			// set an error handler that throws any error.
			verifier
					.setErrorHandler(com.sun.msv.verifier.util.ErrorHandlerImpl.theInstance);

			// parse the file.
			interceptor.parse( 
					new InputSource( 
							new StringReader( container	.getText() ) ) );

		} catch (Exception exc) {			
			if ( exc instanceof SAXParseException ) {
				SAXParseException spe = ( SAXParseException )exc;				
				container.getErrorManager().notifyError(
						null,
						true,
						spe.getSystemId(),
						spe.getLineNumber(),
						spe.getColumnNumber(),
						0,
						spe.getMessage(),
						false );
				error = true;
			} else
			if ( exc instanceof ValidityViolation ) {
				ValidityViolation vv = ( ValidityViolation )exc;
				container.getErrorManager().notifyError(
						null,
						true,
						vv.getSystemId(),
						vv.getLineNumber(),
						vv.getColumnNumber(),
						0,
						vv.getMessage(),
						false );
				
//				container.getEditor().notifyError( vv.getSystemId(), vv.getLineNumber(), vv.getMessage() );
				error = true;
			} else
			if ( exc instanceof VerifierConfigurationException ) {
				VerifierConfigurationException ve = ( VerifierConfigurationException )exc;
				container.getErrorManager().notifyError(
						null,
						true,
						null,
						0,
						0,
						0,
						ve.getMessage(),
						false );
								
//				container.getEditor().notifyError( null, 0, ve.getMessage() );
				error = true;
			}
		}

		return ( error ? Validator.ERROR : Validator.OK );
	}

	public boolean error = false;
	
	private class Interceptor extends XMLFilterImpl {

		private StringBuffer buffer = new StringBuffer();
		private Locator loc = null;

		public void setDocumentLocator(Locator l) {
			this.loc = l;
			super.setDocumentLocator( l );
		}

		public void startElement(String ns, String local, String qname,
				Attributes atts) throws SAXException {

			ErrorInfo ei = null;
			
			try {
				super.startElement(ns, local, qname, atts);
				// there is no error.
			} catch (ValidityViolation vv) {
				// there was an error.
				ei = vv.getErrorInfo();
				if ( !error ) {
					container.getErrorManager().notifyError(
							null,
							true,
							vv.getSystemId(),
							vv.getLineNumber(),
							vv.getColumnNumber(),
							0,
							vv.getMessage(),
							false );					
				}
				//	container.getEditor().notifyError( vv.getSystemId(), ( loc != null ) ? loc.getLineNumber() : 0, vv.getMessage() );
				error = true;
			}
		}

		public void endElement(String ns, String local, String qname)
				throws SAXException {

			ErrorInfo ei = null;

			try {
				super.endElement(ns, local, qname);
				// there is no error.
			} catch (ValidityViolation vv) {
				// there was an error.
				ei = vv.getErrorInfo();
				
				if ( !error ) {
					container.getErrorManager().notifyError(
							null,
							true,
							vv.getSystemId(),
							vv.getLineNumber(),
							vv.getColumnNumber(),
							0,
							vv.getMessage(),
							false );					
				}
				
//				if ( !error )
//					container.getEditor().notifyError( vv.getSystemId(), ( loc != null ) ? loc.getLineNumber() : 0, vv.getMessage() );
				error = true;
			}
		}

	}
}
